@@ -5,14 +5,13 @@
 # sortresources.py <path_to_seach_and_sort>
 # use sortresources.py -h to get command line option help
 
-from __future__ import print_function
-
 from xml import sax
 from xml.sax import handler
+import string
 import os.path
 import os
 import sys
-from io import StringIO
+import StringIO
 
 # parse a resource xml file and fill in data property data strucure
 def parseResource(filename, data):
@@ -30,7 +29,7 @@ def parseResource(filename, data):
 
         def startElement(self,name,attr):
             # just parse the <map> tag and fill data
-            if name in("Resource","Map"):
+            if name in("Resource","Map","Cockpit"):
                 # set type from tag name it is not already set
                 try: oldtype = self.data.type
                 except: self.data.type = name.lower()       
@@ -64,7 +63,7 @@ def parseResource(filename, data):
               path = self.path
 
               # extract data type: the base name of the DTD
-              self.data.type = str.split(sysid,"-")[0]
+              self.data.type = string.split(sysid,"-")[0]
 
               # look in current and parent directories for dtds
               while len(path) > 0:
@@ -76,7 +75,7 @@ def parseResource(filename, data):
 
               # fallback: return empty stream, result: no dtd checking is done.
               # Not horribly bad in this context
-              print("warning, could not find requested entity", sysid)
+              print "warning, could not find requested entity", sysid
               return StringIO.StringIO("")
 
     # parse: create parser...
@@ -93,6 +92,8 @@ def parseResource(filename, data):
     try: 
          if data.type == "map":
             data.type = "aamap"
+         if data.type == "cockpit":
+            data.type = "aacockpit"
     except: pass
 
 # take the parsed data and trandsform it into a filename
@@ -112,10 +113,10 @@ def getCanonicalPath(path):
 
 # scan for all XML files in the directory and rename them according to the rules
 def scanDir(sourceDir, destinationDir, function):
-    for dirname, dirs, files in os.walk(sourceDir):
-        for file in files:
+    for directory in (os.walk(sourceDir)):
+        for file in directory[2]:
             if len(file) > 4 and file[-4:] == ".xml":
-                path = os.path.join(dirname,file)
+                path = os.path.join(directory[0],file)
                 newPath = getCanonicalPath(path)
                 # call the passed function
                 function(path, os.path.join(destinationDir, newPath), newPath)
@@ -142,7 +143,7 @@ def Print(oldFile, newFile, canonicalPath ):
         return False
 
     if doPrint:
-        print("renaming", oldFile, "->", newFile)
+        print "renaming", oldFile, "->", newFile
 
 # move file oldFile to newFile, setting apache rewrite rules to keep the file
 # fetchable from its old position
@@ -164,8 +165,8 @@ def Redirect(oldFile, newFile, canonicalPath ):
             # open .htaccess file in the old directory for reading
             accessFile = open(htaccessPath)
             for line in accessFile:
-                splitLineLower = str.split(line.lower())
-                splitLine      = str.split(line)
+                splitLineLower = string.split(line.lower())
+                splitLine      = string.split(line)
                 if splitLineLower[0] == "rewriteengine":
                     # rewriting is already mentioned; we don't have to enable it laster.
                     needEngine = False
@@ -176,7 +177,7 @@ def Redirect(oldFile, newFile, canonicalPath ):
                         # nothing to do
                         return
                     else:
-                        print("Warning: There already is a different RewriteRule for", oldFile, "in place.")
+                        print "Warning: There already is a different RewriteRule for", oldFile, "in place."
         except: pass
 
         # open .htaccess file for appended writing
@@ -191,15 +192,15 @@ def Redirect(oldFile, newFile, canonicalPath ):
 
 # print usage message and exit.
 def Options( ret ):
-    print()
-    print("Usage: sortresources.py [OPTIONS] <source directory> <destination directory>")
-    print("       Omitting the destination directory makes it equal to the source.")
-    print("       Omitting both directories makes them the current working directory.")
-    print("Options: -r add apache rewrite rules")
-    print("         -n don't do anything, just print what would be done")
-    print("         -v print non-warning status messages")
-    print("         -h print this message")
-    print()
+    print
+    print "Usage: sortresources.py [OPTIONS] <source directory> <destination directory>"
+    print "       Omitting the destination directory makes it equal to the source."
+    print "       Omitting both directories makes them the current working directory."
+    print "Options: -r add apache rewrite rules"
+    print "         -n don't do anything, just print what would be done"
+    print "         -v print non-warning status messages"
+    print "         -h print this message"
+    print
     sys.exit( ret )
     
 if __name__ == "__main__":
@@ -213,7 +214,7 @@ if __name__ == "__main__":
     # parse arguments
     for arg in sys.argv[1:]:
         # parse options
-        if arg[0] == "-":
+        if len(arg)>0 and arg[0] == "-":
             if len(arg) > 1:
                 if arg[1] == "h":
                     Options( 0 )
@@ -228,7 +229,7 @@ if __name__ == "__main__":
                 if arg[1] == "v":
                     doPrint = True
                     continue
-                print("\nUnknown option:", arg)
+                print "\nUnknown option:", arg
                 Options(-1)
             else:
                 Options(-1)
@@ -238,7 +239,7 @@ if __name__ == "__main__":
                 sourceDirectory = arg
             destinationDirectory = arg
     
-    # print(sourceDirectory, destinationDirectory)
+    # print sourceDirectory, destinationDirectory
 
     # do the work
     scanDir(sourceDirectory, destinationDirectory, function)
