@@ -33,8 +33,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "tArray.h"
 #include "nNetwork.h"
 
-#include "tCallbackString.h"
-
 #include <iosfwd>
 #include <memory>
 
@@ -45,9 +43,6 @@ class tPath;
 
 typedef nServerInfo* (sn_ServerInfoCreator)();
 
-//! return the DNS name of this machine, if set
-tString const & sn_GetMyDNSName();
-
 //! Basic server information: everything you need to connect
 class nServerInfoBase
 {
@@ -83,7 +78,7 @@ protected:
 private:
     tString         connectionName_;            //!< the internet name of the server ("192.168.10.10", "atron.dyndns.org")
     unsigned int    port_;                      //!< the network port the server listens on
-    mutable std::unique_ptr< nAddress > address_; //!< the network address of the server
+    mutable std::auto_ptr< nAddress > address_; //!< the network address of the server
 public:
     inline tString const & GetConnectionName( void ) const;	                                 //!< Gets the internet name of the server ("192.168.10.10", "atron.dyndns.org")
     inline nServerInfoBase const & GetConnectionName( tString & connectionName ) const;	     //!< Gets the internet name of the server ("192.168.10.10", "atron.dyndns.org")
@@ -95,23 +90,11 @@ public:
 private:
     nAddress & AccessAddress( void ) const;	                       //!< Accesses the network address of the server
     nServerInfoBase & SetAddress( nAddress const & address );	   //!< Sets the network address of the server
-public:
+protected:
     inline nServerInfoBase & SetConnectionName( tString const & connectionName );	         //!< Sets the internet name of the server ("192.168.10.10", "atron.dyndns.org")
     inline nServerInfoBase & SetPort( unsigned int port );	                                 //!< Sets the network port the server listens on
 };
 
-//! server information, just to redirect and for other immediate applications
-class nServerInfoRedirect: public nServerInfoBase
-{
-public:
-    // construct a server directly with connection name and port
-    nServerInfoRedirect( tString const & connectionName, unsigned int port )
-    {
-        nServerInfoBase::SetConnectionName( connectionName );
-        nServerInfoBase::SetPort( port );
-    }
-};
-
 //! Full server information
 class nServerInfo: public tListItem<nServerInfo>, public nServerInfoBase
 {
@@ -137,7 +120,6 @@ protected:
     bool		login2_;		// flag indicating whether the second version of the logic can be tried
 
     int     timesNotAnswered; // number of times the server did not answer to information queries recently
-    bool    stillOnMasterServer; // flag indicating whether the server is still listed on the master
 
     // human information
     tString name;             // the human name of the server ("Z-Man's Armagetron Server");
@@ -145,7 +127,6 @@ protected:
     int	  maxUsers_;		// maximum number of users allowed
 
     tString userNames_;		// names of the connected users
-    tString userGlobalIDs_;		// IDs of the connected users
     tString userNamesOneLine_;// names of the connected users in one line
     tString options_;			// description of non-default options
     tString url_;				// url asociated with the server
@@ -185,38 +166,6 @@ public:
     virtual void Save(std::ostream &s) const;
     virtual void Load(std::istream &s);
 
-
-    struct SettingsDigest
-    {
-        enum Flags
-        {
-            Flags_AuthenticationRequired = 0x1, //!< is authentication required to play on this server?
-            Flags_NondefaultMap = 0x2, //!< custom map, indicating complicated gameplay
-            Flags_TeamPlay = 0x4, //!< team play is possible
-            Flags_SettingsDigestSent = 0x8000, //!< Did this server send a settings digest?
-            Flags_All = 0xffff
-        };
-
-        unsigned short flags_;      //!< flags
-
-        void SetFlag( Flags flag, bool set ); //!< sets a certain flag to a value
-        bool GetFlag( Flags flag ) const; //!< returns the value of a certain flag
-
-        // play time requirements, filled by ePlayer.cpp
-        int minPlayTimeTotal_;    //!< minimum number of minutes spent playing up to now
-        int minPlayTimeOnline_;   //!< minimum number of minutes spent playing online
-        int minPlayTimeTeam_;     //!< minimum number of minutes spent playing team games
-
-        // filled by gCycleMovement.cpp
-        REAL cycleDelay_;         //!< cycle delay (max .05s if doublebinding has been disabled)
-        REAL acceleration_;       //!< acceleration strength (relative to base speed)
-        REAL rubberWallHump_;     //!< characteristic rubber number: rubber/(base speed*cycle_delay), the number of times you can hump a wall without suiciding
-        REAL rubberHitWallRatio_;  //!< characteristic rubber number: maximum ratio of time spent sitting on walls to total time
-        REAL wallsLength_;         //!< length of walls (divided by speed, so it's in seconds)
-
-        SettingsDigest();
-    };
-
     // sort key selection
     enum PrimaryKey
     {
@@ -274,9 +223,9 @@ public:
     static nServerInfo* GetMasters();              //!< get the list of master servers
     static nServerInfo* GetRandomMaster();         //!< gets a random master server
 
-    static void GetFromMaster(nServerInfoBase *masterInfo=NULL, char const * fileSuffix = NULL );  // get all the basic infos from the master server, stored in the server info file of the given suffix
+    static void GetFromMaster(nServerInfo *masterInfo=NULL);  // get all the basic infos from the master server
 
-    static void TellMasterAboutMe(nServerInfoBase *masterInfo=NULL);  // dedicated server: tell master server about my existence
+    static void TellMasterAboutMe(nServerInfo *masterInfo=NULL);  // dedicated server: tell master server about my existence
 
     static void GetFromLAN(unsigned int pollBeginPort=4534, unsigned int pollEndPort=4544);                            // get all the basic infos from a LAN broadcast
 
@@ -292,6 +241,8 @@ public:
         QUERY_NONE=3    //!< query only manually
     };
 
+    static void DeleteUnreachable();     //!< Removes unreachable servers from the list
+
     static void StartQueryAll( QueryType query = QUERY_ALL );     // start querying the advanced info of each of the servers in our list
 
     static bool DoQueryAll(int simultaneous=10);         // continue querying the advanced info of each of the servers in our list; return value: do we need to go on with this?
@@ -348,36 +299,8 @@ protected:
 
 private:
     QueryType queryType_; //!< the query type to use for this server
-    SettingsDigest settings_; //!< most important settings
 };
 
-//! callback to give other components a chance to help fill in the server info
-class nCallbackFillServerInfo: public tCallback
-{
-    static nServerInfo::SettingsDigest * settings_;
-public:
-    nCallbackFillServerInfo( VOIDFUNC * f );
-
-    //! the server settings to fill
-    static nServerInfo::SettingsDigest *ToFill(){ return settings_; }
-
-    //! fills all server infos
-    static void Fill( nServerInfo::SettingsDigest * settings );
-};
-
-//! callback to give other components a chance to help fill in the server info
-class nCallbackCanPlayOnServer: public tCallbackString
-{
-    static nServerInfo::SettingsDigest const * settings_;
-public:
-    nCallbackCanPlayOnServer( STRINGRETFUNC * f );
-
-    //! the settings
-    static nServerInfo::SettingsDigest const * Settings(){ return settings_; }
-
-    //! return all reasons why you can't play here
-    static tString CantPlayReasons( nServerInfo::SettingsDigest const * settings );
-};
 
 class nServerInfoAdmin
 {
@@ -391,7 +314,6 @@ protected:
 
 private:
     virtual tString GetUsers()		const = 0;
-    virtual tString GetGlobalIDs()	const = 0;
     virtual tString	GetOptions()	const = 0;
     virtual tString GetUrl()		const = 0;
 
