@@ -20,7 +20,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
 ***************************************************************************
 
 */
@@ -31,14 +31,15 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 // Copyright (C) 1996-1997 Id Software, Inc.
 // Modified for Armagetron by Manuel Moos (manuel@moosnet.de)
 
+#include "nSocket.h"
+
 #include "config.h"
 #include "tRandom.h"
-#include "tSysTime.h"
+
 #include "tRandom.h"
 
 #include <string>
 #include <stdio.h>
-#include <string.h>
 
 #include <sys/types.h>
 
@@ -47,10 +48,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <vector>
 
 #ifndef WIN32
-#include <arpa/inet.h> 
-#include <netinet/in_systm.h>
-#include <netinet/in.h>
-#include <netinet/ip.h>
+#include <arpa/inet.h>
 #include <netdb.h>
 #include <sys/param.h>
 #include <sys/ioctl.h>
@@ -84,7 +82,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define ESHUTDOWN WSAESHUTDOWN
 #define EMSGSIZE WSAEMSGSIZE
 #define ETIMEDOUT WSAETIMEDOUT
-#define ENETUNREACH WSAENETUNREACH 
+#define ENETUNREACH WSAENETUNREACH
 #define close closesocket
 #define snprintf _snprintf
 #endif
@@ -103,10 +101,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 
 #if HAVE_UNISTD_H
-#include <unistd.h>
+// #include <unistd.h>
 #endif
 
-#include "nSocket.h"
 #include "tConfiguration.h"
 #include "tRecorder.h"
 
@@ -654,7 +651,7 @@ public:
 bool ANET_ListenOn( char * hostname, int net_hostport,  nSocketListener::SocketArray & sockets, bool & onesocketonly, bool & relaxed, bool & ignoreErrors )
 {
     // skip whitespace
-    while ( isblank( *hostname ) )
+    while ( isspace( *hostname ) )
         hostname ++;
 
     // quit silently on empty hostname
@@ -693,10 +690,10 @@ bool ANET_Listen (bool state, int net_hostport, const tString & net_hostip, nSoc
 
         // first try: generate sockets for explicitly requested IP
         tString ips = net_hostip;   // copy of the hostname string
-        int lastpos = 0;            // position where the current IP starts
-        for ( int pos = 0; pos < ips.Len(); ++pos )
+        size_t lastpos = 0;            // position where the current IP starts
+        for ( size_t pos = 0; pos < ips.Size(); ++pos )
         {
-            if (isblank(ips[pos]))
+            if (isspace(ips[pos]))
             {
                 ips[pos] = '\0';
                 if ( pos > lastpos )
@@ -707,7 +704,7 @@ bool ANET_Listen (bool state, int net_hostport, const tString & net_hostip, nSoc
             }
         }
         // don't forget the last IP entry
-        if ( ips.Len() > lastpos )
+        if ( ips.Size() > lastpos )
             if ( !ANET_ListenOn( &ips[lastpos], net_hostport, sockets, onesocketonly, relaxed, ignoreErrors ) )
                 return false;
 
@@ -833,7 +830,7 @@ int ANET_GetSocketAddr (int sock, struct sockaddr *addr)
 }
 
 //=============================================================================
-/*
+
 int ANET_AddrCompare (struct sockaddr *addr1, struct sockaddr *addr2)
 {
     if (addr1->sa_family != addr2->sa_family)
@@ -847,8 +844,7 @@ int ANET_AddrCompare (struct sockaddr *addr1, struct sockaddr *addr2)
 
     return 0;
 }
-*/
-    
+
 }   // namespace
 
 //=============================================================================
@@ -871,14 +867,14 @@ nSocketListener const & nBasicNetworkSystem::GetListener( void ) const
 
 /*
 // *******************************************************************************************
-// *	
+// *
 // *	GetListener
-// *	
+// *
 // *******************************************************************************************
-//!		
+//!
 //!		@param	listener	listening sockets to fill
 //!		@return		A reference to this to allow chaining
-//!		
+//!
 // *******************************************************************************************
 
 nBasicNetworkSystem const & nBasicNetworkSystem::GetListener( nSocketListener & listener ) const
@@ -888,14 +884,14 @@ nBasicNetworkSystem const & nBasicNetworkSystem::GetListener( nSocketListener &
 }
 
 // *******************************************************************************************
-// *	
+// *
 // *	SetListener
-// *	
+// *
 // *******************************************************************************************
-//!		
+//!
 //!		@param	listener	listening sockets to set
 //!		@return		A reference to this to allow chaining
-//!		
+//!
 // *******************************************************************************************
 
 nBasicNetworkSystem & nBasicNetworkSystem::SetListener( nSocketListener const & listener )
@@ -938,14 +934,14 @@ nSocket const & nBasicNetworkSystem::GetControlSocket( void ) const
 
 /*
 // *******************************************************************************************
-// *	
+// *
 // *	GetControlSocket
-// *	
+// *
 // *******************************************************************************************
-//!		
+//!
 //!		@param	controlSocket	network control socket to fill
 //!		@return		A reference to this to allow chaining
-//!		
+//!
 // *******************************************************************************************
 
 nBasicNetworkSystem const & nBasicNetworkSystem::GetControlSocket( nSocket & controlSocket ) const
@@ -955,14 +951,14 @@ nBasicNetworkSystem const & nBasicNetworkSystem::GetControlSocket( nSocket & con
 }
 
 // *******************************************************************************************
-// *	
+// *
 // *	SetControlSocket
-// *	
+// *
 // *******************************************************************************************
-//!		
+//!
 //!		@param	controlSocket	network control socket to set
 //!		@return		A reference to this to allow chaining
-//!		
+//!
 // *******************************************************************************************
 
 nBasicNetworkSystem & nBasicNetworkSystem::SetControlSocket( nSocket const & controlSocket )
@@ -1089,6 +1085,8 @@ int nAddress::FromString( const char * string )
     int ha1, ha2, ha3, ha4, hp;
     int ipaddr;
 
+    tString source( string );
+
     // parse IP address if the passed name looks like one
     if (string[0] >= '0' && string[0] <= '9')
     {
@@ -1106,23 +1104,20 @@ int nAddress::FromString( const char * string )
     {
         addr_.addr   .sa_family = AF_INET;
 
-        // copy input string
-        tString stringCopy ( string );
-
         // find colon
-        char * colon = const_cast< char * >( strrchr (stringCopy, ':') );
-        if ( colon )
+        int colonPos = source.StrPos(":");
+        if ( colonPos > 0 )
         {
-            *colon = 0;
+            tString addr = source.SubStr( 0, colonPos );
+
             // extract port
-            SetPort( atoi( colon + 1 ) );
+            SetPort( source.ToInt( colonPos + 1 ) );
 
             // extract hostname
-            if ( stringCopy == "*.*.*.*" )
+            if ( addr == "*.*.*.*" )
                 addr_.addr_in.sin_addr.s_addr = INADDR_ANY;
             else
-                SetHostname( stringCopy );
-            *colon = ':';
+                SetHostname( addr );
 
             return 0;
         }
@@ -1408,22 +1403,6 @@ const nAddress & nAddress::GetPort( int & port ) const
     return *this;
 }
 
-
-// *******************************************************************************************
-// *
-// *	IsSet
-// *
-// *******************************************************************************************
-//!
-//!		@return		    true only if address was set to something valid
-//!
-// *******************************************************************************************
-
-bool nAddress::IsSet () const
-{
-    return addr_.addr_in.sin_addr.s_addr != INADDR_ANY;
-}
-
 // *******************************************************************************************
 // *
 // *	Compare
@@ -1432,7 +1411,7 @@ bool nAddress::IsSet () const
 //!
 //!		@param	a1  first address to compare
 //!		@param	a2  second address to compare
-//!		@return -1 if the addresses don't match, +1 if the ports don't match, 0 if everything matches.
+//!		@return
 //!
 // *******************************************************************************************
 
@@ -1517,29 +1496,23 @@ int nSocket::Create( void )
     // initialize networking at OS level
     sn_InitOSNetworking();
 
-    int socktype = socktype_;
-#ifndef WIN32
-#ifndef MACOSX
-    socktype |= SOCK_CLOEXEC;
-#endif
-#endif
-
     // open new socket
-    socket_ = socket( family_, socktype, protocol_ );
+    socket_ = socket( family_, socktype_, protocol_ );
     if ( socket_ < 0 )
         return -1;
 
-    // set TOS to low latency ( see manpages getsockopt(2), ip(7) and socket(7) )
-    // maybe this works for Windows, too?
-#ifndef WIN32
-    char tos = IPTOS_LOWDELAY;
+    // TODO: IP_TOS only Supported under Windows 2000
+    // See: http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winsock/winsock/socket_options.asp
 
-    setsockopt( socket_, IPPROTO_IP, IP_TOS, &tos, sizeof(char) );
-#endif
+    // Tutorial on using Windows 98+ RSVP (QoS / Resource reSerVation Protocol)
+    // http://msdn.microsoft.com/msdnmag/issues/01/04/qos/default.aspx
+
+    // TODO: Linux only way setting the IP_TOS ?
+    // http://homepages.cwi.nl/~aeb/linux/man2html/man2/getsockopt.2.html
 
     // unblock it
-    unsigned long _true = true;
-    return ioctl (socket_, FIONBIO, &_true) == -1;
+    bool _true = true;
+    return ioctl (socket_, FIONBIO, reinterpret_cast<char *>(&_true)) == -1;
 }
 
 // archives the binding procedure
@@ -1942,7 +1915,7 @@ public:
     {
         // start archive block if archiving is active
         Archiver archive;
-        if( archive.Initialize( recordingSectionRead, true ) )
+        if( archive.Initialize( recordingSectionRead ) )
         {
             // archive length of message
             archive.Archive( len );
@@ -2037,7 +2010,6 @@ int nSocket::Read( int8 * buf, int len, nAddress & addr ) const
 
     if ( ret >= 0 )
     {
-        // con << "Read " << ret << " bytes from socket " << GetSocket() << ".\n";
     }
 
 #ifdef PRINTPACKETS
@@ -2082,9 +2054,9 @@ int nSocket::Write( const int8 * buf, int len, const sockaddr * addr, int addrle
 
     // check if return value was archived
     static char const * section = "SEND";
-    static tReproducibleRandomizer randomizer;
     if ( !tRecorder::Playback( section, ret ) )
     {
+        static tReproducibleRandomizer randomizer;
 #ifdef DEBUG
         // pretend send was successful in packet loss simulation
         if ( sn_simulateSendPacketLoss > randomizer.Get() )
@@ -2616,65 +2588,6 @@ nSocket * nBasicNetworkSystem::Init()
 
 }
 
-// *******************************************************************************
-// *
-// *	Select
-// *
-// *******************************************************************************
-//!
-//!		@param	dt	the time in seconds to wait at max
-//!		@return		true if data came in
-//!
-// *******************************************************************************
-
-bool nBasicNetworkSystem::Select( REAL dt )
-{
-    int retval = 0;
-    static char const * section = "NETSELECT";
-    if ( !tRecorder::PlaybackStrict( section, retval ) )
-    {
-        if ( controlSocket_.GetSocket() < 0 )
-        {
-            tDelay( int( dt * 1000000 ) );
-        }
-        else
-        {
-            fd_set rfds; // set of sockets to watch
-            struct timeval tv; // time value to pass to select()
-
-            FD_ZERO( &rfds );
-
-            // watch the control socket
-            FD_SET( controlSocket_.GetSocket(), &rfds );
-            // con << "Watching " << controlSocket_.GetSocket();
-
-            int max = controlSocket_.GetSocket();
-
-            // watch listening sockets
-            for( nSocketListener::SocketArray::const_iterator iter = listener_.GetSockets().begin(); iter != listener_.GetSockets().end(); ++iter )
-            {
-                FD_SET( (*iter).GetSocket(), &rfds );
-                if ( (*iter).GetSocket() > max )
-                    max = (*iter).GetSocket();
-                // con << ", " << (*iter).GetSocket();
-            }
-
-            // set time
-            tv.tv_sec  = static_cast< long int >( dt );
-            tv.tv_usec = static_cast< long int >( (dt-tv.tv_sec)*1000000 );
-
-            // delegate to system select
-            retval = select(max+1, &rfds, NULL, NULL, &tv);
-        }
-    }
-    tRecorder::Record( section, retval );
-
-    // con << " : " << retval << "\n";
-
-    // return result
-    return ( retval > 0 );
-}
-
 // *******************************************************************************************
 // *
 // *	Shutdown
