@@ -28,65 +28,62 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rFont.h"
 #include "rScreen.h"
 #include "tConfiguration.h"
-#include "tColor.h"
+#include "tDirectories.h"
+#include "tCoord.h"
+#include "rTexture.h"
 #include <ctype.h>
 
 #ifndef DEDICATED
 #include "rRender.h"
+#include <FTGLPixmapFont.h>
+#include <FTGLBitmapFont.h>
+#include <FTGLTextureFont.h>
+#include <FTGLPolygonFont.h>
+#include <FTGLOutlineFont.h>
+#include <FTGLExtrdFont.h>
 //#include <GL/gl>
 //#include <SDL>
 #endif
 
-/*
-#include "nConfig.h"
+static REAL sr_bigFontThresholdWidth  = 12;
+static REAL sr_bigFontThresholdHeight = 24;
 
-tString lala_font_extra("Anonymous/original/textures/font_extra.png");
-static nSettingItem<tString> lalala_font_extra("TEXTURE_FONT_EXTRA", lala_font_extra);
-static rFont sr_lowerPartFont(lala_font_extra);
-static rFont sr_lowerPartFont("Anonymous/original/textures/font_extra.png");
+static tSettingItem< REAL > sr_bigFontThresholdWidthConf(  "FONT_BIG_THRESHOLD_WIDTH", sr_bigFontThresholdWidth );
+static tSettingItem< REAL > sr_bigFontThresholdHeightConf( "FONT_BIG_THRESHOLD_HEIGHT", sr_bigFontThresholdHeight );
 
-tString lala_defaultFont("Anonymous/original/textures/font.png");
-static nSettingItem<tString> lalala_defaultFont("TEXTURE_DEFAULT_FONT", lala_defaultFont);
-rFont rFont::s_defaultFont(lala_defaultFont, &sr_lowerPartFont);
-rFont rFont::s_defaultFont("Anonymous/original/textures/font.png", &sr_lowerPartFont);
-
-tString lala_defaultFontSmall("Anonymous/original/textures/font_s.png");
-static nSettingItem<tString> lalala_defaultFontSmall("TEXTURE_DEFAULT_FONT_SMALL", lala_defaultFontSmall);
-rFont rFont::s_defaultFontSmall(lala_defaultFontSmall, 32,5/128.0,9/128.0,1/128.0);
-rFont rFont::s_defaultFontSmall("Anonymous/original/textures/font_s.png", 32,5/128.0,9/128.0,1/128.0);
-*/
+class rFont:public rFileTexture{
+    int offset;
+    REAL cwidth;
+    REAL cheight;
+    REAL onepixel;
+    rFont *lowerPart;
+public:
+    rFont(const char *fileName,int Offset=0,REAL CWidth=(1/16.0),
+          REAL CHeight=(1/8.0),REAL onepixel=1/256.0, rFont *lower=NULL);
+    rFont(const char *fileName, rFont *lower);
+    virtual ~rFont();
 
 #ifndef DEDICATED
-//! like strnlen, but that's nonstandard :-(
-static size_t my_strnlen(char const *c, size_t i) {
-	char const *begin = c;
-	char const *end = c + i;
-	for(; *c && c != end; ++c) ;
-	return c - begin;
-}
+    // displays c
+    void Render(unsigned char c,REAL left,REAL top,REAL right,REAL bot);
 #endif
+    static rFont s_defaultFont,s_defaultFontSmall;
 
-static rFont sr_lowerPartFont("textures/font_extra.png");
-rFont rFont::s_defaultFont("textures/font.png", &sr_lowerPartFont);
-rFont rFont::s_defaultFontSmall("textures/font_s.png",32,5/128.0,9/128.0,1/128.0, -1);
-//rFont rFont::s_defaultFontSmall("textures/Font.png",0,16/256.0,32/256.0);
-//rFont rFont::s_defaultFontSmall("textures/Font.png",0,1/16.0,1/8.0);
-
-rFont::rFont(const char *fileName,int Offset,REAL CWidth,REAL CHeight,REAL op, int border, rFont *lower):
+protected:
+    virtual void ProcessImage(SDL_Surface *);       //!< process the surface before uploading it to GL
+    virtual void OnSelect( bool enforce );
+};
+rFont::rFont(const char *fileName,int Offset,REAL CWidth,REAL CHeight,REAL op, rFont *lower):
         rFileTexture(rTextureGroups::TEX_FONT,fileName,0,0),
         offset(Offset),cwidth(CWidth),cheight(CHeight),
-        onepixel(op),borderExtension(border), lowerPart(lower)
-{
-    StoreAlpha();
-}
+        onepixel(op),lowerPart(lower)
+{StoreAlpha();}
 
 rFont::rFont(const char *fileName, rFont *lower):
         rFileTexture(rTextureGroups::TEX_FONT,fileName,0,0),
         offset(0),cwidth(1/16.0),cheight(1/8.0),
         onepixel(1/256.0),lowerPart(lower)
-{
-    StoreAlpha();
-}
+{StoreAlpha();}
 
 rFont::~rFont(){}
 
@@ -109,18 +106,18 @@ void rFont::ProcessImage( SDL_Surface * surface )
     // pre-blend alpha values
     GLubyte *pixels =reinterpret_cast<GLubyte *>(surface->pixels);
 
-    if (surface->format->BytesPerPixel == 4)
+    if(surface->format->BytesPerPixel == 4)
     {
-        for (int i=surface->w*surface->h-1;i>=0;i--){
+        for(int i=surface->w*surface->h-1;i>=0;i--){
             GLubyte alpha=pixels[4*i+3];
             pixels[4*i  ] = (alpha * pixels[4*i  ]) >> 8;
             pixels[4*i+1] = (alpha * pixels[4*i+1]) >> 8;
             pixels[4*i+2] = (alpha * pixels[4*i+2]) >> 8;
         }
     }
-    else if (surface->format->BytesPerPixel == 2)
+    else if(surface->format->BytesPerPixel == 2)
     {
-        for (int i=surface->w*surface->h-1;i>=0;i--){
+        for(int i=surface->w*surface->h-1;i>=0;i--){
             GLubyte alpha=pixels[2*i+1];
             pixels[2*i  ] = (alpha * pixels[2*i  ]) >> 8;
         }
@@ -136,16 +133,10 @@ void rFont::OnSelect( bool enforce )
         // abort. It makes no sense to continue without a font.
         tERR_ERROR( "Font file " << this->GetFileName() << " could not be loaded.");
     }
-
-#ifndef DEDICATED
-    // wrap around so we can use the transparent pixel on the right side on the left
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
-#endif
 }
 
 // displays c
 #ifndef DEDICATED
-static rFont * sr_lastSelected = 0;
 void rFont::Render(unsigned char c,REAL left,REAL top,REAL right,REAL bot){
     //  if (c > 128 && this == &rFont::s_defaultFont)
     //rFont::s_defaultFontSmall.Render(c, left, top, right, bot);
@@ -164,7 +155,7 @@ void rFont::Render(unsigned char c,REAL left,REAL top,REAL right,REAL bot){
 
         REAL ttop=y*cheight+pix;
         REAL tbot=(y+1)*cheight-pix;
-        REAL tleft=x*cwidth+borderExtension*pix;
+        REAL tleft=x*cwidth+pix;
         REAL tright=(x+1)*cwidth-pix;
 
         rFont* select = this;
@@ -174,72 +165,209 @@ void rFont::Render(unsigned char c,REAL left,REAL top,REAL right,REAL bot){
             ttop -= 1;
             select = select->lowerPart;
         }
-        if ( sr_lastSelected != select )
-        {
-            RenderEnd(true);
-            select->Select(true);
-            sr_lastSelected = select;
-        }
-        
-        BeginQuads();
+        select->Select(true);
 
-        glTexCoord2f(tright,tbot);
-        glVertex2f(   right, bot);
+        BeginQuads();
+        glTexCoord2f(tleft,ttop);
+        glVertex2f(   left, top);
 
         glTexCoord2f(tright,ttop);
         glVertex2f(   right ,top);
 
-        glTexCoord2f(tleft,ttop);
-        glVertex2f(   left, top);
+        glTexCoord2f(tright,tbot);
+        glVertex2f(   right, bot);
 
         glTexCoord2f(tleft,tbot);
         glVertex2f(   left, bot);
+        RenderEnd();
     }
 }
 #endif
 
-// **************************************************
+static rFont sr_lowerPartFont("textures/font_extra.png");
+rFont rFont::s_defaultFont("textures/font.png", &sr_lowerPartFont);
+rFont rFont::s_defaultFontSmall("textures/font_s.png",32,5/128.0,9/128.0,1/128.0);
 
-static REAL sr_bigFontThresholdWidth  = 10;
-static REAL sr_bigFontThresholdHeight = 20;
+#ifndef DEDICATED
 
-static tSettingItem< REAL > sr_bigFontThresholdWidthConf(  "FONT_BIG_THRESHOLD_WIDTH", sr_bigFontThresholdWidth );
-static tSettingItem< REAL > sr_bigFontThresholdHeightConf( "FONT_BIG_THRESHOLD_HEIGHT", sr_bigFontThresholdHeight );
+int sr_fontType = sr_fontTexture;
+static tConfItem< int > sr_fontTypeConf( "FONT_TYPE", sr_fontType, &sr_ReloadFont);
 
-static REAL sr_smallFontThresholdWidth  = 5;
-static REAL sr_smallFontThresholdHeight = 8;
+static float sr_fontSizeFactor = .9;
+static tConfItem< float > sr_fontSizeFactorConf( "FONT_SIZE_FACTOR", sr_fontSizeFactor, &sr_ReloadFont );
 
-static tSettingItem< REAL > sr_smallFontThresholdWidthConf(  "FONT_SMALL_THRESHOLD_WIDTH", sr_smallFontThresholdWidth );
-static tSettingItem< REAL > sr_smallFontThresholdHeightConf( "FONT_SMALL_THRESHOLD_HEIGHT", sr_smallFontThresholdHeight );
+class rFontContainer : std::map<int, FTFont *> {
+    FTFont &New(int size);
+    FTFont *Load(tString const &path);
+public:
+    void clear() {
+        for(iterator i = begin(); i != end(); ++i) {
+            delete i->second;
+        }
+        std::map<int, FTFont *>::clear();
+    }
+    float GetWidth(tString const &str, float height) {
+        if(sr_fontType == sr_fontPixmap) {
+            return height*(height*sr_screenHeight < sr_bigFontThresholdHeight ? .41 : .5)*str.size();
+        }
+        return GetFont(height).Advance(str.c_str())/sr_screenWidth*2.;
+    }
+    void Render(tString const &str, float height, tCoord const &where) {
+        if (sr_fontType != sr_fontOld) {
+            if(sr_fontType >= sr_fontTexture) {
+                glPushMatrix();
+                glTranslatef(where.x, where.y, 0.);
+                glScalef(2./sr_screenWidth, 2./sr_screenHeight, 1.);
+                if(sr_fontType == sr_fontTexture) {
+                    glEnable(GL_TEXTURE_2D);
+                    glEnable(GL_BLEND);
+                    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+                }
+                if(sr_fontType == sr_fontExtruded) {
+                    glEnable( GL_DEPTH_TEST);
+                    glDisable( GL_BLEND);
+                    glEnable(GL_TEXTURE_2D);
+                    static rFileTexture sg_RimWallNoWrap(rTextureGroups::TEX_WALL,"textures/dir_wall.png",1,0);
+                    sg_RimWallNoWrap.Select();
+                    glRotatef(45,1.,0.,0.);
+                }
+            } else {
+                glRasterPos2f(where.x, where.y);
+            }
+            GetFont(height).Render(str.c_str());
+            if(sr_fontType >= sr_fontTexture) {
+                glPopMatrix();
+            }
+        } else {
+            rFont *font;
+            float widthFactor;
+            if (height*sr_screenHeight < sr_bigFontThresholdHeight) {
+                font = &rFont::s_defaultFontSmall;
+                widthFactor = .41;
+            } else {
+                font = &rFont::s_defaultFont;
+                widthFactor = .5;
+            }
+            float left = where.x;
+            for(tString::const_iterator i = str.begin(); i < str.end(); ++i, left += height*widthFactor) {
+                //void rFont::Render(unsigned char c,REAL left,REAL top,REAL right,REAL bot){
+                font->Render(*i, left, where.y+height, left+widthFactor*height, where.y);
+                //F->Render(buffer[realx],l,t,l+cwidth,t-cheight);
+            }
+        }
+    }
+    FTFont &GetFont(float height) {
+        int size = int(height*sr_fontSizeFactor*sr_screenHeight/2.+.5);
+        if(count(size)) {
+            return *((*this)[size]); //already exists
+        } else {
+            return New(size);
+        }
+    }
+    void BBox(tString const &str, float height, tCoord where, float &l, float &b, float &r, float &t) {
+        if(sr_fontType != sr_fontOld) {
+            float rubbish;
+            GetFont(height).BBox(str.c_str(), l, b, rubbish, r, t, rubbish);
+            l/=sr_screenWidth/2.;
+            r/=sr_screenWidth/2.;
+            t/=sr_screenHeight/2.;
+            b/=sr_screenHeight/2.;
+            l+=where.x-0.005;
+            r+=where.x+0.005;
+            t+=where.y+0.005;
+            b+=where.y-0.005;
+            return;
+        } else {
+            l=where.x;
+            r=where.x+GetWidth(str, height);
+            b=where.y;
+            t=where.y+height;
+        }
+    }
+    ~rFontContainer() {
+        clear();
+    }
+};
+
+// Font config items
+
+tString fontFile("Armagetronad.ttf");
+static tConfItemLine ff("FONT_FILE", fontFile, &sr_ReloadFont);
+
+static tString customFont("");
+static tConfItemLine ffc("FONT_FILE_CUSTOM", customFont, &sr_ReloadFont);
+
+static int useCustomFont = 0;
+static tConfItem<int> ufc("USE_CUSTOM_FONT", useCustomFont, &sr_ReloadFont);
+
+static rCallbackBeforeScreenModeChange reloadft(&sr_ReloadFont);
+
+FTFont *rFontContainer::Load(tString const &path) {
+    FTFont *font;
+    switch (sr_fontType) {
+    case sr_fontPixmap:
+        font = new FTGLPixmapFont(path);
+        break;
+    case sr_fontBitmap:
+        font = new FTGLBitmapFont(path);
+        break;
+    case sr_fontPolygon:
+        font = new FTGLPolygonFont(path);
+        break;
+    case sr_fontOutline:
+        font = new FTGLOutlineFont(path);
+        break;
+    case sr_fontExtruded:
+        font = new FTGLExtrdFont(path);
+        reinterpret_cast<FTGLExtrdFont *>(font)->Depth(10.);
+        break;
+    default:
+        font = new FTGLTextureFont(path);
+    }
+    return font;
+}
+FTFont &rFontContainer::New(int size) {
+    FTFont *font;
+    tString theFontFile("");
+
+    if(useCustomFont == 1) {
+        theFontFile = customFont;
+    } else {
+        theFontFile = "textures/" + fontFile;
+        theFontFile = tDirectories::Data().GetReadPath(theFontFile);
+    }
+    font = Load(theFontFile);
+    //std::cout << "Use custom font: " << useCustomFont << std::endl;
+    //std::cout << "The font file: " << theFontFile << std::endl;
+    if(font->Error()) {
+        std::cerr << "Error while loading font from path '" << theFontFile << "'. Error code: " << font->Error() << std::endl;
+        //delete font;
+        std::cerr << "Loading default font instead.  Sorry." << std::endl;
+	font = Load(tDirectories::Data().GetReadPath("textures/Armagetronad.ttf"));
+
+    }
+    font->FaceSize(size);
+    (*this)[size] = font;
+    return *font;
+}
+rFontContainer sr_Font;
+
+void sr_ReloadFont(void) {
+    sr_Font.clear();
+}
+#endif
 
 rTextField::rTextField(REAL Left,REAL Top,
-                       REAL Cwidth,REAL Cheight,
-                       rFont *f)
+                       REAL Cheight, sr_fontClass Type)
         :parIndent(0),
-        left(Left),top(Top),cwidth(Cwidth),cheight(Cheight),
-        F(f),x(0),y(0),realx(0),cursor(0),cursorPos(0){
-    if ( cwidth*sr_screenWidth < sr_bigFontThresholdWidth*2 || cheight*sr_screenHeight < sr_bigFontThresholdHeight*2 )
-        F=&rFont::s_defaultFontSmall;
-    if (cwidth*sr_screenWidth <= sr_smallFontThresholdWidth*2 + 1E-4)
-    {
-        cwidth=sr_smallFontThresholdWidth*2/REAL(sr_screenWidth);
-
-        // try to place font at exact pixels
-        left = Pixelize(left, sr_screenWidth);
-    }
-    if (cheight*sr_screenHeight <= sr_smallFontThresholdHeight*2 + 1E-4)
-    {
-        cheight=sr_smallFontThresholdHeight*2/REAL(sr_screenHeight);
-
-        // try to place font at exact pixels
-        top = Pixelize(top, sr_screenHeight);
-    }
+left(Left),top(Top),cheight(Cheight),x(0),y(0),realx(0),nextx(Left),currentWidth(0),multiline(false),type(Type),cursor(0),cursorPos(0){
+    if (cheight*sr_screenHeight<18)
+        cheight=18/REAL(sr_screenHeight);
 
     color_ = defaultColor_;
 
-    width = int((1-Left)/cwidth);
+    width = 1.-Left;
 
-    buffer.SetLen(0);
     /*
     top=(int(top*sr_screenHeight)+.5)/REAL(sr_screenHeight);
     left=(int(left*sr_screenWidth)+.5)/REAL(sr_screenWidth);
@@ -249,28 +377,11 @@ rTextField::rTextField(REAL Left,REAL Top,
     cursor_y = -100;
 }
 
-REAL rTextField::AspectWidthMultiplier()
-{
-    return std::min(((4.0f/3.0f)*sr_screenHeight)/sr_screenWidth,1.0f);
-}
-
-REAL rTextField::AspectHeightMultiplier()
-{
-    return std::min(((3.0f/4.0f)*sr_screenWidth)/sr_screenHeight,1.0f);
-}
-
-REAL rTextField::Pixelize(REAL xy, int WidthHeight)
-{
-    auto pixelIn = static_cast<int>(.5f * xy * WidthHeight);
-    return (2.0f*(pixelIn+.5f))/WidthHeight;
-}
-
 
 rTextField::~rTextField(){
-#ifndef DEDICATED
     FlushLine();
-    RenderEnd();
 
+#ifndef DEDICATED
     if (cursor && sr_glOut){
         if (cursor==2)
             glColor4f(1,1,1,.5);
@@ -288,21 +399,24 @@ rTextField::~rTextField(){
 #endif
 }
 
+// minimal tolerated values of font color before a white background is rendered
+static REAL sr_minR = .5, sr_minG = .5, sr_minB =.5, sr_minTotal = .7;
+tSettingItem< REAL > sr_minRConf( "FONT_MIN_R", sr_minR );
+tSettingItem< REAL > sr_minGConf( "FONT_MIN_G", sr_minG );
+tSettingItem< REAL > sr_minBConf( "FONT_MIN_B", sr_minB );
+tSettingItem< REAL > sr_minTotalConf( "FONT_MIN_TOTAL", sr_minTotal );
+
 void rTextField::FlushLine(int len,bool newline){
 #ifndef DEDICATED
-    // reload textures if alpha blending changed
-    {
-        static bool alphaBlendBefore = sr_alphaBlend;
-        if ( alphaBlendBefore != sr_alphaBlend )
-        {
-            alphaBlendBefore = sr_alphaBlend;
-            sr_lowerPartFont.Unload();
-            rFont::s_defaultFont.Unload();
-            rFont::s_defaultFontSmall.Unload();
-        }
+    float realTop = top-y*cheight;
+    tString str(buffer.SubStr(realx, len));
+    realx += len;
+    if (len >= cursorPos && cursorPos >= 0) {
+        cursor_y=realTop;
+        cursor_x=nextx+sr_Font.GetWidth(str.SubStr(0, cursorPos), cheight);
     }
-
-    int i;
+    cursorPos -= len+1;
+    float thisx = nextx+sr_Font.GetWidth(str, cheight);
 
     REAL r = color_.r_;
     REAL g = color_.g_;
@@ -312,28 +426,30 @@ void rTextField::FlushLine(int len,bool newline){
     if (sr_glOut)
     {
         // render bright background
-        if ( color_.IsDark() )
+        if ( r < sr_minR && g < sr_minG && b < sr_minG || r+g+b < sr_minTotal )
         {
-            RenderEnd(true);
-            glDisable(GL_TEXTURE_2D);
-            if ( sr_alphaBlend )
+            if ( sr_alphaBlend && !str.empty() )
             {
+
+                glDisable(GL_TEXTURE_2D);
+
                 glColor4f( blendColor_.r_, blendColor_.g_, blendColor_.b_, a * blendColor_.a_ );
 
-                REAL l=left+realx*cwidth;
-                REAL t=top-y*cheight;
-                REAL r=l + cwidth * len;
-                REAL b=t - cheight;
+                float l,t,r,b;
+
+                sr_Font.BBox(str, cheight, tCoord(nextx, realTop-cheight), l, b, r, t);
+
+                //sr_ResetRenderState(true);
 
                 BeginQuads();
-
-                glVertex2f(   l, b);
-
-                glVertex2f(   r, b);
+                glVertex2f(   l, t);
 
                 glVertex2f(   r ,t);
 
-                glVertex2f(   l, t);
+                glVertex2f(   r, b);
+
+                glVertex2f(   l, b);
+                RenderEnd();
             }
             else
             {
@@ -341,32 +457,16 @@ void rTextField::FlushLine(int len,bool newline){
                 if ( g < .5 ) g = .5;
                 if ( b < .5 ) b = .5;
             }
-            RenderEnd(true);
-            glEnable(GL_TEXTURE_2D);
         }
 
-
-        if ( len > 0 )
-        {
-            RenderEnd(true);
-            glColor4f(r * blendColor_.r_,g * blendColor_.g_,b * blendColor_.b_,a * blendColor_.a_);
-            sr_lastSelected = 0;
-        }
-        for (i=0;i<=len;i++){
-            REAL l=left+realx*cwidth;
-            REAL t=top-y*cheight;
-            
-            if (0 <= cursorPos--){
-                cursor_x=l;
-                cursor_y=t;
-            }
-            if (i<len){
-                F->Render(buffer[realx],l,t,l+cwidth,t-cheight);
-                realx++;
-            }
-        }
+        glColor4f(r * blendColor_.r_,g * blendColor_.g_,b * blendColor_.b_,a * blendColor_.a_);
     }
 
+    //F->Render(buffer[realx],l,t,l+cwidth,t-cheight);
+    glRasterPos2f(nextx, realTop-cheight);
+    sr_Font.Render(str, cheight, tCoord(nextx, realTop-cheight));
+    nextx = thisx;
+
 #endif
     /*
     for(i=0;i<buffer.Len()-len;i++)
@@ -378,6 +478,7 @@ void rTextField::FlushLine(int len,bool newline){
     if (newline){
         y++;
         realx=x=0;
+        nextx=left;
     }
     else
     {
@@ -388,18 +489,19 @@ void rTextField::FlushLine(int len,bool newline){
 }
 
 void rTextField::FlushLine(bool newline){
-    FlushLine(buffer.Len()-realx,newline);
+    FlushLine(buffer.Size()-realx,newline);
 }
 
 inline void rTextField::WriteChar(unsigned char c)
 {
-    switch (c){
+    switch(c){
     case('\n'):
                     FlushLine();
-        buffer.SetLen(0);
+        buffer.Clear();
         break;
     default:
-        buffer[x++]=c;
+        buffer += c;
+        x++;
         break;
     }
 }
@@ -441,21 +543,61 @@ rTextField & rTextField::operator<<(unsigned char c){
 }
 */
 
-rTextField & rTextField::StringOutput(const char * c, ColorMode colorMode )
+#ifndef DEDICATED
+static REAL CTR(int x){
+    return x/255.0;
+}
+#endif
+
+static char hex_array[]="0123456789abcdef";
+
+char int_to_hex(int i){
+    if (i<0 || i >15)
+        return 'Q';
+    else
+        return hex_array[i];
+}
+
+int hex_to_int(char c){
+    int ret=0;
+    for(int i=15;i>=0;i--)
+        if (hex_array[i]==c)
+            ret=i;
+    return ret;
+}
+
+rTextField & rTextField::StringOutput(const char * c, ColorMode colorMode)
 {
 #ifndef DEDICATED
+    //float currentWidth = nextx - left;
+    float const &maxWidth = width;
+    bool lastIsNewline = true;
+    bool trouble = false; // Do we have a word that won't fit on a line?
     // run through string
     while (*c!='\0')
     {
+        if (trouble && !(*c=='0' && strlen(c)>=8 && c[1]=='x' && colorMode != COLOR_IGNORE)) {
+            tString str;
+            str << *c;
+            currentWidth += sr_Font.GetWidth(str, cheight);
+            if(isspace(*c)) {
+                trouble = false;
+            } else if ( currentWidth >= maxWidth) {
+                WriteChar('\n');
+                c++;
+                currentWidth = 0.;
+            }
+        }
         // break line if next space character is too far away
-        if ( isblank(*c) )
+        if ( !trouble && multiline && (isspace(*c) || lastIsNewline) )
         {
+            lastIsNewline = false;
             // count number of nonblank characters following
             char const * nextSpace = c+1;
             int wordLen = 0;
-            while ( *nextSpace != '\0' && *nextSpace != '\n' && !isblank(*nextSpace) )
+            while ( *nextSpace != '\0' && !isspace(*nextSpace) )
             {
-                if (*nextSpace=='0' && my_strnlen(nextSpace, 8)>=8 && nextSpace[1]=='x' && colorMode != COLOR_IGNORE )
+                if (*nextSpace=='0' && strlen(nextSpace)>=8 && nextSpace[1]=='x' && colorMode != COLOR_IGNORE )
                 {
                     // skip color code
                     nextSpace += 8;
@@ -467,47 +609,54 @@ rTextField & rTextField::StringOutput(const char * c, ColorMode colorMode )
                     wordLen++;
                 }
             }
+            tString str(c, nextSpace);
+            str = tColoredString::RemoveColors(str.c_str());
+            float wordWidth = sr_Font.GetWidth(str, cheight);
 
-            // see if the word plus the space fit into the current line
-            if ( wordLen + x + 1 >= width )
+            currentWidth += wordWidth;
+            if ( currentWidth >= maxWidth)
             {
-                // no. Skip to the next line
                 WriteChar('\n');
                 c++;
-                for ( int i = parIndent-1; i >= 0; --i )
-                {
-                    WriteChar(' ');
-                    cursorPos++;
+                if (wordWidth >= maxWidth) {
+                    trouble = true;
+                    currentWidth = 0.;
+                } else {
+                    currentWidth = wordWidth;
                 }
                 continue;
             }
         }
-
-        // linebreak if line has gotten too long anyway
-        if ( x >= width )
-        {
-            WriteChar('\n');
-            cursorPos += 1;
+        if ( *c == '\n' ) {
+            lastIsNewline = true;
+            currentWidth = 0.;
         }
 
-        // detect presence of color code
-        if (*c=='0' && my_strnlen(c, 8)>=8 && c[1]=='x' && colorMode != COLOR_IGNORE )
-        {
-            tColor color;
-            bool use = false;
+        //// linebreak if line has gotten too long anyway
+        //if ( x >= width )
+        //{
+        //    WriteChar('\n');
+        //    currentWidth = 0.;
+        //}
 
+        // detect presence of color code
+        if (*c=='0' && strlen(c)>=8 && c[1]=='x' && colorMode != COLOR_IGNORE )
+        {
             if ( 0 ==strncmp(c,"0xRESETT",8) )
             {
                 // color reset to default requested
-                color = defaultColor_;
-                use = true;
+                ResetColor();
             }
             else
             {
                 // found! extract colors
-                cursorPos-=8;
-		color = tColor( c );
-                use = true;
+                FlushLine(false);
+
+                cursorPos-=7;
+
+                color_.r_=CTR(hex_to_int(c[2])*16+hex_to_int(c[3]));
+                color_.g_=CTR(hex_to_int(c[4])*16+hex_to_int(c[5]));
+                color_.b_=CTR(hex_to_int(c[6])*16+hex_to_int(c[7]));
             }
 
             // advance
@@ -517,84 +666,42 @@ rTextField & rTextField::StringOutput(const char * c, ColorMode colorMode )
             }
             else
             {
-                // write color code out
-                cursorPos+=8;
-                for(int i=7; i>=0;--i)
-                    WriteChar(*(c++));
-            }
-
-            // apply color
-            if ( use )
-            {
-                FlushLine(false);
-                cursorPos++;
-                color_ = color;
+                c++;
             }
         }
         else
             // normal operation: add char
             WriteChar(*(c++));
     }
-
-    RenderEnd( true );
 #endif
-
     return *this;
 }
 
-void DisplayText(REAL x,REAL y,REAL w,REAL h,const char *text,int center,int cursor,int cursorPos, rTextField::ColorMode colorMode ){
-    int colorlen = strlen(text);
-
-    if ( colorMode == rTextField::COLOR_USE )
-    {
-        for(char const *c = text; *c != 0; ++c)
-        {
-            if(*c == '0' && c[1] == 'x')
-                colorlen -= 8;
-        }
-    }
-
-    // calculate top position so that does not move when we shrink the font
-    REAL top = y + h*.5;
+void DisplayText(REAL x,REAL y,REAL h,const char *text,sr_fontClass type,int center,int cursor,int cursorPos){
+#ifndef DEDICATED
+    float height;
+    float width = rTextField::GetTextLength(tString(text), h, true, true, &height);
 
     // shrink fields that don't fit the screen
-    REAL availw = 1.9f;
-    if (center < 0) availw = (.9f-x);
-    if (center > 0) availw = (x + .9f);
-    if ( colorlen * w > availw )
+    REAL maxw = 1.95 - x;
+    if ( width > maxw )
     {
-        h *= availw/(colorlen * w);
-        w = availw/REAL(colorlen);
+        h *= maxw/(width);
+        width=maxw;
     }
 
-    rTextField c(x-(center+1)*colorlen*.5*w,y+h*.5,w,h);
+    rTextField c(x-(center+1)*width*.5,y+h*.5,h,type);
     if (center==-1)
-        c.SetWidth(int((.95-x)/c.GetCWidth()));
+        c.SetWidth(1.-x);
     else
         c.SetWidth(10000);
 
-    // did the text field enlarge the font? If yes, there will be wrapping; better make some more room
-    if ( c.GetWidth() < colorlen )
-    {
-        c.SetTop( top );
-    }
-
     c.SetIndent(5);
     if (cursor)
         c.SetCursor(cursor,cursorPos);
-    c.StringOutput(text, colorMode );
+    c << text;
+#endif
 }
-
-void DisplayTextAutoWidth(REAL x, REAL y, const char *text, REAL h, int center, int cursor, int cursorPos, rTextField::ColorMode colorMode)
-{
-    DisplayText(x, y, h*(rCWIDTH_NORMAL/rCHEIGHT_NORMAL)*rTextField::AspectWidthMultiplier(), h, text, center, cursor, cursorPos, colorMode);
-}
-
-void DisplayTextAutoHeight(REAL x, REAL y, const char *text, REAL w, int center, int cursor, int cursorPos, rTextField::ColorMode colorMode)
-{
-    DisplayText(x, y, w, w*(rCHEIGHT_NORMAL/rCWIDTH_NORMAL)*rTextField::AspectHeightMultiplier(), text, center, cursor, cursorPos, colorMode);
-}
-
 // *******************************************************************************************
 // *
 // *	GetDefaultColor
@@ -638,13 +745,6 @@ void rTextField::GetDefaultColor( tColor & defaultColor )
 void rTextField::SetDefaultColor( tColor const & defaultColor )
 {
     defaultColor_ = defaultColor;
-    if ( !sr_alphaBlend )
-    {
-        defaultColor_.r_ *= defaultColor_.a_;
-        defaultColor_.g_ *= defaultColor_.a_;
-        defaultColor_.b_ *= defaultColor_.a_;
-        defaultColor_.a_ = 1;
-    }
     blendColor_ = tColor();
 }
 
@@ -691,15 +791,22 @@ void rTextField::GetBlendColor( tColor & blendColor )
 void rTextField::SetBlendColor( tColor const & blendColor )
 {
     blendColor_ = blendColor;
-    if ( !sr_alphaBlend )
-    {
-        blendColor_.r_ *= blendColor_.a_;
-        blendColor_.g_ *= blendColor_.a_;
-        blendColor_.b_ *= blendColor_.a_;
-        blendColor_.a_ = 1;
-    }
 }
 
 tColor rTextField::defaultColor_;
 tColor rTextField::blendColor_;
 
+#ifndef DEDICATED
+//! @param str the string to be used
+//! @param height the height of one character
+//! @param stripColors should colors be recognized?
+//! @param useNewline should newlines be recognized (and the longest line be found)?
+//! @param resultingHeight address to store the number of lines (height times the number of newlines+1)
+//! @returns the width of the string if it was printed
+float rTextField::GetTextLength (tString const &str, float height, bool stripColors, bool useNewline, float *resultingHeight) {
+    if(stripColors) {
+        return sr_Font.GetWidth(tColoredString::RemoveColors(str.c_str()), height); //TODO: Implement all the rest!
+    }
+    return sr_Font.GetWidth(str, height); //TODO: Implement all the rest!
+}
+#endif
