@@ -20,7 +20,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-
+  
 ***************************************************************************
 
 */
@@ -31,46 +31,56 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rSDL.h"
 
 #include "defs.h"
-#include "rTexture.h"
+//#include "rTexture.h"
 #include "tString.h"
 #include "tColor.h"
 
-class rFont:public rFileTexture{
-    int offset;
-    REAL cwidth;
-    REAL cheight;
-    REAL onepixel;
-    int borderExtension{1}; // does the left side pixel border of a character get moved half a pixel left or right?
-    rFont *lowerPart;
-public:
-    rFont(const char *fileName,int Offset=0,REAL CWidth=(1/16.0),
-          REAL CHeight=(1/8.0),REAL onepixel=1/256.0, int borderExtension = 1, rFont *lower=NULL);
-    rFont(const char *fileName, rFont *lower);
-    virtual ~rFont();
+#include <map>
 
-#ifndef DEDICATED
-    // displays c
-    void Render(unsigned char c,REAL left,REAL top,REAL right,REAL bot);
-#endif
-    static rFont s_defaultFont,s_defaultFontSmall;
-
-protected:
-    virtual void ProcessImage(SDL_Surface *);       //!< process the surface before uploading it to GL
-    virtual void OnSelect( bool enforce );
+//! Different types of fonts, they might get different font files assigned
+enum sr_fontClass {
+    sr_fontConsole       = 00001, //!< for the console
+    sr_fontMenu          = 00002, //!< for menu text
+    sr_fontMenuTitle     = 00004, //!< for the titles above menus
+    sr_fontScoretable    = 00010, //!< for the score table (the one that comes when you press TAB)
+    sr_fontMenuHelp      = 00020, //!< for the help that pops up if you idle in the menu
+    sr_fontError         = 00040, //!< for error messages like the one that pops up if you get kicked from the server
+    sr_fontCockpit       = 00100, //!< for all font in the cockpit
+    sr_fontCenterMessage = 00200, //!< for CENTER_MESSAGEs
+    sr_fontServerBrowser = 00400, //!< for the LAN and master server browser
+    sr_fontCycleLabel    = 01000, //!< for the player name displayed over cycles
+    sr_fontServerDetails = 02000  //!< for the details displayed in the server browser (server description, player names etc)
 };
 
+//! Different ways to render fonts
+enum sr_fontTypes {
+    sr_fontOld       = 0, //!< The old font, deprecated
+    sr_fontPixmap    = 1, //!< FTGLPixmapFont
+    sr_fontBitmap    = 2, //!< FTGLBitmapFont
+    sr_fontTexture   = 3, //!< FTGLTextureFont
+    sr_fontPolygon   = 4, //!< FTGLPolygonFont
+    sr_fontOutline   = 5, //!< FTGLOutlineFont
+    sr_fontExtruded  = 6  //!< FTGLExtrdFont (experimental)
+};
 
-// **********************************************+
+extern int sr_fontType;
+
+class FTFont;
 
 // maybe make this a child of std::ostream...
 class rTextField{
     tString buffer;       // buffer where we store stuff before we print it
-    int  width;          // width in characters
+    float  width;          // width in openGL units
     int  parIndent;      // number of spaces to insert after automatic newline
     REAL left,top;       // top left corner of the console
-    REAL cwidth,cheight; // character dimensions
-    rFont *F;             // the font
+    REAL cheight; // character dimensions
+    //    rFont *F;             // the font
     int  x,y,realx;      // current cursor position
+    float nextx;          // x-coordinate the next char should go to for rendering
+    float currentWidth;   //The current position where the next char will go to for caching
+    bool multiline;        // linewrapping enabled?
+    FTFont *font;
+    sr_fontClass type;    //what is the type of this font?
 
     tColor color_;               //!< current color
     static tColor defaultColor_; //!< default color
@@ -84,69 +94,24 @@ class rTextField{
     void FlushLine(int len,bool newline=true);
     void FlushLine(bool newline=true);
 public:
-#define  rCWIDTH_NORMAL  (16/640.0)
-#define  rCHEIGHT_NORMAL (32/480.0)
+ #define  rCWIDTH_NORMAL  (16/640.0)
+ #define  rCHEIGHT_NORMAL (32/480.0)
 
     rTextField(REAL Left,REAL Top,
-               REAL Cwidth=rCWIDTH_NORMAL,REAL Cheight=rCHEIGHT_NORMAL,
-               rFont *f=&rFont::s_defaultFont);
-
-    // all the basic code assumes a 4:3 screen. We won't fix that here on 0.2.9.
-    // instead, users need to either multiply their witdh (and maybe left) with
-    // AspectWidthMultiplier() or their height (and Top) with AspectHeightMultiplier().
-    // Pick one for each context and stick with it. The Width modification should be
-    // the default.
-    static REAL AspectWidthMultiplier();
-    static REAL AspectHeightMultiplier();
-
-    // puts x/y right into the middle of a screen pixel given Width or Height.
-    static REAL Pixelize(REAL xy, int WidthHeight);
+               REAL Cheight, sr_fontClass Type);
 
     virtual ~rTextField(); // for future extensions (buffered console?)
 
-    REAL GetCWidth() const {
-        return cwidth;
-    }
-    REAL GetCHeight() const {
-        return cheight;
-    }
+    REAL GetCHeight(){ return cheight; }
 
-    void SetTop( REAL t ){
-        top = t;
-    }
-
-    void SetLeft( REAL l ){
-        top = l;
-    }
-    
-    REAL GetTop() const{
-        return top;
-    }
-    REAL GetBottom() const{
-        return top - cheight * y;
-    }
-
-
-    REAL GetLeft() const{
-        return left;
-    }
-
-    void SetWidth(int w){
+    void SetWidth(float w){
         width=w;
     }
 
-    int GetWidth() const {
-        return width;
-    }
-
     void SetIndent(int i){
         parIndent=i;
     }
 
-    int GetIndent() const {
-        return parIndent;
-    }
-
     void SetCursor(int c,int p){
         cursor=c;
         cursorPos=p;
@@ -157,6 +122,10 @@ public:
         color_ = defaultColor_;
     }
 
+    void EnableLineWrap() {
+        multiline = true;
+    }
+
     // rTextField & operator<<(unsigned char c);
 
     enum ColorMode {
@@ -167,9 +136,7 @@ public:
 
     rTextField & StringOutput(const char *c, ColorMode colorMode = COLOR_USE );
 
-    int Lines(){
-        return y;
-    }
+    int Lines(){return y;}
 
     inline rTextField & SetColor( tColor const & color );	//!< Sets current color
     inline tColor const & GetColor( void ) const;	//!< Gets current color
@@ -181,6 +148,8 @@ public:
     static tColor const & GetBlendColor( void );	//!< Gets color all other colors are multiplied with
     static void GetBlendColor( tColor & blendColor );	//!< Gets color all other colors are multiplied with
 
+    static float GetTextLength (tString const &str, float height, bool stripColors=false, bool useNewline=true, float *resultingHeight=0); //!< Predict the dimenstions of a string
+
 private:
     inline void WriteChar(unsigned char c); //!< writes a single character as it is, no automatic newline breaking
 };
@@ -191,14 +160,8 @@ template<class T> rTextField & operator<<(rTextField &c,const T &x){
     return c.StringOutput(out);
 }
 
-void DisplayText(REAL x,REAL y,REAL w,REAL h,const char *text,int center=0,
-                 int cursor=0,int cursorPos=0, rTextField::ColorMode colorMode = rTextField::COLOR_USE );
-
-void DisplayTextAutoWidth(REAL x,REAL y,const char *text,REAL h=rCHEIGHT_NORMAL,int center=0,
-                 int cursor=0,int cursorPos=0, rTextField::ColorMode colorMode = rTextField::COLOR_USE );
-
-void DisplayTextAutoHeight(REAL x,REAL y,const char *text,REAL w=rCWIDTH_NORMAL,int center=0,
-                 int cursor=0,int cursorPos=0, rTextField::ColorMode colorMode = rTextField::COLOR_USE );
+void DisplayText(REAL x,REAL y,REAL h,const char *text, sr_fontClass type,int center=0,
+                 int cursor=0,int cursorPos=0);
 
 // *******************************************************************************************
 // *
@@ -248,6 +211,8 @@ rTextField & rTextField::SetColor( tColor const & color )
     this->color_ = color;
     return *this;
 }
+//! Reloads the font (in case the resolution or font type changes)
+void sr_ReloadFont(void);
 
 #endif
 
