@@ -30,9 +30,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <errno.h>
 #include <sys/types.h>
-#ifndef WIN32
-#include <sys/stat.h>
-#endif
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -50,7 +47,7 @@ static const char * s_topSourceDir = TOP_SOURCE_DIR;
 #else
 static const char * s_topSourceDir = ".";
 #endif
-#endif
+#endif // !MACOSX_XCODE
 
 // program name definition
 #ifndef PROGNAME
@@ -69,6 +66,26 @@ static const char * s_topSourceDir = ".";
 #define PROGDIR_SUFFIX ""
 #endif
 
+#ifdef WIN32
+// activate used function from shlobj.h (z-man does not know if this is a bad hack)
+#ifdef __MINGW32__
+#define _WIN32_IE 0x400
+#endif
+
+#include <direct.h>
+#include <windows.h>
+#include <shlobj.h>
+#define mkdir(x, y)   _mkdir(x)
+#ifndef strdup
+#define strdup        _strdup
+#endif
+#ifndef _stat
+#define _stat stat
+#endif
+#else
+#include <pwd.h>
+#endif
+
 #ifdef TOP_SOURCE_DIR
 // #include "tPaths.h"
 #include "tUniversalVariables.h"
@@ -108,6 +125,7 @@ static tString st_bindirCompiled(BINDIR);
 #define _WIN32_IE 0x400
 #endif
 
+#undef DATADIR
 #include <direct.h>
 #include <windows.h>
 #include <shlobj.h>
@@ -146,7 +164,11 @@ static tString st_DataDir(expand_home_c(DATA_DIR));    // directory for game dat
 static tString st_DataDir(".");    // directory for game data
 #endif
 
-// #define USER_DATA_DIR  "${APPDATA}/Armagetron"
+#ifdef MUSIC_DIR
+static tString st_MusicDir(expand_home_c(DATA_DIR));    // directory for game music
+#else
+static tString st_MusicDir(".");    // directory for game music
+#endif
 
 #ifdef USER_DATA_DIR
 static tString st_UserDataDir(expand_home_c(USER_DATA_DIR));    // directory for game data
@@ -172,6 +194,14 @@ static tString st_VarDir(expand_home_c(VAR_DIR));     // directory for dynamic l
 static tString st_VarDir("");     // directory for dynamic logs and highscores
 #endif
 
+#ifdef DEDICATED
+#ifdef WWWROOTDIR
+static tString st_WwwDir(expand_home_c(WWWROOTDIR));     // directory for dynamic logs and highscores
+#else
+static tString st_WwwDir("");     // directory for dynamic logs and highscores
+#endif
+#endif
+
 #ifdef RESOURCE_DIR
 static tString st_ResourceDir(expand_home_c(RESOURCE_DIR));
 #else
@@ -455,11 +485,12 @@ char *eh_getdir(const char *da, size_t *len) {
 tString expand_home(tString const & pathname) {
     const char *pn = (const char *)pathname;
     char *s;
-    size_t len = 0;
+    size_t len;
     tString r;
 
     if ((pn[0] == '~' || !strncmp(pn, "${", 2)) && (s = eh_getdir(pn, &len))) {
-        r = tString(s) << (pn + len);
+        tString sStr(s);
+        r = sStr << (pn + len);
         free(s);
     }
     else
@@ -474,7 +505,6 @@ tString expand_home(tString const & pathname) {
 class tPathConfig: public tPath
 {
 public:
-    tPathConfig() {}
 private:
     void Paths ( tArray< tString >& paths ) const
     {
@@ -506,7 +536,6 @@ static const tPathConfig st_Config;
 class tPathData: public tPath
 {
 public:
-    tPathData() {}
 private:
     void Paths ( tArray< tString >& paths ) const
     {
@@ -524,11 +553,48 @@ private:
 
 static const tPathData st_Data;
 
+// This doesn't work because I don't understand this stuff at all :(  --Lucifer
+class tPathMusic: public tPath
+{
+public:
+private:
+    void Paths ( tArray< tString >& paths ) const
+    {
+        paths.SetLen( 0 );
+        int pos = 0;
+
+        paths[ pos++ ] = st_MusicDir;
+    }
+};
+
+static const tPathMusic st_Music;
+
+#ifdef DEDICATED
+void tPathWebroot::Paths ( tArray< tString >& paths ) const
+{
+    paths.SetLen( 0 );
+    int pos = 0;
+
+    paths[ pos++ ] = st_WwwDir;
+}
+
+tString tPathWebroot::GetDirPath() {
+    //std::cout << st_WwwDir;
+    return st_WwwDir;
+}
+
+static const tPathWebroot st_Webroot;
+
+const tPathWebroot& tDirectories::Webroot()
+{
+    return st_Webroot;
+}
+
+#endif
 
 class tPathVar: public tPath
 {
 public:
-    tPathVar() {}
 private:
     void Paths ( tArray< tString >& paths ) const
     {
@@ -555,7 +621,6 @@ static const tPathVar st_Var;
 class tPathScreenshot: public tPath
 {
 public:
-    tPathScreenshot() {}
 private:
     void Paths ( tArray< tString >& paths ) const
     {
@@ -602,7 +667,8 @@ tString tPathResource::GetWritePath(const char *filename) const {
     return fullname;
 }
 
-tString tPathResource::GetIncluded() const {
+tString tPathResource::GetDirPath()
+{
     if ( st_IncludedResourceDir.Len() > 2 )
         return st_IncludedResourceDir;
     else
@@ -684,13 +750,9 @@ bool tPath::Open    ( std::ifstream& f,
     return false;
 }
 
-static bool st_protectFiles = true;
-tSettingItem<bool> st_protectFilesConf("PROTECT_SENSITIVE_FILES", st_protectFiles);
-
 bool tPath::Open    ( std::ofstream& f,
                       const char* filename,
-                      std::ios::openmode mode,
-                      bool sensitive) const
+                      std::ios::openmode mode ) const
 {
     if ( !tPath::IsValidPath( filename ) )
         return false;
@@ -700,25 +762,21 @@ bool tPath::Open    ( std::ofstream& f,
 
     tString fullname = GetWritePath(filename);
 
-#ifndef WIN32
-    mode_t oldmask=0;
-    if(sensitive && st_protectFiles)
-    {
-        oldmask = umask(0600);
-    }
-#endif
     f.open( fullname, mode );
-#ifndef WIN32
-    if(sensitive && st_protectFiles)
-    {
-        chmod( &fullname(0), 0600 );
-        umask(oldmask);
-    }
-#endif
 
     return ( f && f.good() );
 }
 
+bool tPath::Open(std::fstream& f, const char* filename) const
+{
+    //	std::cout << "open\n";
+
+    tString fullname = GetWritePath(filename);
+    f.open(fullname, std::ios::in | std::ios::out);
+
+    return ( f.good() );
+}
+
 tString tPath::GetReadPath   ( const char* filename   ) const
 {
     if ( !tPath::IsValidPath( filename ) )
@@ -789,6 +847,11 @@ const tPath& tDirectories::Data()
     return st_Data;
 }
 
+const tPath& tDirectories::Music()
+{
+    return st_Music;
+}
+
 const tPath& tDirectories::Config()
 {
     return st_Config;
@@ -931,7 +994,6 @@ bool tDirectories::FileMatchesWildcard(const char *str, const char *pattern,
             if (*pattern)
                 c = *pattern++;
 
-            // fallthrough, we're just unescaping
         default:
             if (ignoreCase)
             {
@@ -1242,7 +1304,7 @@ static tString GetParent( char const * child, int levels )
 
     // strip last two path segments
     int toStrip = levels;
-    int stripCurrent = ret.Len()-1;
+    int stripCurrent = ret.Size()-1;
     while (stripCurrent >= 0 && toStrip > 0)
     {
         char & c = ret[ stripCurrent ];
@@ -1278,15 +1340,13 @@ public:
         char const * bestGuess = "./" PROGNAME;
 #endif// win32
 
-#ifdef ENABLE_BINRELOC
-	if ( !bestGuess || 0 == strlen(bestGuess) )
+#ifndef ENABLE_BINRELOC
+        // if the passed default path is a real path, let it override the best guess
+        if ( strstr( defaultPath, "/" ) || strstr( defaultPath, "\\" ) )
+            bestGuess = defaultPath;
+        //            bestGuess = "./armagetronad-dedicated";
 #endif
-        {
-            // if the passed default path is a real path, let it override the best guess
-            if ( strstr( defaultPath, "/" ) || strstr( defaultPath, "\\" ) )
-                bestGuess = defaultPath;
-            //            bestGuess = "./armagetronad-dedicated";
-        }
+
         path_ = bestGuess;
 
 #ifdef DEBUG_PATH
@@ -1326,8 +1386,6 @@ static tString GeneratePrefix()
     tString const & bindirCompiled = st_bindirCompiled;
     // and the current binary path
     tString bindirNow(GenerateParentOfExecutable(1));
-    if(bindirNow.Len() <= 1)
-        return prefixCompiled;
 
     // the length of the bindir suffix, the part that is added below prefix
     int bindirSuffixLength=bindirCompiled.Len() - prefixCompiled.Len();
@@ -1392,6 +1450,7 @@ static tString AddPrefix( const char * suffix )
 }
 */
 
+#ifndef WIN32
 static tString st_RelocatePath( tString const & original )
 {
     // fetch prefix as it was compiled in
@@ -1411,6 +1470,7 @@ static tString st_RelocatePath( tString const & original )
         return original;
     }
 }
+#endif
 
 // tries to find the path to the data files, given the location of the executable
 static void FindDataPath()
@@ -1467,7 +1527,7 @@ static bool ReadDir( tCommandLineParser & parser, tString & target, const char*
 class tDirectoriesCommandLineAnalyzer: public tCommandLineAnalyzer
 {
 private:
-    void DoInitialize( tCommandLineParser & parser ) override
+    virtual void DoInitialize( tCommandLineParser & parser )
     {
         // Puts the data files in the executable's bundle
 #ifndef MACOSX_XCODE
@@ -1490,16 +1550,13 @@ private:
                 return;
             }
         }
-#endif
+#endif // !MACOSX_XCODE
 
 
     }
 
-    bool DoAnalyze( tCommandLineParser & parser, int pass ) override
+    virtual bool DoAnalyze( tCommandLineParser & parser )
     {
-        if(pass > 0)
-            return false;
-
         if( ReadDir( parser, st_DataDir, "--datadir" ) ) return true;
         if( ReadDir( parser, st_UserDataDir, "--userdatadir" ) ) return true;
         if( ReadDir( parser, st_ConfigDir, "--configdir" ) ) return true;
@@ -1536,7 +1593,7 @@ private:
         return false;
     }
 
-    void DoHelp( std::ostream & s ) override
+    virtual void DoHelp( std::ostream & s )
     {                                      //
         s << "--datadir <Directory>        : read game data (textures, sounds and texts)\n"
         <<   "                               from this directory\n";
@@ -1556,35 +1613,3 @@ private:
 };
 
 static tDirectoriesCommandLineAnalyzer analyzer;
-
-tString tPath::GetPaths(void) const {
-    tString ret;
-    tArray<tString> paths;
-    Paths(paths);
-    for (int i = 0; i < paths.Len(); ++i) {
-        if(i > 0 && paths[i - 1] == paths[i]) continue;
-        ret << " - " << paths[i] << "\n";
-    }
-    return ret;
-}
-
-tString tPath::GetPaths(char const * delimiter, char const * finalizer) const {
-    tString ret;
-    tArray<tString> paths;
-    Paths(paths);
-    for (int i = 0; i < paths.Len(); ++i) {
-        ret << paths[i];
-        ret << ( (i == paths.Len() - 1) ? finalizer : delimiter );
-    }
-    return ret;
-}
-
-void st_PrintPathInfo(tOutput &buf) {
-    tString const hcol("0xff8888");
-    buf << hcol << "$path_info_user_cfg"   << "0xRESETT\n   " << tDirectories::Var().GetReadPath("user.cfg") << "\n"
-    << hcol << "$path_info_config"     << "0xRESETT\n" << tDirectories::Config().GetPaths()
-    << hcol << "$path_info_resource"   << "0xRESETT\n" << tDirectories::Resource().GetPaths()
-    << hcol << "$path_info_data"       << "0xRESETT\n" << tDirectories::Data().GetPaths()
-    << hcol << "$path_info_screenshot" << "0xRESETT\n" << tDirectories::Screenshot().GetPaths()
-    << hcol << "$path_info_var"        << "0xRESETT\n" << tDirectories::Var().GetPaths();
-}
