@@ -35,8 +35,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <string>
 #include <map>
 
-const tString st_internalEncoding("latin1");
-
 class tLocaleSubItem; // identifies a single string in a single language
 
 static tArray<tString> st_TemplateParameters;
@@ -417,12 +415,12 @@ void tLocaleItem::Load(const char *file, bool complete)  // load the language de
                 pre.ReadLine(s);
                 r->translation.Clear();
 
-                for (int i=0; i< pre.Len(); i++)
+                for (size_t i=0; i< pre.Size(); i++)
                 {
                     char c = pre(i);
                     if (c != '\\')
                         r->translation += c;
-                    else if (i < pre.Len()-1)
+                    else if (i < pre.Size())
                         switch (pre(i+1))
                         {
                         case 'n':
@@ -596,18 +594,7 @@ void tOutput::AddLocale(const char *x)
 {
     tNEW(tOutputItemLocale)(*this, tLocale::Find(x));
 }
-void tOutput::AddString(char const * locale)
-{
-    int len = strlen(locale);
-    if (len == 0)
-        return;
-    if (len == 1 && locale[0] == ' ')
-        tNEW(tOutputItemSpace)(*this);
-    else if (locale[0] == '$')
-        tNEW(tOutputItemLocale)(*this, tLocale::Find(locale+1));
-    else
-        tNEW(tOutputItem<tString>)(*this, tString(locale));
-}
+
 
 tOutput & tOutput::SetTemplateParameter(int num, const char *parameter)
 {
@@ -634,6 +621,12 @@ tOutput & tOutput::SetTemplateParameter(int num, float parameter)
 }
 
 
+tOutput::tOutput(const std::string & x)
+        :anchor(NULL)
+{
+    tNEW(tOutputItem<tString>)(*this, x);
+}
+
 tOutput::tOutput(const tString& x)
         :anchor(NULL)
 {
@@ -744,7 +737,16 @@ tOutput& operator << (tOutput &o, char *locale)
 
 // and a special implementation for the locales and strings:
 tOutput& operator << (tOutput &o, const char *locale){
-    o.AddString(locale);
+    int len = strlen(locale);
+    if (len == 0)
+        return o;
+    if (len == 1 && locale[0] == ' ')
+        tNEW(tOutputItemSpace)(o);
+    else if (locale[0] == '$')
+        tNEW(tOutputItemLocale)(o, tLocale::Find(locale+1));
+    else
+        tNEW(tOutputItem<tString>)(o, tString(locale));
+
     return o;
 }
 
