@@ -86,59 +86,6 @@ static CRITICAL_SECTION  mutex;
 
 static bool reported=false;
 
-#ifdef HAVE_LIBZTHREAD
-#include <zthread/FastRecursiveMutex.h>
-
-typedef ZThread::FastRecursiveMutex MUTEX;
-#elif defined(HAVE_PTHREAD)
-#include "pthread-binding.h"
-typedef tPThreadRecursiveMutex MUTEX;
-#else
-class tMockMutex
-{
-public:
-    void acquire(){}
-    void release(){}
-};
-
-typedef tMockMutex MUTEX;
-#endif
-
-static bool inited=true;
-
-static MUTEX & st_CreateMutex()
-{
-    inited = false;
-    static MUTEX newMutex;
-    inited = true;
-
-#ifdef WIN32
-    InitializeCriticalSection(&mutex);
-#endif
-
-    return newMutex;
-}
-
-static MUTEX & st_Mutex()
-{
-    static MUTEX & mutex = st_CreateMutex();
-    return mutex;
-}
-
-class tBottleNeck
-{
-public:
-    tBottleNeck()
-    {
-        st_Mutex().acquire();
-    }
-
-    ~tBottleNeck()
-    {
-        st_Mutex().release();
-    }
-};
-
 // replacement for wmemset function
 #ifndef HAVE_WMEMSET
 static inline wchar_t *wmemset(wchar_t *wcs, wchar_t wc, size_t n) throw()
@@ -164,7 +111,7 @@ void leak(){
         std::cerr << "\n\nMemory leak detected!\n\n";
 #endif
 #endif
-        tMemManBase::Profile();
+        tMemMan::Profile();
     }
 }
 
@@ -181,38 +128,6 @@ class memblock;
 #endif
 #define PAD 197
 
-// information about allocation in process
-struct tAllocationInfo
-{
-#ifdef LEAKFINDER
-    const char *filename;
-    const char *classname;
-    int checksum;
-    int line;
-    bool array;
-#endif
-
-    tAllocationInfo( bool 
-#ifdef LEAKFINDER
-                     array_ 
-#endif
-        )
-#ifdef LEAKFINDER
-    : filename( "XXX" ), classname( "XXX" ), checksum(-1), line(0), array( array_ )
-#endif
-    {
-    }
-};
-
-class tMemMan: public tMemManBase
-{
-public:
-    static void *Alloc(tAllocationInfo const & info, size_t s);
-    static void  Dispose(tAllocationInfo const & info, void *p);
-    static void  DisposeButKeep(tAllocationInfo const & info, void *p);
-};
-
-
 class tMemManager{
     friend class memblock;
 public:
@@ -220,21 +135,13 @@ public:
     tMemManager(int size);
     ~tMemManager();
 
-    void *      Alloc( tAllocationInfo const & info );
-    static void * AllocDefault(tAllocationInfo const & info, size_t size);
-    static void Dispose(tAllocationInfo const & info, void *p, bool keep=false);
+    void *      Alloc();
+    static void * AllocDefault(size_t size);
+    static void Dispose(void *p, bool keep=false);
     void        complete_Dispose(memblock *m);
     void        Check(); // check integrity
 
-    tMemManager( tMemManager const & other )
-    : size( other.size )
-    , blocksize( other.blocksize )
-    , semaphore( 1 )
-    {
-    }
 private:
-    tMemManager & operator =( tMemManager const & );
-
     int  Lower(int i){ // the element below i in the heap
         if(i%2==0)  // just to make sure what happens; you never know what 1/2 is..
             return i/2-1;
@@ -253,7 +160,7 @@ private:
     void CheckHeap(); // checks the heap structure
     //#endif
 
-    static int  UpperL(int i){return 2*i+1;} // the elements left and
+static int  UpperL(int i){return 2*i+1;} // the elements left and
     static int  UpperR(int i){return 2*i+2;} // right above i
 
     void Insert(memblock *b);  // starts to manage object e
@@ -273,13 +180,16 @@ private:
     int semaphore;
 };
 
+static bool inited=false;
+
 #ifdef LEAKFINDER
 #include <fstream>
 #define MAXCHECKSUM 100001
+static int checksum=-1;
 static int counter[MAXCHECKSUM];
 static int leaks[MAXCHECKSUM];
 #ifdef PROFILER
-static const char *checksum_filename[MAXCHECKSUM];
+static const char *checksum_fileName[MAXCHECKSUM];
 static const char *checksum_classname[MAXCHECKSUM];
 static int         checksum_line[MAXCHECKSUM];
 static int         checksum_blocks[MAXCHECKSUM];
@@ -287,8 +197,12 @@ static int         checksum_bytes[MAXCHECKSUM];
 #endif
 
 
-static char const *leakname="leak.log";
+static char *leakname="leak.log";
 static bool checkleaks=true;
+static const char *fileName="XXX";
+static const char *classname="XXX";
+static int line;
+static bool array;
 #endif
 
 void begin_checkleaks(){
@@ -389,7 +303,7 @@ public: // everything is local to this file anyway...
     }
 
 
-    void *Alloc( tAllocationInfo const & info ){
+    void *Alloc(){
 #ifdef MEM_DEB_SMALL
         Check();
 #endif
@@ -424,25 +338,25 @@ public: // everything is local to this file anyway...
         //   tASSERT(_CrtCheckMemory());
 
 #ifdef LEAKFINDER
-        if (info.checksum >= 0){
-            tASSERT(info.checksum < MAXCHECKSUM);
+        if (checksum >= 0){
+            tASSERT(checksum < MAXCHECKSUM);
 
 #ifdef PROFILER
-            if (!counter[info.checksum]){
-                checksum_filename [info.checksum] = info.filename;
-                checksum_classname[info.checksum] = info.classname;
-                checksum_line     [info.checksum] = info.line;
+            if (!counter[checksum]){
+                checksum_fileName [checksum] = fileName;
+                checksum_classname[checksum] = classname;
+                checksum_line     [checksum] = line;
             }
-            checksum_blocks[info.checksum]++;
-            checksum_bytes[info.checksum]+=size;
+            checksum_blocks[checksum]++;
+            checksum_bytes[checksum]+=size;
             chunk(ret).realSize = size;
 #endif
-            chunk(ret).array=info.array;
-            chunk(ret).checksum=info.checksum;
-            chunk(ret).counter =++counter[info.checksum];
+            chunk(ret).array=array;
+            chunk(ret).checksum=checksum;
+            chunk(ret).counter =++counter[checksum];
 
             static int count =0;
-            if (checkleaks && counter[info.checksum] == leaks[info.checksum]){
+            if (checkleaks && counter[checksum] == leaks[checksum]){
                 count ++;
                 if (count <= 0)
                 {
@@ -467,14 +381,6 @@ public: // everything is local to this file anyway...
         }
 #endif
 
-#ifdef LEAKFINDER
-#ifdef PROFILER
-        chunk(ret).realSize = size;
-#endif
-        chunk(ret).array=info.array;
-        chunk(ret).checksum=info.checksum;
-#endif
-
 #ifdef DEBUG
         wmemset(static_cast< wchar_t * >( data(ret) ), wchar_t(0xfedabeef), size / sizeof( wchar_t ));
 #else
@@ -491,7 +397,7 @@ public: // everything is local to this file anyway...
         return NULL;
     }
 
-    static memblock * Dispose(void *p, int &size, tAllocationInfo const & info, bool keep=false){
+    static memblock * Dispose(void *p, int &size,bool keep=false){
         chunkinfo &c=((chunkinfo *)p)[-1];
         size=c.size_in_dwords*4;
 
@@ -507,7 +413,7 @@ public: // everything is local to this file anyway...
 #endif
 
 #ifndef WIN32 // grr. on windows, this assertion fails on calls from STL.
-        tASSERT( c.array == info.array );
+        tASSERT( c.array == array );
 #endif
 
         if (size==0){
@@ -596,7 +502,7 @@ public: // everything is local to this file anyway...
 #ifdef LEAKFINDER
     void dumpleaks(std::ostream &s){
         for (int i=myman->blocksize-1;i>=0;i--)
-            if (chunk(i).occupied && chunk(i).checksum >= 0 ){
+            if (chunk(i).occupied){
                 s << chunk(i).checksum << " " << chunk(i).counter << '\n';
                 leak();
             }
@@ -631,11 +537,6 @@ bool tMemManager::SwapIf(int i,int j){
 
 tMemManager::~tMemManager(){
 #ifdef LEAKFINDER
-    bool warn = true;
-#ifdef HAVE_LIBZTHREAD
-    warn = false;
-#endif
-
     if (inited){
         // l???sche das ding
         std::ofstream lw(leakname);
@@ -658,10 +559,10 @@ tMemManager::~tMemManager(){
             memblock::destroy(blocks(i));
         else{
 #ifdef LEAKFINDER
+            static bool warn = true;
             if ( warn )
             {
                 std::cout << "Memmanager warning: leaving block untouched.\n";
-                warn = false;
             }
             std::ofstream l(leakname,std::ios::app);
             blocks(i)->dumpleaks(l);
@@ -674,11 +575,7 @@ tMemManager::~tMemManager(){
             memblock::destroy(full_blocks(i));
         else{
 #ifdef LEAKFINDER
-            if ( warn )
-            {
-                std::cout << "Memmanager warning: leaving block untouched.\n";
-                warn = false;
-            }
+            std::cout << "Memmanager warning: leaving block untouched.\n";
             std::ofstream l(leakname,std::ios::app);
             full_blocks(i)->dumpleaks(l);
 #endif
@@ -693,14 +590,15 @@ tMemManager::tMemManager(int s,int bs):size(s),blocksize(s){
         blocksize=255;
     semaphore = 1;
 
-    tBottleNeck neck;
+#ifdef WIN32
+    if (!inited)
+        InitializeCriticalSection(&mutex);
+#endif
 
     inited = true;
 }
 
 tMemManager::tMemManager(int s):size(s){//,blocks(1000),full_blocks(1000){
-    tBottleNeck neck;
-
     inited = false;
 
     blocks.SetLen(0);
@@ -718,7 +616,7 @@ tMemManager::tMemManager(int s):size(s){//,blocks(1000),full_blocks(1000){
             leaks[i] = 0;
             counter[i] = 0;
 #ifdef PROFILER
-            checksum_filename[i] = 0;
+            checksum_fileName[i] = 0;
             checksum_classname[i] = 0;
             checksum_line[i] = 0;
             checksum_blocks[i] = 0;
@@ -728,7 +626,7 @@ tMemManager::tMemManager(int s):size(s){//,blocks(1000),full_blocks(1000){
 
         std::ifstream l(leakname);
         while (l.good() && !l.eof()){
-            int cs{-1},ln{0};
+            int cs,ln;
             l >> cs >> ln;
 
             if (cs>=0 && cs < MAXCHECKSUM && (ln < leaks[cs] || leaks[cs] == 0))
@@ -742,6 +640,11 @@ tMemManager::tMemManager(int s):size(s){//,blocks(1000),full_blocks(1000){
     }
 #endif
 
+#ifdef WIN32
+    if (!inited)
+        InitializeCriticalSection(&mutex);
+#endif
+
     inited = true;
 }
 
@@ -875,11 +778,11 @@ memblock * tMemManager::Remove(int i){
     return NULL;
 }
 
-void * tMemManager::Alloc( tAllocationInfo const & info ){
+void * tMemManager::Alloc(){
     semaphore--;
     if (semaphore<0 || size == 0){
         semaphore++;
-        return AllocDefault(info, size);
+        return AllocDefault(size);
     }
 
 #ifdef WIN32
@@ -892,7 +795,7 @@ void * tMemManager::Alloc( tAllocationInfo const & info ){
 
     memblock *block=blocks(0);
     tASSERT (block->size == size);
-    void *mem = block->Alloc( info );
+    void *mem = block->Alloc();
     //tASSERT(_CrtCheckMemory());
     if (block->value == 0){
         Remove(block);
@@ -960,9 +863,7 @@ void tMemManager::Check(){
 #define MAX_SIZE 109
 
 
-static tMemManager & st_MemMan(int id)
-{
-    static tMemManager memman[MAX_SIZE+1]={
+static tMemManager memman[MAX_SIZE+1]={
                                           tMemManager(0),
                                           tMemManager(4),
                                           tMemManager(8),
@@ -1073,25 +974,19 @@ static tMemManager & st_MemMan(int id)
                                           tMemManager(428),
                                           tMemManager(432),
                                           tMemManager(436)
-    };
-
-    tASSERT(id >= 0 && id <= MAX_SIZE);
-    return memman[id];
-}
+                                      };
 
 
-
-void tMemManager::Dispose(tAllocationInfo const & info, void *p, bool keep){
+void tMemManager::Dispose(void *p,bool keep){
     int size;
 
     if (!p)
         return;
 
-    memblock *block = memblock::Dispose(p,size,info, keep);
+    memblock *block = memblock::Dispose(p,size,keep);
 #ifndef DOUBLEFREEFINDER
     if (inited && block){
-        tBottleNeck neck;
-        st_MemMan(size >> 2).complete_Dispose(block);
+        memman[size >> 2].complete_Dispose(block);
 #ifdef WIN32
         LeaveCriticalSection(&mutex);
 #endif
@@ -1101,23 +996,20 @@ void tMemManager::Dispose(tAllocationInfo const & info, void *p, bool keep){
 #endif
 }
 
-void *tMemManager::AllocDefault(tAllocationInfo const & info, size_t s){
+void *tMemManager::AllocDefault(size_t s){
 #ifdef LEAKFINDER
     void *ret=malloc(s+sizeof(chunkinfo));//,classname,fileName,line);
-    ((chunkinfo *)ret)->checksum=info.checksum;
-    ((chunkinfo *)ret)->array=info.array;
+    ((chunkinfo *)ret)->checksum=checksum;
+    ((chunkinfo *)ret)->array=array;
 #ifdef PROFILER
     ((chunkinfo *)ret)->realSize=s;
-    if ( info.checksum >= 0 )
-    {
-        if (!counter[info.checksum]){
-            checksum_filename [info.checksum] = info.filename;
-            checksum_classname[info.checksum] = info.classname;
-            checksum_line     [info.checksum] = info.line;
-        }
-        checksum_blocks[info.checksum]++;
-        checksum_bytes[info.checksum]+=s;
+    if (!counter[checksum]){
+        checksum_fileName [checksum] = fileName;
+        checksum_classname[checksum] = classname;
+        checksum_line     [checksum] = line;
     }
+    checksum_blocks[checksum]++;
+    checksum_bytes[checksum]+=s;
 #endif
 #else
     void *ret=malloc(s+sizeof(chunkinfo));
@@ -1128,7 +1020,7 @@ void *tMemManager::AllocDefault(tAllocationInfo const & info, size_t s){
     return ret;
 }
 
-void *tMemMan::Alloc(tAllocationInfo const & info, size_t s){
+void *tMemMan::Alloc(size_t s){
 #ifndef DONTUSEMEMMANAGER
 #ifdef MEM_DEB
 #ifdef WIN32
@@ -1138,13 +1030,9 @@ void *tMemMan::Alloc(tAllocationInfo const & info, size_t s){
 #endif
     void *ret;
     if (inited && s < (MAX_SIZE << 2))
-    {
-        tBottleNeck neck;
-        ret=st_MemMan(((s+3)>>2)).Alloc( info );
-    }
-    else
-    {
-        ret=tMemManager::AllocDefault(info, s);
+        ret=memman[((s+3)>>2)].Alloc();
+    else{
+        ret=tMemManager::AllocDefault(s);
     }
 #ifdef MEM_DEB
     Check();
@@ -1160,7 +1048,7 @@ void *tMemMan::Alloc(tAllocationInfo const & info, size_t s){
 
 }
 
-void tMemMan::DisposeButKeep( tAllocationInfo const & info, void *p){
+void tMemMan::DisposeButKeep(void *p){
 #ifndef DONTUSEMEMMANAGER
 #ifdef MEM_DEB
 #ifdef WIN32
@@ -1168,7 +1056,7 @@ void tMemMan::DisposeButKeep( tAllocationInfo const & info, void *p){
 #endif
     Check();
 #endif
-    tMemManager::Dispose(info, p, true);
+    tMemManager::Dispose(p,true);
 #ifdef MEM_DEB
     Check();
 #ifdef WIN32
@@ -1178,7 +1066,7 @@ void tMemMan::DisposeButKeep( tAllocationInfo const & info, void *p){
 #endif
 }
 
-void tMemMan::Dispose(tAllocationInfo const & info, void *p){
+void tMemMan::Dispose(void *p){
 #ifndef DONTUSEMEMMANAGER
 #ifdef MEM_DEB
 #ifdef WIN32
@@ -1186,7 +1074,7 @@ void tMemMan::Dispose(tAllocationInfo const & info, void *p){
 #endif
     Check();
 #endif
-    tMemManager::Dispose(info, p );
+    tMemManager::Dispose(p);
 #ifdef MEM_DEB
     Check();
 #ifdef WIN32
@@ -1199,23 +1087,19 @@ void tMemMan::Dispose(tAllocationInfo const & info, void *p){
 }
 
 
-void tMemManBase::Check(){
-#ifndef DONTUSEMEMMANAGER
+void tMemMan::Check(){
     if (!inited)
         return;
-
 #ifdef WIN32
     EnterCriticalSection(&mutex);
 #endif
 
     for (int i=MAX_SIZE;i>=0;i--)
-        st_MemMan(i).Check();
+        memman[i].Check();
 
 #ifdef WIN32
     LeaveCriticalSection(&mutex);
 #endif
-
-#endif // DONTUSEMEMMANAGER
 }
 
 /*
@@ -1265,6 +1149,7 @@ void * operator new(
 #endif 
         line * 19671;
 
+    checksum = checksum % MAXCHECKSUM;
     int c=1379;
     while (*file){
         checksum = (checksum + (*file)*c) % MAXCHECKSUM;
@@ -1289,135 +1174,143 @@ void * operator new(
 
 
 void* _cdecl operator new	(size_t size) THROW_BADALLOC{
-    tAllocationInfo info( false );
-
 #ifdef LEAKFINDER
-#ifndef HAVE_LIBZTHREAD
-    info.checksum = size;
+    checksum = size % MAXCHECKSUM;
+    array = false;
 #endif
-#endif
-    return tMemMan::Alloc(info, size);
+    return tMemMan::Alloc(size);
 }
 
 void  _cdecl operator delete(void *ptr) THROW_NOTHING{
-    tAllocationInfo info( false );
-
+#ifdef LEAKFINDER
+    checksum = -1;
+    array = false;
+#endif
     if (ptr)
-        tMemMan::Dispose(info, ptr);
+    tMemMan::Dispose(ptr);
 }
 
 void  operator delete(void *ptr,bool keep) THROW_NOTHING{
-    tAllocationInfo info( false );
-
-    if (ptr){
+#ifdef LEAKFINDER
+        checksum = -1;
+        array = false;
+#endif
+        if (ptr){
         if (keep)
-            tMemMan::DisposeButKeep(info, ptr);
-        else
-            tMemMan::Dispose(info, ptr);
-        
-    }
+                tMemMan::DisposeButKeep(ptr);
+            else
+                tMemMan::Dispose(ptr);
+
+        }
 }
 
 
 
 void* operator new	(size_t size,const char *classn,const char *file,int l) THROW_BADALLOC{
-    tAllocationInfo info( false );
 #ifdef LEAKFINDER
-    info.filename=file;
-    info.classname=classn;
-    info.line=l;
+    fileName=file;
+    classname=classn;
+    line=l;
+    array = false;
 
-    info.checksum =
+    checksum =
 #ifndef PROFILER 
         size +
 #endif 
-        info.line * 19671;
+        line * 19671;
+
+    checksum = checksum % MAXCHECKSUM;
 
     int c=1379;
     while (*file){
-        info.checksum = (info.checksum + (*file)*c) % MAXCHECKSUM;
-        c             = (c * 79              ) % MAXCHECKSUM;
+    checksum = (checksum + (*file)*c) % MAXCHECKSUM;
+        c        = (c * 79              ) % MAXCHECKSUM;
         file++;
     }
     while (*classn){
-        info.checksum = (info.checksum + (*classn)*c) % MAXCHECKSUM;
-        c             = (c * 79                ) % MAXCHECKSUM;
+    checksum = (checksum + (*classn)*c) % MAXCHECKSUM;
+        c        = (c * 79                ) % MAXCHECKSUM;
         classn++;
     }
 
 #ifdef PROFILER
-    while (checksum_filename[info.checksum] && (checksum_filename[info.checksum] != info.filename || checksum_line[info.checksum] != info.line))
-    info.checksum = (info.checksum+1) % MAXCHECKSUM;
+    while (checksum_fileName[checksum] && (checksum_fileName[checksum] != fileName || checksum_line[checksum] != line))
+    checksum = (checksum+1) % MAXCHECKSUM;
 #endif
 
 #endif
-    return tMemMan::Alloc(info, size);
+    return tMemMan::Alloc(size);
 }
 
 void  operator delete   (void *ptr,const char *classname,const char *file,int line)  THROW_NOTHING{
-    tAllocationInfo info( false );
-
-    if (ptr) tMemMan::Dispose(info, ptr);
-}
-
-void* operator new[]	(size_t size) THROW_BADALLOC{
-    tAllocationInfo info( true );
 #ifdef LEAKFINDER
-#ifndef HAVE_LIBZTHREAD
-    info.checksum = size % MAXCHECKSUM;
+        checksum = 0;
+        array = false;
 #endif
-#endif
-    return tMemMan::Alloc(info, size);
-}
-
-void  operator delete[](void *ptr) THROW_NOTHING {
-    tAllocationInfo info( true );
-
-    if (ptr)
-        tMemMan::Dispose(info, ptr);
-}
-
-
-
-void* operator new[]	(size_t size,const char *classn,const char *file,int l)  THROW_BADALLOC{
-    tAllocationInfo info( true );
-#ifdef LEAKFINDER
-    info.filename=file;
-    info.classname=classn;
-    info.line=l;
-
-    info.checksum =
-#ifndef PROFILER 
-    size +
-#endif 
-    info.line * 19671;
-    
-    int c=1379;
-    while (*file){
-        info.checksum = (info.checksum + (*file)*c) % MAXCHECKSUM;
-        c             = (c * 79              ) % MAXCHECKSUM;
-        file++;
+        if (ptr) tMemMan::Dispose(ptr);
     }
+    void* operator new[]	(size_t size) THROW_BADALLOC{
+#ifdef LEAKFINDER
+            checksum = size % MAXCHECKSUM;
+            array = true;
+#endif
+            return tMemMan::Alloc(size);
+        }
+
+        void  operator delete[](void *ptr) THROW_NOTHING {
+#ifdef LEAKFINDER
+            checksum = -1;
+            array = true;
+#endif
+            if (ptr)
+            tMemMan::Dispose(ptr);
+        }
+
+
+
+        void* operator new[]	(size_t size,const char *classn,const char *file,int l)  THROW_BADALLOC{
+#ifdef LEAKFINDER
+                fileName=file;
+                classname=classn;
+                line=l;
+                array = true;
+
+                checksum =
+#ifndef PROFILER 
+                    size +
+#endif 
+                    line * 19671;
+
+                checksum = checksum % MAXCHECKSUM;
+
+                int c=1379;
+                while (*file){
+                checksum = (checksum + (*file)*c) % MAXCHECKSUM;
+                    c        = (c * 79              ) % MAXCHECKSUM;
+                    file++;
+                }
     while (*classn){
-        info.checksum = (info.checksum + (*classn)*c) % MAXCHECKSUM;
-        c             = (c * 79                ) % MAXCHECKSUM;
+    checksum = (checksum + (*classn)*c) % MAXCHECKSUM;
+        c        = (c * 79                ) % MAXCHECKSUM;
         classn++;
     }
-    
+
 #ifdef PROFILER
-    while (checksum_filename[info.checksum] && (checksum_filename[info.checksum] != info.filename || checksum_line[info.checksum] != info.line))
-    info.checksum = (info.checksum+1) % MAXCHECKSUM;
+    while (checksum_fileName[checksum] && (checksum_fileName[checksum] != fileName || checksum_line[checksum] != line))
+    checksum = (checksum+1) % MAXCHECKSUM;
 #endif
 
 #endif
-    return tMemMan::Alloc(info, size);
+    return tMemMan::Alloc(size);
 }
 
 void  operator delete[]   (void *ptr,const char *classname,const char *file,int line)   THROW_NOTHING{
-    tAllocationInfo info( true );
-
-    if (ptr) tMemMan::Dispose(info, ptr);
-}
+#ifdef LEAKFINDER
+        checksum = 0;
+        array = true;
+#endif
+        if (ptr) tMemMan::Dispose(ptr);
+    }
 
 #endif
 
@@ -1448,8 +1341,7 @@ void  operator delete[]   (void *ptr,const char *classname,const char *file,int
 #define snprintf _snprintf
 #endif
 
-void  tMemManBase::Profile(){
-    tBottleNeck neck;
+void  tMemMan::Profile(){
 #ifdef PROFILER
 
     int sort_checksum[MAXCHECKSUM];
@@ -1478,7 +1370,7 @@ void  tMemManBase::Profile(){
     for (i=size-1; i>=0; i--)
     {
         int cs = sort_checksum[i];
-        const char *fn = checksum_filename[cs];
+        const char *fn = checksum_fileName[cs];
         const char *cn = checksum_classname[cs];
 
         if (!fn)
