@@ -82,27 +82,6 @@ bool tRecorderBase::IsRunning( void )
     return IsRecording() || IsPlayingBack();
 }
 
-// *****************************************************************************************
-// *
-// *   Stop
-// *
-// *****************************************************************************************
-//!
-//!
-// *****************************************************************************************
-
-void tRecorderBase::StopRecording( void )
-{
-    return tRecording::Stop();
-}
-static void st_StopRecording(std::istream &)
-{
-    tRecorderBase::StopRecording();
-}
-
-static tConfItemFunc snm("STOP_RECORDING",&st_StopRecording);
-
-
 // *******************************************************************************************
 // *
 // *    Record
@@ -154,42 +133,6 @@ bool tRecorder::PlaybackStrict( char const * section )
     return tRecorderTemplate1< tPlaybackBlock >::Archive( true, section );
 }
 
-static bool st_desyncedPlayback{};
-static bool st_probablyDesyncedPlayback{};
-
-bool tRecorder::DesyncedPlayback()
-{
-    return st_desyncedPlayback;
-}
-
-void tRecorder::ActivateDesyncedPlayback()
-{
-    if(!st_desyncedPlayback)
-    {
-#ifndef DEBUG
-        if(!st_probablyDesyncedPlayback)
-#endif
-        {
-            tERR_WARN("Playback no longer in sync.");
-        }
-        constexpr auto desyncTag = "DESYNC_DETECTED";
-        tRecorder::Record(desyncTag);
-        tRecorder::Playback(desyncTag);
-
-        st_desyncedPlayback = true;
-    }
-}
-
-bool tRecorder::ProbablyDesyncedPlayback()
-{
-    return st_probablyDesyncedPlayback;
-}
-
-void tRecorder::ActivateProbablyDesyncedPlayback()
-{
-    st_probablyDesyncedPlayback = true;
-}
-
 // *****************************************************************************************
 // *****************************************************************************************
 // *****************************************************************************************
@@ -275,10 +218,10 @@ std::istream & operator >> ( std::istream & s, tLineString & line )
     line.Clear();
 
     // copy line, replacing "\n" with real newline
-    for(int i=0; i<read.Len()-1; ++i)
+    for(size_t i=0; i<read.Size(); ++i)
     {
         char c = read[i];
-        if ( c != '\\' || i+1 == read.Len() || ( read[i+1] != 'n' && read[i+1] != '\\' ) )
+        if ( c != '\\' || i+1 == read.Size() || ( read[i+1] != 'n' && read[i+1] != '\\' ) )
         {
             line << c;
         }
@@ -314,7 +257,7 @@ std::istream & operator >> ( std::istream & s, tLineString & line )
 //!
 // *****************************************************************************************
 
-bool tRecordingBlockBase::Initialize( char const * section, tRecording * recording, bool )
+bool tRecordingBlockBase::Initialize( char const * section, tRecording * recording )
 {
     // initialize recording pointer
     recording_ = recording;
@@ -339,10 +282,10 @@ bool tRecordingBlockBase::Initialize( char const * section, tRecording * recordi
 //!
 // *****************************************************************************************
 
-bool tRecordingBlockBase::Initialize( char const * section, bool skipToIt )
+bool tRecordingBlockBase::Initialize( char const * section )
 {
     // delegate
-    return Initialize( section, tRecording::currentRecording_, skipToIt );
+    return Initialize( section, tRecording::currentRecording_ );
 }
 
 // *****************************************************************************************
@@ -467,28 +410,18 @@ tRecordingBlock::~tRecordingBlock( void )
 //!
 // *****************************************************************************************
 
-bool tPlaybackBlockBase::Initialize( char const * section, tPlayback * playback, bool skipToIt )
+bool tPlaybackBlockBase::Initialize( char const * section, tPlayback * playback )
 {
     // initialize playback pointer
     playback_ = playback;
     if (!playback_)
         return false;
 
-    // std::cout << playback_->GetNextSection() << "," << section << "\n";
-
     // read section
-    if(skipToIt)
+    if( playback_->GetNextSection() != section )
     {
-        while(playback->GetNextSection() != section && playback->GetNextSection() != "EOF")
-            playback->AdvanceSection(true);
-    }
-    else
-    {
-        if( playback_->GetNextSection() != section )
-        {
-            playback_ = NULL;
-            return false;
-        }
+        playback_ = NULL;
+        return false;
     }
 
     // return success
@@ -506,9 +439,9 @@ bool tPlaybackBlockBase::Initialize( char const * section, tPlayback * playback,
 //!
 // *****************************************************************************************
 
-bool tPlaybackBlockBase::Initialize( char const * section, bool skipToIt )
+bool tPlaybackBlockBase::Initialize( char const * section )
 {
-    return Initialize( section, tPlayback::currentPlayback_, skipToIt );
+    return Initialize( section, tPlayback::currentPlayback_ );
 }
 
 // *****************************************************************************************
@@ -628,7 +561,7 @@ static int st_GetDebugLevelPlayback()
     // sync level with recording
     int level = st_debugLevelRecording;
     tRecorder::Playback( "DEBUGLEVEL", level );
-    tRecorder::Record( "DEBUGLEVEL", level );
+    tRecorder::Record( "DEBUGLEVEL", st_debugLevelRecording );
 
     return level;
 }
@@ -663,9 +596,7 @@ int tRecorderSyncBase::GetDebugLevelPlayback( void )
 
 int tRecorderSyncBase::GetDebugLevelRecording( void )
 {
-    // delegate so this doesn't change when the config changes
-    return GetDebugLevelPlayback();
-    // return st_debugLevelRecording;
+    return st_debugLevelRecording;
 }
 
 REAL st_GetDifference( REAL a, REAL b)
