@@ -28,18 +28,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef     TRECORDER_H_INCLUDED
 #define     TRECORDER_H_INCLUDED
 
-#ifdef DEBUG
-#ifndef DEBUG_DIFFERENCE
-#define DEBUG_DIFFERENCE
-#endif
-#endif
-
 // self include
 #ifndef     TRECORDER_H_INCLUDED
 #include    "tRecorder.h"
 #endif
 
 #include    "tString.h"
+#include    "tError.h"
 
 // #include    "tRecorderInternal.h"
 
@@ -86,7 +81,6 @@ public:
     static bool IsRecording();                          //!< returns whether there is a recording running
     static bool IsPlayingBack();                        //!< returns whether there is a playback running
     static bool IsRunning();                            //!< returns whether recording or playback are running
-    static void StopRecording();                        //!< stops recording
 };
 
 //! simple interface to recording functionality
@@ -126,11 +120,6 @@ public:
     template< class DATA1, class DATA2 >
     static bool PlaybackStrict( char const * section, DATA1 & data1, DATA2 & data2 )
     { return tRecorderTemplate3< tPlaybackBlock, DATA1 &, DATA2 & >::Archive( true, section, data1, data2 ); }
-
-    static bool DesyncedPlayback();
-    static void ActivateDesyncedPlayback();
-    static bool ProbablyDesyncedPlayback();
-    static void ActivateProbablyDesyncedPlayback();
 };
 
 class tPath;
@@ -263,8 +252,8 @@ class tRecording;
 class tRecordingBlockBase
 {
 public:
-    bool Initialize( char const * section, tRecording * recording, bool skipToIt = false  );  //!< initializes this for recording
-    bool Initialize( char const * section, bool skipToIt = false );                           //!< initializes this for recording
+    bool Initialize( char const * section, tRecording * recording );  //!< initializes this for recording
+    bool Initialize( char const * section );                          //!< initializes this for recording
 
     void Separator();                                                 //!< separates two recorded elements (more than usual)
     static tRecording * GetArchive();                                 //!< returns the active recording
@@ -319,8 +308,8 @@ class tPlayback;
 class tPlaybackBlockBase
 {
 public:
-    bool Initialize( char const * section, tPlayback * playback, bool skipToIt = false  );    //!< initializes this for recording
-    bool Initialize( char const * section, bool skipToIt = false  );                          //!< initializes this for recording
+    bool Initialize( char const * section, tPlayback * playback );    //!< initializes this for recording
+    bool Initialize( char const * section );                          //!< initializes this for recording
 
     void Separator() const;                                           //!< separate output (Nothing done here while playing back)
     static tPlayback * GetArchive();                                  //!< returns the active playback
@@ -917,7 +906,7 @@ void tRecorderSync< DATA >::Archive( char const * section, int level, DATA & dat
         // read data from archive
         if ( tRecorder::PlaybackStrict( section, copy ) )
         {
-#ifdef DEBUG_DIFFERENCE
+#ifdef DEBUG
             // determine difference
             REAL diff = GetDifference( data, copy );
 
@@ -939,12 +928,6 @@ void tRecorderSync< DATA >::Archive( char const * section, int level, DATA & dat
                 data = copy;
             }
         }
-        else if ( tRecorder::IsPlayingBack() )
-        {
-            std::cout << "Syncing difference found: expected " << section << ".\n";
-
-            st_Breakpoint();
-        }
     }
 
     // archive data
@@ -1002,9 +985,7 @@ bool tRecorderTemplate1< BLOCK >::Archive( bool strict, char const * section )
     }
 
     // report failure
-    if(strict && BLOCK::GetArchive())
-        tRecorder::ActivateDesyncedPlayback();
-
+    tASSERT( !strict  || !BLOCK::GetArchive() );
     return false;
 }
 
@@ -1038,9 +1019,7 @@ bool tRecorderTemplate2< BLOCK, DATA >::Archive( bool strict, char const * secti
     }
 
     // report failure
-    if(strict && BLOCK::GetArchive())
-        tRecorder::ActivateDesyncedPlayback();
-
+    tASSERT( !strict || !BLOCK::GetArchive() );
     return false;
 }
 
@@ -1075,9 +1054,7 @@ bool tRecorderTemplate3< BLOCK, DATA1, DATA2 >::Archive( bool strict, char const
     }
 
     // report failure
-    if(strict && BLOCK::GetArchive())
-        tRecorder::ActivateDesyncedPlayback();
-
+    tASSERT( !strict  || !BLOCK::GetArchive() );
     return false;
 }
 
