@@ -40,7 +40,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include    "tCommandLine.h"
 #include    "tConsole.h"
-#include    "tDirectories.h"
+#include    "tError.h"
 
 #undef  INLINE_DEF
 #define INLINE_DEF
@@ -84,20 +84,6 @@ tRecording::tRecording()
     currentRecording_ = this;
 }
 
-// ******************************************************************************************
-// *
-// *    Stop
-// *
-// ******************************************************************************************
-//!
-//!
-// ******************************************************************************************
-
-void tRecording::Stop()
-{
-    currentRecording_ = NULL;
-}
-
 // ******************************************************************************************
 // *
 // * tRecording
@@ -268,15 +254,10 @@ void EatWhite( std::istream & stream )
 //!
 // ******************************************************************************************
 
-void tPlayback::AdvanceSection( bool skipLines )
+void tPlayback::AdvanceSection( void )
 {
     std::istream& stream = DoGetStream();
 
-    if(skipLines)
-    {
-        std::string line;
-        std::getline(stream, line);
-    }
     std::ws( stream );
     stream >> nextSection_;
     std::ws( stream );
@@ -300,54 +281,31 @@ void tPlayback::AdvanceSection( bool skipLines )
 class tRecordingCommandLineAnalyzer: public tCommandLineAnalyzer
 {
 private:
-    bool DoAnalyze( tCommandLineParser & parser, int pass ) override
+    virtual bool DoAnalyze( tCommandLineParser & parser )
     {
-        switch(pass)
+        tString filename;
+        if ( parser.GetOption( filename, "--record" ) )
         {
-            // pass 0: only accept strict options
-            case 0:
-            {
-                tString filename;
-                if ( parser.GetOption( filename, "--record" ) )
-                {
-                    // start a recorder
-                    static tRecorderImp< tRecording, std::ofstream > recorder( static_cast< char const * >( filename ) );
-                    return true;
-                }
-
-                if ( parser.GetOption( filename, "--playback" ) )
-                {
-                    // start a playback
-                    static tRecorderImp< tPlayback, std::ifstream > recorder( static_cast< char const * >( filename ) );
-                    recorder.InitPlayback();
-                    return true;
-                }
-            }
-            break;
-            // pass 1: anything may be a recording to play back
-            case 1:
-            {
-                auto *filename = parser.Current();
-                if(tDirectories::FileMatchesWildcard(filename, "*.aarec"))
-                {
-                    static tRecorderImp< tPlayback, std::ifstream > recorder( parser.Current() );
-                    recorder.InitPlayback();
-                    parser.Advance();
-                    return true;
-                }
-            }
-            break;
+            // start a recorder
+            static tRecorderImp< tRecording, std::ofstream > recorder( static_cast< char const * >( filename ) );
+            return true;
         }
 
+        if ( parser.GetOption( filename, "--playback" ) )
+        {
+            // start a playback
+            static tRecorderImp< tPlayback, std::ifstream > recorder( static_cast< char const * >( filename ) );
+            recorder.InitPlayback();
+            return true;
+        }
 
         return false;
     }
 
-    void DoHelp( std::ostream & s ) override
+    virtual void DoHelp( std::ostream & s )
     {                                      //
         s << "--record <filename>          : creates a DEBUG recording while running\n";
         s << "--playback <filename>        : plays back a DEBUG recording\n";
-        s << "<filename.aarec>             : short form, plays back a DEBUG recording, recording must end in .aarec\n";
     }
 };
 
