@@ -31,7 +31,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "config.h"
 #include <stddef.h>
 #if HAVE_UNISTD_H
-#include <unistd.h>
+// #include <unistd.h>
 #endif
 
 #include "tError.h"
@@ -63,19 +63,18 @@ public:
 template<class T> class tCheckedPTR:public tCheckedPTRBase{
     typedef T myclass;
 public:
-    tCheckedPTR(T *x):tCheckedPTRBase(x){}
-    tCheckedPTR(const tCheckedPTR<T> &x):tCheckedPTRBase(x.target){}
-    tCheckedPTR():tCheckedPTRBase(){}
-    ~tCheckedPTR(){}
+    tCheckedPTR(T *x):tCheckedPTRBase(x){};
+    tCheckedPTR(const tCheckedPTR<T> &x):tCheckedPTRBase(x.target){};
+    tCheckedPTR():tCheckedPTRBase(){};
+    ~tCheckedPTR(){};
 
     tCheckedPTR<T> &operator=(T *x){tCheckedPTRBase::operator=(x); return *this;}
     tCheckedPTR<T> &operator=(const tCheckedPTR<T> &x)
     {tCheckedPTRBase::operator=(x.target); return *this;}
-
-    T * get() const {return static_cast<T*>(target);}
-    T * operator->() const {return get();}
-    T & operator*() const {return get();}
-    operator T*() const {return get();}
+    T * operator->() const {return reinterpret_cast<T*>(target);}
+    T & operator*() const {return *reinterpret_cast<T*>(target);}
+    operator T*() const {return reinterpret_cast<T*>(target);}
+    //  T** operator&() {return reinterpret_cast<T **>(&target);}
 
     bool operator==(const T* x)const{return target==x;}
     bool operator!=(const T* x)const{return target!=x;}
@@ -94,11 +93,11 @@ public:
 template<class T> class tCheckedPTRConst:public tCheckedPTRBase{
     typedef T myclass;
 public:
-    tCheckedPTRConst():tCheckedPTRBase(NULL){}
-    tCheckedPTRConst(const T *x):tCheckedPTRBase(reinterpret_cast<void *>(x)){}
-    tCheckedPTRConst(const tCheckedPTRConst<T> &x):tCheckedPTRBase(x.target){}
-    tCheckedPTRConst(const tCheckedPTR<T> &x):tCheckedPTRBase(x.operator->()){}
-    ~tCheckedPTRConst(){}
+    tCheckedPTRConst():tCheckedPTRBase(NULL){};
+    tCheckedPTRConst(const T *x):tCheckedPTRBase(reinterpret_cast<void *>(x)){};
+    tCheckedPTRConst(const tCheckedPTRConst<T> &x):tCheckedPTRBase(x.target){};
+    tCheckedPTRConst(const tCheckedPTR<T> &x):tCheckedPTRBase(x.operator->()){};
+    ~tCheckedPTRConst(){};
 
     tCheckedPTRConst<T> &operator=(const T *x)
     {tCheckedPTRBase::operator=(reinterpret_cast<T *>(x)); return *this;}
@@ -107,10 +106,10 @@ public:
     tCheckedPTRConst<T> &operator=(const tCheckedPTR<T> &x)
     {tCheckedPTRBase::operator=(x.operator->()); return *this;}
 
-    const T * get() const {return static_cast<T const *>(target);}
-    const T * operator->() const {return get();}
-    const T & operator*() const {return *get();}
-    operator const T*() const {return get();}
+    const T * operator->() const {return target;}
+    const T & operator*() const {return *reinterpret_cast<T*>(target);}
+    operator const T*() const {return reinterpret_cast<const T*>(target);}
+    //  const T** operator&() {return reinterpret_cast<const T **>(&target);}
 
     bool operator==(const T* x)const{return target==x;}
     bool operator!=(const T* x)const{return target!=x;}
@@ -195,7 +194,7 @@ public:
     tControlledPTR(T *x):target(x){AddRef();}
     tControlledPTR(const tCheckedPTR<T> &x):target(x.operator->()){AddRef();}
     tControlledPTR(const tControlledPTR<T> &x):target(x.target){AddRef();}
-    tControlledPTR():target(NULL){}
+    tControlledPTR():target(NULL){};
 
 
     tControlledPTR<T> &operator=(T *x){
@@ -216,10 +215,6 @@ public:
         return *this;
     }
 
-    T * get() const {
-        return target;
-    }
-
     T * operator->() const {
         return target;
     }
@@ -286,7 +281,7 @@ public:
 
     ~tControlledPTR(){
         Release();
-    }
+    };
 };
 
 template<class T> class tJUST_CONTROLLED_PTR{
@@ -310,7 +305,7 @@ public:
     tJUST_CONTROLLED_PTR(T *x):target(x){AddRef();}
     tJUST_CONTROLLED_PTR(const tCheckedPTR<T> &x):target(x.operator->()){AddRef();}
     tJUST_CONTROLLED_PTR(const tJUST_CONTROLLED_PTR<T> &x):target(x.target){AddRef();}
-    tJUST_CONTROLLED_PTR():target(NULL){}
+    tJUST_CONTROLLED_PTR():target(NULL){};
 
 
     tJUST_CONTROLLED_PTR<T> &operator=(T *x){
@@ -327,10 +322,6 @@ public:
         return *this;
     }
 
-    T * get() const {
-        return target;
-    }
-
     T * operator->() const {
         return target;
     }
@@ -381,7 +372,7 @@ public:
 
     ~tJUST_CONTROLLED_PTR(){
         Release();
-    }
+    };
 };
 
 
@@ -430,33 +421,23 @@ void st_AddRefBreakpint( void const * object );
 void st_ReleaseBreakpint( void const * object );
 #endif
 
-// not thread-safe mutex
-struct tNonMutex
-{
-    void acquire(){}
-    void release(){}
-};
-
-template< class T, class MUTEX = tNonMutex > class tReferencable
+template< class T> class tReferencable
 {
     friend class tStackObject< T >;
 
 public:
-    tReferencable()												:refCtr_(0) {}
-    tReferencable				( const tReferencable& )						:refCtr_(0) {}
+    tReferencable()												:refCtr_(0) {};
+    tReferencable				( const tReferencable& )						:refCtr_(0) {};
     tReferencable& operator =	( const tReferencable& ){ return *this; }
 
     void		AddRef		()	const
     {
 #ifdef DEBUG
         st_AddRefBreakpint( this );
-#endif
-        tASSERT_THIS();
-        tASSERT( refCtr_ >= 0 );
-        mutex_.acquire();
+#endif        
+        tASSERT( this && refCtr_ >= 0 );
         ++refCtr_;
-        mutex_.release();
-        tASSERT( refCtr_ >= 0 );
+        tASSERT( this && refCtr_ >= 0 );
     }
 
     void		Release		() const
@@ -465,14 +446,8 @@ public:
         st_ReleaseBreakpint( this );
 #endif        
 
-        tASSERT_THIS();
-        tASSERT ( refCtr_ >= 0 );
-        mutex_.acquire();
-        --refCtr_;
-        bool kill = (refCtr_ <= 0);
-        mutex_.release();
-
-        if ( kill )
+        tASSERT ( this && refCtr_ >= 0 );
+        if ( --refCtr_ <= 0 )
         {
             refCtr_ = -1000;
             delete static_cast< const T* >( this );
@@ -481,19 +456,17 @@ public:
 
     int			GetRefcount	() const
     {
-        tASSERT_THIS();
+        tASSERT( this );
         return refCtr_;
     }
 protected:
     ~tReferencable()
     {
-        tASSERT_THIS();
-        tASSERT( ( refCtr_ == -1000 || refCtr_ == 0 ) );
+        tASSERT( this && ( refCtr_ == -1000 || refCtr_ == 0 ) );
         refCtr_ = -1000;
     }
 private:
     mutable int refCtr_;
-    mutable MUTEX mutex_;
 };
 
 
