@@ -104,7 +104,8 @@ void gArena::PrepareGrid(eGrid *grid, gParser *aParser)
     grid->Create();
     grid->SetWinding(axes);
 
-    aParser->InstantiateMap(sizeMultiplier);
+    aParser->setSizeMultiplier(sizeMultiplier);
+    aParser->Parse();
 
     init_grid_in_process=false;
 
