@@ -33,17 +33,18 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rConsole.h"
 #include "gGame.h"
 #include "gLogo.h"
-#include "eSound.h"
+#include "gCommandLineJumpStart.h"
+
+#include "eSoundMixer.h"
+
 #include "rScreen.h"
 #include "rSysdep.h"
 #include "uInputQueue.h"
 //#include "eTess.h"
 #include "rTexture.h"
 #include "tConfiguration.h"
-#include "tRandom.h"
 #include "tRecorder.h"
 #include "tCommandLine.h"
-#include "tToDo.h"
 #include "eAdvWall.h"
 #include "eGameObject.h"
 #include "uMenu.h"
@@ -62,12 +63,23 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef DEDICATED
 #include "rRender.h"
 #include "rSDL.h"
+#include <SDL_syswm.h>
+
+static gCommandLineJumpStartAnalyzer sg_jumpStartAnalyzer;
 #endif
 
+#ifndef DEDICATED
+#ifdef MACOSX
+#include "AAURLHandler.h"
+#endif
+#endif
+
+
 // data structure for command line parsing
 class gMainCommandLineAnalyzer: public tCommandLineAnalyzer
 {
 public:
+    bool     daemon_;
     bool     fullscreen_;
     bool     windowed_;
     bool     use_directx_;
@@ -75,6 +87,7 @@ public:
 
     gMainCommandLineAnalyzer()
     {
+        daemon_ = false;
         windowed_ = false;
         fullscreen_ = false;
         use_directx_ = false;
@@ -83,12 +96,13 @@ public:
 
 
 private:
-    bool DoAnalyze( tCommandLineParser & parser, int pass ) override
+    virtual bool DoAnalyze( tCommandLineParser & parser )
     {
-        if(pass > 0)
-            return false;
-
-        if ( parser.GetSwitch( "-fullscreen", "-f" ) )
+        if ( parser.GetSwitch( "--daemon","-d") )
+        {
+            daemon_ = true;
+        }
+        else if ( parser.GetSwitch( "-fullscreen", "-f" ) )
         {
             fullscreen_=true;
         }
@@ -96,6 +110,7 @@ private:
         {
             windowed_=true;
         }
+
 #ifdef WIN32
         else if ( parser.GetSwitch( "+directx") )
         {
@@ -114,7 +129,7 @@ private:
         return true;
     }
 
-    void DoHelp( std::ostream & s ) override
+    virtual void DoHelp( std::ostream & s )
     {                                      //
 #ifndef DEDICATED
         s << "-f, --fullscreen             : start in fullscreen mode\n";
@@ -124,147 +139,26 @@ private:
         << "                               initialisation under MS Windows\n\n";
         s << "\n\nYes, I know this looks ugly. Sorry about that.\n";
 #endif
+#else
+#ifndef WIN32
+        s << "-d, --daemon                 : allow the dedicated server to run as a daemon\n"
+        << "                               (will not poll for input on stdin)\n";
+#endif
 #endif
     }
 };
 
 static gMainCommandLineAnalyzer commandLineAnalyzer;
 
-extern bool sr_useDirectX; // rScreen.cpp
+static bool use_directx=true;
 #ifdef WIN32
 static tConfItem<bool> udx("USE_DIRECTX","makes use of the DirectX input "
                            "fuctions; causes some graphic cards to fail to work (VooDoo 3,...)",
-                           sr_useDirectX);
+                           use_directx);
 #endif
 
 extern void exit_game_objects(eGrid *grid);
 
-enum gConnection
-{
-    gLeave,
-    gDialup,
-    gISDN,
-    gDSL
-    // gT1
-};
-
-// initial setup menu
-void sg_StartupPlayerMenu()
-{
-    uMenu firstSetup("$first_setup", false);
-    firstSetup.SetBot(-.2);
-    
-    uMenuItemExit e2(&firstSetup, "$menuitem_accept", "$menuitem_accept_help");
-    
-    ePlayer * player = ePlayer::PlayerConfig(0);
-    tASSERT( player );
-
-    gConnection connection = gDSL;
-
-    uMenuItemSelection<gConnection> net(&firstSetup, "$first_setup_net", "$first_setup_net_help", connection );
-    if ( !st_FirstUse )
-    {
-        net.NewChoice( "$first_setup_leave", "$first_setup_leave_help", gLeave );
-        connection = gLeave;
-    }
-    net.NewChoice( "$first_setup_net_dialup", "$first_setup_net_dialup_help", gDialup );
-    net.NewChoice( "$first_setup_net_isdn", "$first_setup_net_isdn_help", gISDN );
-    net.NewChoice( "$first_setup_net_dsl", "$first_setup_net_dsl_help", gDSL );
-
-    tString keyboardTemplate("keys_cursor.cfg");
-    uMenuItemSelection<tString> k(&firstSetup, "$first_setup_keys", "$first_setup_keys_help", keyboardTemplate );
-    if ( !st_FirstUse )
-    {
-        k.NewChoice( "$first_setup_leave", "$first_setup_leave_help", tString("") );
-        keyboardTemplate="";
-    }
-    k.NewChoice( "$first_setup_keys_cursor", "$first_setup_keys_cursor_help", tString("keys_cursor.cfg") );
-    k.NewChoice( "$first_setup_keys_wasd", "$first_setup_keys_wasd_help", tString("keys_wasd.cfg") );
-    k.NewChoice( "$first_setup_keys_zqsd", "$first_setup_keys_zqsd_help", tString("keys_zqsd.cfg") );
-    k.NewChoice( "$first_setup_keys_cursor_single", "$first_setup_keys_cursor_single_help", tString("keys_cursor_single.cfg") );
-    // k.NewChoice( "$first_setup_keys_both", "$first_setup_keys_both_help", tString("keys_twohand.cfg") );
-    k.NewChoice( "$first_setup_keys_x", "$first_setup_keys_x_help", tString("keys_x.cfg") );
-
-    tColor leave(0,0,0,0);
-    tColor color(1,0,0);
-    uMenuItemSelection<tColor> c(&firstSetup,
-                                 "$first_setup_color",
-                                 "$first_setup_color_help",
-                                 color);   
-
-    if ( !st_FirstUse )
-    {
-        color = leave;
-        c.NewChoice( "$first_setup_leave", "$first_setup_leave_help", leave );
-    }
-
-    c.NewChoice( "$first_setup_color_red", "", tColor(1,0,0) );
-    c.NewChoice( "$first_setup_color_blue", "", tColor(0,0,1) );
-    c.NewChoice( "$first_setup_color_green", "", tColor(0,1,0) );
-    c.NewChoice( "$first_setup_color_yellow", "", tColor(1,1,0) );
-    c.NewChoice( "$first_setup_color_orange", "", tColor(1,.5,0) );
-    c.NewChoice( "$first_setup_color_purple", "", tColor(.5,0,1) );
-    c.NewChoice( "$first_setup_color_magenta", "", tColor(1,0,1) );
-    c.NewChoice( "$first_setup_color_cyan", "", tColor(0,1,1) );
-    c.NewChoice( "$first_setup_color_white", "", tColor(1,1,1) );
-    c.NewChoice( "$first_setup_color_dark", "", tColor(0,0,0) );
-    
-    if ( st_FirstUse )
-    {
-        for(int i=tRandomizer::GetInstance().Get(4); i>=0; --i)
-        {
-            c.LeftRight(1);
-        }
-    }
-
-    uMenuItemString n(&firstSetup,
-                      "$player_name_text",
-                      "$player_name_help",
-                      player->name, 16);
-    
-    uMenuItemExit e(&firstSetup, "$menuitem_accept", "$menuitem_accept_help");
-    
-    firstSetup.Enter();
-
-    // apply network rates
-    switch(connection)
-    {
-    case gDialup:
-        sn_maxRateIn  = 6;
-        sn_maxRateOut = 4;
-        break;
-    case gISDN:
-        sn_maxRateIn  = 8;
-        sn_maxRateOut = 8;
-        break;
-    case gDSL:
-        sn_maxRateIn  = 64;
-        sn_maxRateOut = 16;
-        break;
-    case gLeave:
-        break;
-    }
-
-    // store color
-    if( ! (color == leave) )
-    {
-        player->rgb[0] = int(color.r_*15);
-        player->rgb[1] = int(color.g_*15);
-        player->rgb[2] = int(color.b_*15);
-    }
-
-    // load keyboard layout
-    if( keyboardTemplate.Len() > 1 )
-    {
-        std::ifstream s;
-        if( tConfItemBase::OpenFile( s, keyboardTemplate, tConfItemBase::Config ) )
-        {
-            tCurrentAccessLevel level( tAccessLevel_Owner, true );
-            tConfItemBase::ReadFile( s );
-        }
-    }
-}
-
 #ifndef DEDICATED
 static void welcome(){
     bool textOutBack = sr_textOut;
@@ -294,6 +188,7 @@ static void welcome(){
 
     if (st_FirstUse)
     {
+        st_FirstUse=false;
         sr_LoadDefaultConfig();
         textOutBack = sr_textOut;
         sr_textOut = false;
@@ -307,6 +202,12 @@ static void welcome(){
         showSplash = false;
 #endif
 
+        // Start the music up
+        eSoundMixer* mixer;
+        mixer = eSoundMixer::GetMixer();
+        mixer->SetMode(TITLE_TRACK);
+        mixer->Update();
+
         // disable splash screen when recording (it's annoying)
         static const char * splashSection = "SPLASH";
         if ( tRecorder::IsRunning() )
@@ -317,6 +218,19 @@ static void welcome(){
             if ( !tRecorder::Playback( splashSection, showSplash ) )
                 showSplash = tRecorder::IsPlayingBack();
         }
+
+#ifndef DEDICATED
+        if ( sg_jumpStartAnalyzer.ShouldConnect() )
+        {
+            showSplash = false;
+            gLogo::SetDisplayed(false);
+            sg_jumpStartAnalyzer.Connect();
+        }
+#endif
+
+#ifdef MACOSX
+        StartAAURLHandler();
+#endif
         tRecorder::Record( splashSection, showSplash );
 
         if ( showSplash )
@@ -324,7 +238,7 @@ static void welcome(){
             timeout = tSysTimeFloat() + 6;
 
             uInputProcessGuard inputProcessGuard;
-            while ((!su_GetSDLInput(tEvent) || tEvent.type!=SDL_KEYDOWN) &&
+            while((!su_GetSDLInput(tEvent) || tEvent.type!=SDL_KEYDOWN) &&
                     tSysTimeFloat() < timeout)
             {
                 if ( sr_glOut )
@@ -346,7 +260,7 @@ static void welcome(){
         // catch some keyboard input
         {
             uInputProcessGuard inputProcessGuard;
-            while (su_GetSDLInput(tEvent)) ;
+            while (su_GetSDLInput(tEvent));
         }
 
         sr_textOut = textOutBack;
@@ -356,48 +270,70 @@ static void welcome(){
     if ( sr_glOut )
     {
         rSysDep::ClearGL();
-        rFont::s_defaultFont.Select();
-        rFont::s_defaultFontSmall.Select();
+        //        rFont::s_defaultFont.Select();
+        //        rFont::s_defaultFontSmall.Select();
         gLogo::Display();
         rSysDep::ClearGL();
     }
     rSysDep::SwapGL();
 
-    sr_textOut = textOutBack;
-    sg_StartupLanguageMenu();
+    sg_LanguageMenu();
 
-    sr_textOut = textOutBack;
-    sg_StartupPlayerMenu();
+    // catch some keyboard input
+    {
+        uInputProcessGuard inputProcessGuard;
+        while (su_GetSDLInput(tEvent));
+    }
 
-    st_FirstUse=false;
+    timeout = tSysTimeFloat() + 10;
 
-    sr_textOut = textOutBack;
-    uMenu::Message( tOutput("$welcome_message_heading"), tOutput("$welcome_message"), 300 );
+    sr_UnlockSDL();
+    uInputProcessGuard inputProcessGuard;
+    while((!su_GetSDLInput(tEvent) || tEvent.type!=SDL_KEYDOWN) &&
+            tSysTimeFloat() < timeout){
 
-    // start a first single player game
-    auto speedFactor = sg_currentSettings->speedFactor;
-    auto autoNum = sg_currentSettings->autoNum;
-    auto sizeFactor = sg_currentSettings->sizeFactor;
-    auto wallsLength = sg_currentSettings->wallsLength;
-    auto rubber = sg_rubberCycle;
-    auto delayCycle = sg_delayCycle;
-    sg_currentSettings->speedFactor = -2;
-    sg_currentSettings->autoNum = 0;
-    sg_currentSettings->sizeFactor -= 2;
-    sg_currentSettings->wallsLength = 400;
-    sg_rubberCycle = 5;
-    sg_delayCycle = 0.05;
-    sr_textOut = textOutBack;
-    sg_SinglePlayerGame();
-    sg_currentSettings->autoNum = autoNum;
-    sg_currentSettings->speedFactor = speedFactor;
-    sg_currentSettings->sizeFactor= sizeFactor;
-    sg_currentSettings->wallsLength = wallsLength;
-    sg_rubberCycle = rubber;
-    sg_delayCycle = delayCycle;
+        sr_ResetRenderState(true);
+        rViewport::s_viewportFullscreen.Select();
 
-    sr_textOut = textOutBack;
-    uMenu::Message( tOutput("$welcome_message_2_heading"), tOutput("$welcome_message_2"), 300 );
+        if ( sr_glOut )
+        {
+            rSysDep::ClearGL();
+
+            uMenu::GenericBackground();
+
+            REAL w=16*2/640.0;
+            REAL h=32*2/480.0;
+
+
+            //REAL middle=-.6;
+
+            Color(1,1,1);
+            DisplayText(0,.8,h,tOutput("$welcome_message_heading"), sr_fontError);
+
+            w/=2;
+            h/=2;
+
+            rTextField c(-.8,.6, h, sr_fontError);
+
+
+            c << tOutput("$welcome_message_intro");
+
+            c.SetIndent(12);
+
+            c << tOutput("$welcome_message_vendor")   << gl_vendor   << '\n';
+            c << tOutput("$welcome_message_renderer") << gl_renderer << '\n';
+            c << tOutput("$welcome_message_version")  << gl_version  << '\n';
+
+            c.SetIndent(0);
+
+            c << tOutput("$welcome_message_finish");
+
+            rSysDep::SwapGL();
+        }
+
+        tAdvanceFrame();
+    }
+    sr_LockSDL();
 
     sr_textOut = textOutBack;
 }
@@ -415,18 +351,18 @@ void cleanup(eGrid *grid){
           if (playerConfig[i])
           destroy(playerConfig[i]->cam);
           }
-          
-          
+
+
           gNetPlayerWall::Clear();
 
           eFace::Clear();
           eEdge::Clear();
           ePoint::Clear();
-          
+
           eFace::Clear();
           eEdge::Clear();
           ePoint::Clear();
-          
+
           eGameObject::DeleteAll();
 
 
@@ -444,7 +380,6 @@ void cleanup(eGrid *grid){
             rITexture::UnloadAll();
         }
 
-        se_SoundExit();
         sr_glOut=false;
         sr_ExitDisplay();
 
@@ -456,12 +391,6 @@ void cleanup(eGrid *grid){
 }
 
 #ifndef DEDICATED
-static bool sg_active = true;
-static void sg_DelayedActivation()
-{
-    Activate( sg_active );
-}
-
 int filter(const SDL_Event *tEvent){
     // recursion avoidance
     static bool recursion = false;
@@ -500,12 +429,12 @@ int filter(const SDL_Event *tEvent){
             tRecorder::Record("END");
 
             st_SaveConfig();
-            uMenu::quickexit=uMenu::QuickExit_Total;
+            uMenu::quickexit=true;
             return false;
         }
 
-        if (tEvent->type==SDL_MOUSEMOTION)
-            if (tEvent->motion.x==sr_screenWidth/2 && tEvent->motion.y==sr_screenHeight/2)
+        if(tEvent->type==SDL_MOUSEMOTION)
+            if(tEvent->motion.x==sr_screenWidth/2 && tEvent->motion.y==sr_screenHeight/2)
                 return 0;
         if (su_mouseGrab &&
                 tEvent->type!=SDL_MOUSEBUTTONDOWN &&
@@ -520,23 +449,13 @@ int filter(const SDL_Event *tEvent){
         {
             // Jonathans fullscreen bugfix.
 #ifdef MACOSX
-            if (currentScreensetting.fullscreen ^ lastSuccess.fullscreen) return false;
+            if(currentScreensetting.fullscreen ^ lastSuccess.fullscreen) return false;
 #endif
             int flags = SDL_APPINPUTFOCUS;
-            if ( tEvent->active.state & flags )
-            {
-                // con << tSysTimeFloat() << " " << "active: " << (tEvent->active.gain ? "on" : "off") << "\n";
-                sg_active = tEvent->active.gain;
-                st_ToDo(sg_DelayedActivation);
-            }
-
-            // reload GL stuff if application gets reactivated
-            if ( tEvent->active.gain && tEvent->active.state & SDL_APPACTIVE )
-            {
-                // just treat it like a screen mode change, gets the job done
-                st_ToDo(rCallbackBeforeScreenModeChange::Exec);
-                st_ToDo(rCallbackAfterScreenModeChange::Exec);
-            }
+            if ( tEvent->active.gain && tEvent->active.state & flags )
+                Activate(true);
+            if ( !tEvent->active.gain && tEvent->active.state & flags )
+                Activate(false);
             return false;
         }
 
@@ -556,53 +475,29 @@ void Update_netPlayer();
 void sg_SetIcon()
 {
 #ifndef DEDICATED
-    rSurface tex( "textures/icon.png" );
+#ifndef MACOSX
+#ifdef  WIN32
+    SDL_SysWMinfo	info;
+    HICON			icon;
+    // get the HWND handle
+    SDL_VERSION( &info.version );
+    if( SDL_GetWMInfo( &info ) )
+    {
+        icon = LoadIcon( GetModuleHandle( NULL ), MAKEINTRESOURCE( 1 ) );
+        SetClassLong( info.window, GCL_HICON, (LONG) icon );
+    }
+#else
+    rSurface tex( "desktop/icons/medium/armagetronad.png" );
     //    SDL_Surface *tex=IMG_Load( tDirectories::Data().GetReadPath( "textures/icon.png" ) );
 
     if (tex.GetSurface())
         SDL_WM_SetIcon(tex.GetSurface(),NULL);
 #endif
-}
-
-class gAutoStringArray
-{
-public:
-    ~gAutoStringArray()
-    {
-#ifdef HAVE_CLEARENV
-        // Optional. Systems that don't have this function better make copies of putenv() arguments.
-        clearenv();
 #endif
-
-        for ( std::vector< char * >::iterator i = strings.begin(); i != strings.end(); ++i )
-        {
-            free( *i );
-        }
-    }
-
-    char * Store( char const * s )
-    {
-        char * ret = strdup( s );
-        strings.push_back( ret );
-        return ret;
-    }
-private:
-    std::vector< char * > strings; // the stored raw C strings
-};
-
-// wrapper for putenv, taking care of the peculiarity that the argument
-// is kept in use for the rest of the program's lifetime
-void sg_PutEnv( char const * s )
-{
-    static gAutoStringArray store;
-    putenv( store.Store( s ) );
+#endif
 }
 
-namespace
-{
-tString sn_configurationSavedInVersion{"0.2.8"};
-tConfItem<tString> sn_configurationSavedInVersionConf("SAVED_IN_VERSION",sn_configurationSavedInVersion);
-}
+static const char * dedicatedSection = "DEDICATED";
 
 int main(int argc,char **argv){
     //std::cout << "enter\n";
@@ -624,35 +519,14 @@ int main(int argc,char **argv){
         if ( ! commandLine.Analyse(argc, argv) )
             return 0;
 
-
+        // read RAND_MAX from recording or write it to recording
+        if ( !tRecorder::PlaybackStrict( dedicatedSection, dedicatedServer ) )
         {
-            // embed version in recording
-            const char * versionSection = "VERSION";
-            tString version( sn_programVersion );
-            tRecorder::Playback( versionSection, version );
-            tRecorder::Record( versionSection, version );
-#ifndef DEDICATED
-            if(version != sn_programVersion)
-            {
-#ifdef DEBUG
-                tERR_WARN( "Recording from a different version, consider at high risk of desync." );
-#endif
-                tRecorder::ActivateProbablyDesyncedPlayback();
-            }
-#endif
-        }
-
-        {
-            // read/write server/client mode from/to recording
-            const char * dedicatedSection = "DEDICATED";
-            if ( !tRecorder::PlaybackStrict( dedicatedSection, dedicatedServer ) )
-            {
 #ifdef DEDICATED
-                dedicatedServer = true;
+            dedicatedServer = true;
 #endif
-            }
-            tRecorder::Record( dedicatedSection, dedicatedServer );
         }
+        tRecorder::Record( dedicatedSection, dedicatedServer );
 
         // tERR_MESSAGE( "Initializing player data." );
         ePlayer::Init();
@@ -662,19 +536,6 @@ int main(int argc,char **argv){
 
         st_LoadConfig();
 
-        // migrate user configuration from previous versions
-        if(sn_configurationSavedInVersion != sn_programVersion)
-        {
-            if(st_FirstUse)
-            {
-                sn_configurationSavedInVersion = "0.0";
-            }
-
-            tConfigMigration::Migrate(sn_configurationSavedInVersion);
-        }
-        if(tConfigMigration::SavedBefore(sn_configurationSavedInVersion, sn_programVersion))
-            sn_configurationSavedInVersion = sn_programVersion;
-
         // record and play back the recording debug level
         tRecorderSyncBase::GetDebugLevelPlayback();
 
@@ -683,19 +544,16 @@ int main(int argc,char **argv){
         if ( commandLineAnalyzer.windowed_ )
             currentScreensetting.fullscreen   = false;
         if ( commandLineAnalyzer.use_directx_ )
-            sr_useDirectX                       = true;
+            use_directx                       = true;
         if ( commandLineAnalyzer.dont_use_directx_ )
-            sr_useDirectX                       = false;
+            use_directx                       = false;
 
-        gAICharacter::LoadAll(tString( "aiplayers.cfg" ) );
+        //gAICharacter::LoadAll(tString( "aiplayers.cfg" ) );
+        gAICharacter::LoadAll( aiPlayersConfig );
 
         sg_LanguageInit();
         atexit(tLocale::Clear);
 
-        static eLadderLogWriter sg_encodingWriter( "ENCODING", true );
-        sg_encodingWriter << st_internalEncoding;
-        sg_encodingWriter.write();
-
         if ( commandLine.Execute() )
         {
             gCycle::PrivateSettings();
@@ -718,11 +576,11 @@ int main(int argc,char **argv){
                 if (! tDirectories::Var().Open( s, "scorelog.txt", std::ios::app ) )
                 {
                     char const * error = "var directory not writable or does not exist. It should reside inside your user data directory and should have been created automatically on first start, but something must have gone wrong."
-#ifdef WIN32
+                    #ifdef WIN32
                                          " You can access your user data directory over one of the start menu entries we installed."
-#else
+                    #else
                                          " Your user data directory is subdirectory named .armagetronad in your home directory."
-#endif
+                    #endif
                                          ;
 
                     tERR_ERROR( error );
@@ -738,56 +596,57 @@ int main(int argc,char **argv){
                 }
             }
 
-#ifndef DEDICATED
+		#ifndef DEDICATED
             sr_glOut=1;
-#endif
+		#endif
 
             //std::cout << "checked mp\n";
 
             // while DGA mouse is buggy in XFree 4.0:
-#ifdef linux
+		#ifdef linux
             // Sam 5/23 - Don't ever use DGA, we don't need it for this game.
             if ( ! getenv("SDL_VIDEO_X11_DGAMOUSE") ) {
-                sg_PutEnv( "SDL_VIDEO_X11_DGAMOUSE=0" );
+                putenv("SDL_VIDEO_X11_DGAMOUSE=0");
             }
-#endif
+		#endif
 
-#ifdef WIN32
+		#ifdef WIN32
             // disable DirectX by default; it causes problems with some boards.
-            if (!getenv( "SDL_VIDEODRIVER") ) {
-                if (sr_useDirectX) {
-                    // NO. Causes crashes with SDL1.2 on Windows on window deactivation/alt-tabbing away
-                    // sg_PutEnv( "SDL_VIDEODRIVER=directx" );
-                }
-                else
-                {
-                    sg_PutEnv( "SDL_VIDEODRIVER=windib" );
-                }
+            if (!use_directx && !getenv("SDL_VIDEODRIVER") ) {
+                putenv("SDL_VIDEODRIVER=windib");
             }
-#endif
+		#endif
 
             // atexit(ANET_Shutdown);
 
-#ifndef WIN32
-#ifdef DEBUG
-#define NOSOUND
-#endif
-#endif
+		#ifndef WIN32
+		#ifdef DEBUG
+            #define NOSOUND
+		#endif
+		#endif
 
-#ifndef DEDICATED
-            if (
-#ifndef NOSOUND
-#ifndef DEFAULT_SDL_AUDIODRIVER
-                SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0 &&
+		#ifndef DEDICATED
+            Uint32 flags = SDL_INIT_VIDEO;
+#ifdef DEBUG
+            flags |= SDL_INIT_NOPARACHUTE;
 #endif
-#endif
-                SDL_Init(SDL_INIT_VIDEO) < 0 )            {
+            if (SDL_Init(flags) < 0) {
                 tERR_ERROR("Couldn't initialize SDL: " << SDL_GetError());
             }
             atexit(SDL_Quit);
 
             sr_glRendererInit();
 
+		#ifndef NOJOYSTICK
+            if (SDL_InitSubSystem(SDL_INIT_JOYSTICK))
+                std::cout << "Error initializing joystick subsystem\n";
+            else
+            {
+                std::cout << "Joystick(s) initialized\n";
+                su_JoystickInit();
+            }
+		#endif
+
             SDL_SetEventFilter(&filter);
 
             //std::cout << "set filter\n";
@@ -795,17 +654,6 @@ int main(int argc,char **argv){
             sg_SetIcon();
 
             tConsole::RegisterMessageCallback(&uMenu::Message);
-            tConsole::RegisterIdleCallback(&uMenu::IdleInput);
-
-#ifndef NOSOUND
-            se_SoundInit();
-            atexit(se_SoundExit);
-#ifndef DEBUG
-            // double sound initialisation for dodgy cards
-            se_SoundExit();
-            se_SoundInit();
-#endif
-#endif
 
             if (sr_InitDisplay()){
 
@@ -821,8 +669,6 @@ int main(int argc,char **argv){
 
                     sr_con.autoDisplayAtSwap=false;
 
-                    se_SoundPause(false);
-
                     //std::cout << "sound started\n";
 
                     gLogo::SetBig(false);
@@ -839,6 +685,7 @@ int main(int argc,char **argv){
                     nNetObject::ClearAll();
 
                     rITexture::UnloadAll();
+                    sr_RendererCleanup();
                 }
                 catch (tException const & e)
                 {
@@ -851,7 +698,6 @@ int main(int argc,char **argv){
                 }
 
                 sr_ExitDisplay();
-                sr_RendererCleanup();
 
                 //std::cout << "exit\n";
 
@@ -862,16 +708,21 @@ int main(int argc,char **argv){
                 //    cleanup(grid);
                 SDL_QuitSubSystem(SDL_INIT_VIDEO);
             }
-            se_SoundExit();
+
+            eSoundMixer::ShutDown();
+
             SDL_Quit();
-#else
+		#else
+            if (!commandLineAnalyzer.daemon_)
+                sr_Unblock_stdin();
+
             sr_glOut=0;
 
             //  nServerInfo::TellMasterAboutMe();
 
             while (!uMenu::quickexit)
                 sg_HostGame();
-#endif
+		#endif
             nNetObject::ClearAll();
             nServerInfo::DeleteAll();
         }
@@ -881,17 +732,13 @@ int main(int argc,char **argv){
 
         //	tLocale::Clear();
     }
-    catch ( tCleanQuit const & e )
-    {
-        return 0;
-    }
-    catch ( tException const & e )
+    catch( tException const & e )
     {
         try
         {
             st_PresentError( e.GetName(), e.GetDescription() );
         }
-        catch (...)
+        catch(...)
         {
         }
 
@@ -901,20 +748,20 @@ int main(int argc,char **argv){
 #pragma warning ( disable : 4286 )
     // GRR. Visual C++ dones not handle generic exceptions with the above general statement.
     // A specialized version is needed. The best part: it warns about the code below being redundant.
-    catch ( tGenericException const & e )
+    catch( tGenericException const & e )
     {
         try
         {
             st_PresentError( e.GetName(), e.GetDescription() );
         }
-        catch (...)
+        catch(...)
         {
         }
 
         return 1;
     }
 #endif
-    catch (...)
+    catch(...)
     {
         return 1;
     }
@@ -924,10 +771,7 @@ int main(int argc,char **argv){
 
 #ifdef DEDICATED
 // settings missing in the dedicated server
-static void st_Dummy(std::istream &s){
-    tString rest;
-    rest.ReadLine(s);
-}
+static void st_Dummy(std::istream &s){tString rest; rest.ReadLine(s);}
 static tConfItemFunc st_Dummy10("MASTER_QUERY_INTERVAL", &st_Dummy);
 static tConfItemFunc st_Dummy11("MASTER_SAVE_INTERVAL", &st_Dummy);
 static tConfItemFunc st_Dummy12("MASTER_IDLE", &st_Dummy);
