@@ -31,24 +31,20 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 //#define USE_HEADLIGHT
 
 #include "gStuff.h"
-//#include "eSound.h"
 //#include "rTexture.h"
 //#include "rModel.h"
 #include "eNetGameObject.h"
 #include "tList.h"
 #include "nObserver.h"
-#include "rDisplayList.h"
 
 #include "gCycleMovement.h"
 
 class rModel;
 class gTextureCycle;
-class eSoundPlayer;
 class gSensor;
 class gNetPlayerWall;
 class gPlayerWall;
 class eTempEdge;
-struct gPredictPositionData;
 
 // minimum time between two cycle turns
 extern REAL sg_delayCycle;
@@ -84,6 +80,28 @@ public:
     ~gCycleMemory();
 };
 
+class gEnemyInfluence{
+private:
+    nObserverPtr< ePlayerNetID >	lastEnemyInfluence;  	// the last enemy wall we encountered
+    REAL							lastTime;				// the time it was drawn at
+
+public:
+    gEnemyInfluence();
+
+    ePlayerNetID const *            GetEnemy() const;	    // the last enemy possibly responsible for our death
+    REAL                            GetTime() const;        // the time of the influence
+    void							AddSensor( const gSensor& sensor, REAL timePenalty, gCycle * thisCycle ); // add the result of the sensor scan to our data
+    void							AddWall( const eWall * wall, eCoord const & point, REAL timePenalty, gCycle * thisCycle ); // add the interaction with a wall to our data
+    void							AddWall( const gPlayerWall * wall, REAL timeBuilt, gCycle * thisCycle ); // add the interaction with a wall to our data
+};
+
+struct gRealColor {
+    REAL r,g,b;
+
+    gRealColor():r(1), g(1), b(1){}
+
+};
+
 // class used to extrapolate the movement of a lightcycle
 class gCycleExtrapolator: public gCycleMovement
 {
@@ -99,9 +117,7 @@ public:
 
     virtual bool EdgeIsDangerous(const eWall *w, REAL time, REAL a) const;
 
-    virtual void PassEdge(const eWall *w,REAL time,REAL a,int recursion=1);
-
-    virtual bool TimestepCore(REAL currentTime, bool calculateAcceleration = true );
+    virtual bool TimestepCore(REAL currentTime);
 
     // virtual bool DoTurn(int dir);
 
@@ -114,66 +130,21 @@ private:
     const gCycleMovement* parent_;												// the cycle that is extrapolated
 };
 
-class gCycleChatBot;
-
-#ifndef DEDICATED
-class gCycleWallsDisplayListManager
-{
-    friend class gNetPlayerWall;
-
-public:
-    gCycleWallsDisplayListManager();
-    ~gCycleWallsDisplayListManager();
-
-    //! checks whether a wall at a certain distance can have a display list
-    static bool CannotHaveList( REAL distance, gCycle const * cycle );
-
-    //! renders all walls scheduled for display list usage
-    void RenderAllWithDisplayList( eCamera const * camera, gCycle * cycle );
-
-    //! render all walls in a list
-    static void RenderAll( eCamera const * camera, gCycle * cycle, gNetPlayerWall * list );
-
-    //! render all walls
-    void RenderAll( eCamera const * camera, gCycle * cycle );
-    bool Walls() const
-    {
-        return wallList_ || wallsWithDisplayList_;
-    }
-
-    void Clear( int inhibit = 0 )
-    {
-        displayList_.Clear( inhibit );
-    }
-private:
-    gNetPlayerWall *                wallList_;                      //!< linked list of all walls
-    gNetPlayerWall *                wallsWithDisplayList_;          //!< linked list of all walls with display list    
-    rDisplayList                    displayList_;                   //!< combined display list
-    REAL                            wallsWithDisplayListMinDistance_; //!< minimal distance of the walls with display list
-    int                             wallsInDisplayList_;            //!< number of walls in the current display list
-};
-#endif
-
 // a complete lightcycle
 class gCycle: public gCycleMovement
 {
     friend class gPlayerWall;
     friend class gNetPlayerWall;
     friend class gDestination;
-    friend class gCycleWallRenderer;
-
-    eSoundPlayer *engine;
-    eSoundPlayer *turning;
-    eSoundPlayer *spark;
 
     REAL spawnTime_;    //!< time the cycle spawned at
     REAL lastTimeAnim;  //!< last time animation was simulated at
+
     REAL timeCameIntoView;
 
-    friend class gCycleChatBot;
-    std::unique_ptr< gCycleChatBot > chatBot_;
-
+    REAL nextChatAI;
     bool dropWallRequested_; //!< flag indicating that someone requested a wall drop
+
 public:
     eCoord            lastGoodPosition_;    // the location of the last known good position
 
@@ -202,7 +173,6 @@ public:
     // pos is always the correct simulated position; the displayed position is calculated as pos + correctPosSmooth
     // and correctPosSmooth decays with time.
     eCoord correctPosSmooth;
-    eCoord predictPosition_; //!< the best guess of where the cycle is at at display time
 
     // every frame, a bit of this variable is taken away and added to the step the cycle makes.
     REAL correctDistanceSmooth;
@@ -210,9 +180,7 @@ public:
 private:
     void TransferPositionCorrectionToDistanceCorrection();
 
-#ifndef DEDICATED
-    gCycleWallsDisplayListManager displayList_;                     //!< display list manager
-#endif
+    gEnemyInfluence				enemyInfluence;
 
     tCHECKED_PTR(gNetPlayerWall)	currentWall;                    //!< the wall that currenly is attached to the cycle
     tCHECKED_PTR(gNetPlayerWall)	lastWall;                       //!< the last wall that was attached to this cycle
@@ -228,30 +196,23 @@ private:
     void	SyncFromExtrapolator();							// take over the extrapolator's data
 
     virtual void OnNotifyNewDestination(gDestination *dest);   //!< called when a destination is successfully inserted into the destination list
-    virtual void OnDropTempWall        ( gPlayerWall * wall, eCoord const & position, eCoord const & direction );   //!< called when another cycle grinds a wall; this cycle should then drop its current wall if the grinding is too close.
+    virtual void OnDropTempWall        (gPlayerWall * wall);   //!< called when another cycle grinds a wall; this cycle should then drop its current wall if the grinding is too close.
 
     //	unsigned short currentWallID;
 
     nTimeRolling nextSync, nextSyncOwner;
-    REAL lastSyncOwnerGameTime_;    //!< time of the last sync to the owner in game time
 
     void MyInitAfterCreation();
 
     void SetCurrentWall(gNetPlayerWall *w);
-
-    void PreparePredictPosition( gPredictPositionData & data ); //!< prepares CalculatePredictPosition() call, requesting a raycast to the front
-    REAL CalculatePredictPosition( gPredictPositionData & data ); //!< Calculates predictPosition_
 protected:
     virtual ~gCycle();
 
-    virtual void OnRemoveFromGame(); // called when the cycle is physically removed from the game
-
-    virtual void OnRoundEnd();    //!< called when the round ends
+    virtual void RemoveFromGame(); // call this instead of the destructor
 
     // virtual REAL            DoGetDistanceSinceLastTurn  (                               ) const     ;   //!< returns the distance since the last turn
 public:
-    virtual void Die ( REAL time )  ;  //!< dies at the specified time
-    void KillAt( const eCoord& pos );  //!< kill this cycle at the given position and take care of scoring
+    void KillAt( const eCoord& pos ); //!< kill this cycle at the given position and take care of scoring
 
     int WindingNumber() const {return windingNumber_;}
 
@@ -260,7 +221,7 @@ public:
     // bool CanMakeTurn() const { return pendingTurns <= 0 && lastTime >= nextTurn; }
 
     virtual void InitAfterCreation();
-    gCycle(eGrid *grid, const eCoord &pos,const eCoord &dir,ePlayerNetID *p=NULL);
+    gCycle(eGrid *grid, const eCoord &pos,const eCoord &dir,ePlayerNetID *p=NULL,bool autodelete=1);
 
     static	void 	SetWallsStayUpDelay		( REAL delay );				//!< the time the cycle walls stay up ( negative values: they stay up forever )
     static	void 	SetWallsLength			( REAL length);				//!< the maximum total length of the walls
@@ -270,18 +231,13 @@ public:
     static	REAL	WallsLength()	 	 { return wallsLength;		}	//!< the default total length of the walls
     REAL	        MaxWallsLength() const;                             //!< the maximum total length of the walls (including max effect of rubber growth)
     REAL	        ThisWallsLength() const;                            //!< the maximum total length of this cycle's wall (including rubber shrink)
-    REAL	        WallEndSpeed() const;                               //!< the speed the end of the trail is receeding with right now
     static	REAL	ExplosionRadius()	 { return explosionRadius;	}	//!< the radius of the holes blewn in by an explosion
 
-    bool    IsMe( eGameObject const * other ) const;              //!< checks whether the passed pointer is logically identical with this cycle
-
     // the network routines:
     gCycle(nMessage &m);
     virtual void WriteCreate(nMessage &m);
     virtual void WriteSync(nMessage &m);
     virtual void ReadSync(nMessage &m);
-    virtual void RequestSyncOwner(); //!< requests special syncs to the owner on important points (just passed an enemy trail end safely...)
-    virtual void RequestSyncAll(); //!< requests special syncs to everyone on important points (just passed an enemy trail end safely...)
 
     virtual void SyncEnemy ( const eCoord& begWall );    //!< handle sync message for enemy cycles
     // virtual void SyncFriend( const eCoord& begWall );    //!< handle sync message for enemy cycles
@@ -295,7 +251,7 @@ public:
     //virtual bool ClearToTransmit(int user) const;
 
     virtual bool Timestep(REAL currentTime);
-    virtual bool TimestepCore(REAL currentTime,bool calculateAcceleration = true);
+    virtual bool TimestepCore(REAL currentTime);
 
     virtual void InteractWith(eGameObject *target,REAL time,int recursion=1);
 
@@ -325,13 +281,13 @@ public:
 
     virtual bool RenderCockpitFixedBefore(bool primary=true);
 
-    virtual void SoundMix(unsigned char *dest,unsigned int len,
-                          int viewer,REAL rvol,REAL lvol);
+    //virtual void SoundMix(unsigned char *dest,unsigned int len,
+    //                      int viewer,REAL rvol,REAL lvol);
 #endif
 
-    virtual eCoord CamPos() const;
-    virtual eCoord PredictPosition() const;
-    virtual eCoord  CamTop() const;
+    virtual eCoord CamPos();
+    virtual eCoord PredictPosition();
+    virtual eCoord  CamTop();
 
     virtual void RightBeforeDeath( int numTries );
 
