@@ -28,17 +28,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef ArmageTron_explosion_H
 #define ArmageTron_explosion_H
 
-#include "eSound.h"
 #include "eGameObject.h"
-#include "gParticles.h"
 
-class gCycle;
 struct gRealColor;
 
-class gExplosion: virtual public eReferencableGameObject
+class gExplosion: virtual public eGameObject
 { // Boom!
 public:
-    gExplosion(eGrid *grid, const eCoord &pos,REAL time, gRealColor& color, gCycle * owner );
+    gExplosion(eGrid *grid, const eCoord &pos,REAL time, gRealColor& color);
     virtual ~gExplosion();
 
     virtual bool Timestep(REAL currentTime);
@@ -51,27 +48,16 @@ public:
 #ifndef DEDICATED
     virtual void Render(const eCamera *cam);
 
-    virtual void SoundMix(Uint8 *dest,unsigned int len,
-                          int viewer,REAL rvol,REAL lvol);
+    //virtual void SoundMix(Uint8 *dest,unsigned int len,
+    //                      int viewer,REAL rvol,REAL lvol);
 #endif
 
-    bool AccountForHole(); // will return true exactly once per explosion; to be used to make the holing score only count once.
-
     static void OnNewWall( eWall* w );	// blow holes into a new wall
-
-    // returns the owner
-    gCycle * GetOwner() const
-    {
-        return owner_;
-    }
-
-protected:
-    virtual void OnRemoveFromGame(); // called last when the object is removed from the game
+//#ifdef USEPARTICLES
+    int particle_handle_circle;
+    int particle_handle_cylinder;
+//#endif
 private:
-    eSoundPlayer sound;
-
-    gParticles *theExplosion;
-
     REAL        createTime;
 
     //	gRealColor	color_;
@@ -84,10 +70,6 @@ private:
     static int	expansionSteps;
     static REAL expansionTime;
 
-    bool holeAccountedFor_;  //!< flag indicating whether we already gave the player credit for making a hole for his teammates
-
     int 		listID;
-
-    tJUST_CONTROLLED_PTR< gCycle > owner_; // the owner/victim of the explosion
 };
 #endif
