@@ -20,7 +20,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
 ***************************************************************************
 
 */
@@ -28,6 +28,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "gMenus.h"
 #include "ePlayer.h"
 #include "rScreen.h"
+#include "rTexture.h"
 #include "nConfig.h"
 #include "rConsole.h"
 #include "tToDo.h"
@@ -38,7 +39,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rModel.h"
 #include "gGame.h"
 #include "gCycle.h"
-#include "gHud.h"
 #include "tRecorder.h"
 #include "rSysdep.h"
 
@@ -59,7 +59,6 @@ static uMenuItemFunction defaul
 uMenu screen_menu_detail("$detail_settings_menu");
 uMenu screen_menu_tweaks("$performance_tweaks_menu");
 uMenu screen_menu_prefs("$preferences_menu");
-uMenu hud_prefs("$hud_menu");
 
 static void sg_ScreenModeMenu();
 
@@ -72,10 +71,6 @@ static uMenuItemSubmenu smp(&sg_screenMenu,&screen_menu_prefs,
 static uMenuItemFunction smm(&sg_screenMenu,"$screen_mode_menu",
                              "$screen_mode_menu_help", sg_ScreenModeMenu );
 
-static uMenuItemSubmenu huds(&screen_menu_prefs,&hud_prefs,
-                             "$hud_menu_help");
-
-
 static tConfItemLine c_ext("GL_EXTENSIONS",gl_extensions);
 static tConfItemLine c_ver("GL_VERSION",gl_version);
 static tConfItemLine c_rEnd("GL_RENDERER",gl_renderer);
@@ -83,14 +78,14 @@ static tConfItemLine c_vEnd("GL_VENDOR",gl_vendor);
 // static tConfItemLine a_ver("ARMAGETRON_VERSION",sn_programVersion);
 
 static std::deque<tString> sg_consoleHistory; // global since the class doesn't live beyond the execution of the command
-static int sg_consoleHistoryMaxSize=10; // size of the console history
+static int sg_consoleHistoryMaxSize=100; // size of the console history
 static tSettingItem< int > sg_consoleHistoryMaxSizeConf("HISTORY_SIZE_CONSOLE",sg_consoleHistoryMaxSize);
 
 class ArmageTron_feature_menuitem: public uMenuItemSelection<int>{
-    void NewChoice(uSelectItem<bool> *){}
-    void NewChoice(char *,bool ){}
+    void NewChoice(uSelectItem<bool> *){};
+    void NewChoice(char *,bool ){};
 public:
-    ArmageTron_feature_menuitem(uMenu *m,char const * tit,char const * help,int &targ)
+    ArmageTron_feature_menuitem(uMenu *m,char *tit,const char *help,int &targ)
             :uMenuItemSelection<int>(m,tit,help,targ){
         uMenuItemSelection<int>::NewChoice(
             "$feature_disabled_text",
@@ -106,15 +101,15 @@ public:
             rFEAT_ON);
     }
 
-    ~ArmageTron_feature_menuitem(){}
+    ~ArmageTron_feature_menuitem(){};
 };
 
 
 class ArmageTron_texmode_menuitem: public uMenuItemSelection<int>{
-    void NewChoice(uSelectItem<bool> *){}
-    void NewChoice(char *,bool ){}
+    void NewChoice(uSelectItem<bool> *){};
+    void NewChoice(char *,bool ){};
 public:
-    ArmageTron_texmode_menuitem(uMenu *m,char const * tit,int &targ,
+    ArmageTron_texmode_menuitem(uMenu *m,char *tit,int &targ,
                                 bool font=false)
             :uMenuItemSelection<int>
     (m,tit,"$texture_menuitem_help",targ){
@@ -147,14 +142,16 @@ public:
     #endif
     }
 
-    ~ArmageTron_texmode_menuitem(){}
+    ~ArmageTron_texmode_menuitem(){};
 };
 
+static tConfItem<int>     la("LINE_ANTIALIAS",sr_lineAntialias);
+static tConfItem<int>     pa("POLY_ANTIALIAS",sr_polygonAntialias);
+static tConfItem<int>     pc("PERSP_CORRECT",sr_perspectiveCorrection);
 static tConfItem<bool>    ab("ALPHA_BLEND",sr_alphaBlend);
 static tConfItem<bool>    ss("SMOOTH_SHADING",sr_smoothShading);
 static tConfItem<bool>    to("TEXT_OUT",sr_textOut);
 static tConfItem<bool>    fps("SHOW_FPS",sr_FPSOut);
-static tConfItem<bool>    pbt("SHOW_RECORDING_TIME",sr_RecordingTimeOut);
 // tConfItem<> ("",&);
 static tConfItem<int> fm("FLOOR_MIRROR",sr_floorMirror);
 static tConfItem<int> fd("FLOOR_DETAIL",sr_floorDetail);
@@ -165,10 +162,7 @@ static tConfItem<bool> ls("LOWER_SKY",sr_lowerSky);
 static tConfItem<bool> wos("SKY_WOBBLE",sr_skyWobble);
 static tConfItem<bool> ip("INFINITY_PLANE",sr_infinityPlane);
 
-extern bool sg_axesIndicator;
-
 static tConfItem<bool> lm("LAG_O_METER",sr_laggometer);
-static tConfItem<bool> ai("AXES_INDICATOR",sg_axesIndicator);
 static tConfItem<bool> po("PREDICT_OBJECTS",sr_predictObjects);
 static tConfItem<bool> t32("TEXTURES_HI",sr_texturesTruecolor);
 
@@ -235,16 +229,12 @@ public:
         else
         {
             // add custom resolution
-            NewChoice( ArmageTron_Custom );
-
-            // add desktop resolution
-            if ( sr_DesktopScreensizeSupported() && !addFixed )
-                NewChoice( ArmageTron_Desktop );
+            NewChoice( ArmageTron_Custom );//, "$screen_custom_text","$screen_custom_help" );
 
             // the maximal allowed screen size
             rScreenSize maxSize(0,0);
 
-            // fill in available modes (avoid duplicates)
+            // fill in available modes (avoid dublicates)
             for(i=0;modes[i];++i)
             {
                 // add mode (if it's new)
@@ -259,7 +249,7 @@ public:
             // add fixed resolutions (as window sizes)
             if ( addFixed )
             {
-                for ( i = ArmageTron_Custom; i>=ArmageTron_Min; --i )
+                for ( i = ArmageTron_Custom; i>=0; --i )
                 {
                     rScreenSize size( static_cast< rResolution >(i) );
 
@@ -276,12 +266,8 @@ public:
                 rScreenSize const & size = *iter;
 
                 std::stringstream s;
-                if ( size.width + size.height > 0 )
-                    s << size.width << " x " << size.height;
-                else
-                    s << tOutput("$screen_size_desktop");
-
-                res_men.NewChoice( s.str().c_str(), "", size );
+                s << size.width << " x " << size.height;
+                res_men.NewChoice( s.str().c_str(), help, size );
             }
 
 #endif
@@ -408,16 +394,6 @@ static uMenuItemToggle fs_dither
  "$detail_dither_help",
  sr_dither);
 
-#ifndef DEDICATED
-// from gWall.cpp
-extern bool sg_simpleTrail;
-static uMenuItemToggle sgm_simpleTrail
-(&screen_menu_detail,
- "$detail_simple_trail_text",
- "$detail_simple_trail_help",
- sg_simpleTrail);
-#endif
-
 static uMenuItemSelection<int> mfd
 (&screen_menu_detail,
  "$detail_floor_text",
@@ -437,6 +413,17 @@ static uSelectEntry<int> mfdd(mfd,"$detail_floor_2tex_text",
                               "$detail_floor_2tex_help",
                               rFLOOR_TWOTEXTURE);
 
+
+static ArmageTron_feature_menuitem pam
+(&screen_menu_detail,"$detail_polyantialias_text",
+ "$detail_polyantialias_help",
+ sr_polygonAntialias);
+
+static ArmageTron_feature_menuitem lam
+(&screen_menu_detail,"$detail_lineantialias_text",
+ "$detail_lineantialias_help",
+ sr_lineAntialias);
+
 static uMenuItemToggle  abm
 (&screen_menu_detail,"$detail_alpha_text",
  "$detail_alpha_help",
@@ -447,6 +434,11 @@ static uMenuItemToggle  ssm
  "$detail_smooth_help",
  sr_smoothShading);
 
+static ArmageTron_feature_menuitem pcm
+(&screen_menu_detail,"$detail_persp_text",
+ "$detail_persp_help",
+ sr_perspectiveCorrection);
+
 extern bool crash_sparks;		// from gCycle.cpp
 extern bool white_sparks;		// from gSparks.cpp
 static tConfItem<bool> cs2("SPARKS",crash_sparks);
@@ -458,6 +450,7 @@ static tConfItem<bool> crexp("EXPLOSION",sg_crashExplosion);
 #ifndef DEDICATED
 //extern bool png_screenshot;		// from rSysdep.cpp
 //static tConfItem<bool> pns("PNG_SCREENSHOT",png_screenshot);
+#endif
 
 static uMenuItemToggle  t32b
 (&screen_menu_detail,"$detail_text_truecolor_text",
@@ -482,10 +475,6 @@ static ArmageTron_texmode_menuitem tmm3(&screen_menu_detail,
                                         rTextureGroups::TextureMode[3],true);
 
 
-uMenuItemToggle bpt2
-(&screen_menu_prefs,"$misc_recording_time_text",
- "$misc_recording_time_help",sr_RecordingTimeOut);
-
 static uMenuItemToggle s2
 (&screen_menu_prefs,"$pref_highrim_text",
  "$pref_highrim_help",sr_highRim);
@@ -525,12 +514,13 @@ static uMenuItemToggle cs
  "$pref_sparks_help",
  crash_sparks);
 
-static uMenuItemSelection<rDisplayListUsage> dl
+static uMenuItemToggle dl
 (&screen_menu_tweaks,"$tweaks_displaylists_text",
  "$tweaks_displaylists_help", sr_useDisplayLists);
-static uSelectEntry<rDisplayListUsage> dl_off(dl,"$tweaks_displaylists_off_text","$tweaks_displaylists_off_help",rDisplayList_Off);
-static uSelectEntry<rDisplayListUsage> dl_cac(dl,"$tweaks_displaylists_cac_text","$tweaks_displaylists_cac_help",rDisplayList_CAC);
-static uSelectEntry<rDisplayListUsage> dl_cae(dl,"$tweaks_displaylists_cae_text","$tweaks_displaylists_cae_help",rDisplayList_CAE);
+
+static uMenuItemToggle zt
+(&screen_menu_tweaks,"$tweaks_ztrick_text",
+ "$tweaks_ztrick_help",sr_ZTrick);
 
 static uMenuItemToggle infp
 (&screen_menu_tweaks,"$tweaks_infinity_text",
@@ -560,134 +550,88 @@ static uSelectEntry<rSysDep::rSwapMode> swapMode_80Hz(swapMode,"$swapmode_80hz_t
 static uSelectEntry<rSysDep::rSwapMode> swapMode_60Hz(swapMode,"$swapmode_60hz_text","$swapmode_60hz_help",rSysDep::rSwap_60Hz);
 */
 
-tCONFIG_ENUM( rSysDep::rSwapMode );
-
-static tConfItem< rSysDep::rSwapMode > swapModeCI("SWAP_MODE", rSysDep::swapMode_ );
-
-static tConfItem<REAL> sgs("SPEED_GAUGE_SIZE",subby_SpeedGaugeSize);
-static tConfItem<REAL> sgx("SPEED_GAUGE_LOCX",subby_SpeedGaugeLocX);
-static tConfItem<REAL> sgy("SPEED_GAUGE_LOCY",subby_SpeedGaugeLocY);
-
-static tConfItem<REAL> bgs("BRAKE_GAUGE_SIZE",subby_BrakeGaugeSize);
-static tConfItem<REAL> bgx("BRAKE_GAUGE_LOCX",subby_BrakeGaugeLocX);
-static tConfItem<REAL> bgy("BRAKE_GAUGE_LOCY",subby_BrakeGaugeLocY);
-
-static tConfItem<REAL> rgs("RUBBER_GAUGE_SIZE",subby_RubberGaugeSize);
-static tConfItem<REAL> rgx("RUBBER_GAUGE_LOCX",subby_RubberGaugeLocX);
-static tConfItem<REAL> rgy("RUBBER_GAUGE_LOCY",subby_RubberGaugeLocY);
-
-static tConfItem<bool> showh("SHOW_HUD",subby_ShowHUD);
-static tConfItem<bool> showf("SHOW_FASTEST",subby_ShowSpeedFastest);
-static tConfItem<bool> shows("SHOW_SCORE",subby_ShowScore);
-static tConfItem<bool> showae("SHOW_ALIVE",subby_ShowAlivePeople);
-static tConfItem<bool> showp("SHOW_PING",subby_ShowPing);
-static tConfItem<bool> showsm("SHOW_SPEED",subby_ShowSpeedMeter);
-static tConfItem<bool> showbm("SHOW_BRAKE",subby_ShowBrakeMeter);
-static tConfItem<bool> showrm("SHOW_RUBBER",subby_ShowRubberMeter);
-static tConfItem<bool> showtim("SHOW_TIME",showTime);
-static tConfItem<bool> show24("SHOW_TIME_24",show24hour);
-
-static tConfItem<REAL> scorex("SCORE_LOCX",subby_ScoreLocX);
-static tConfItem<REAL> scorey("SCORE_LOCY",subby_ScoreLocY);
-static tConfItem<REAL> scores("SCORE_SIZE",subby_ScoreSize);
-
-static tConfItem<REAL> fastx("FASTEST_LOCX",subby_FastestLocX);
-static tConfItem<REAL> fasty("FASTEST_LOCY",subby_FastestLocY);
-static tConfItem<REAL> fasts("FASTEST_SIZE",subby_FastestSize);
-
-static tConfItem<REAL> aex("ALIVE_LOCX",subby_AlivePeopleLocX);
-static tConfItem<REAL> aey("ALIVE_LOCY",subby_AlivePeopleLocY);
-static tConfItem<REAL> aes("ALIVE_SIZE",subby_AlivePeopleSize);
-
-static tConfItem<REAL> px("PING_LOCX",subby_PingLocX);
-static tConfItem<REAL> py("PING_LOCY",subby_PingLocY);
-static tConfItem<REAL> ps("PING_SIZE",subby_PingSize);
-
-uMenuItemToggle hud3
-(&hud_prefs,"$pref_showfastest_text",
- "$pref_showfastest_help",subby_ShowSpeedFastest);
-
-uMenuItemToggle mud4
-(&hud_prefs,"$pref_showscore_text",
- "$pref_showscore_help",subby_ShowScore);
-
-uMenuItemToggle hud5
-(&hud_prefs,"$pref_showenemies_text",
- "$pref_showenemies_help",subby_ShowAlivePeople);
-
-uMenuItemToggle hud6
-(&hud_prefs,"$pref_showping_text",
- "$pref_showping_help",subby_ShowPing);
-
-uMenuItemToggle hud7
-(&hud_prefs,"$pref_showspeed_text",
- "$pref_showspeed_help",subby_ShowSpeedMeter);
-
-uMenuItemToggle hud8
-(&hud_prefs,"$pref_showbrake_text",
- "$pref_showbrake_help",subby_ShowBrakeMeter);
-
-uMenuItemToggle hud9
-(&hud_prefs,"$pref_showrubber_text",
- "$pref_showrubber_help",subby_ShowRubberMeter);
-
-uMenuItemToggle hud10
-(&hud_prefs,"$pref_showtime_text",
- "$pref_showtime_help",showTime);
-
-uMenuItemToggle hud11
-(&hud_prefs,"$pref_show24hour_text",
- "$pref_show24hour_help",show24hour);
-
-uMenuItemToggle hud2
-(&hud_prefs,"$pref_showhud_text",
- "$pref_showhud_help",subby_ShowHUD);
+static tConfItem<int> swapModeCI("SWAP_MODE", reinterpret_cast< int & >( rSysDep::swapMode_ ) );
 
 static tConfItem<bool> WRAP("WRAP_MENU",uMenu::wrap);
 
-class gMemuItemConsole: uMenuItemStringWithHistory{
+
+
+#ifndef DEDICATED
+
+class gAutoCompleterConsole : public uAutoCompleter {
 public:
-    gMemuItemConsole(uMenu *M,tString &c):
-    uMenuItemStringWithHistory(M,"Con:","", c, 1024, sg_consoleHistory, sg_consoleHistoryMaxSize) {}
-
-    virtual ~gMemuItemConsole(){}
-
-    //virtual void Render(REAL x,REAL y,REAL alpha=1,bool selected=0);
-
-    virtual bool Event(SDL_Event &e){
-        if (e.type==SDL_KEYDOWN &&
-                (e.key.keysym.sym==SDLK_KP_ENTER || e.key.keysym.sym==SDLK_RETURN)){
-
-            con << tColoredString::ColorString(.5,.5,1) << " > " << *content << '\n';
-
-            // direct commands are executed at owner level
-            tCurrentAccessLevel level( tAccessLevel_Owner, true );
-
-            // pass the console command to the configuration system
-            std::stringstream s(&((*content)[0]));
-            tConfItemBase::LoadAll( s, false );
-
-            MyMenu()->Exit();
-            return true;
+    int Complete(tString &string, unsigned pos) {
+        if(string.find_first_of(' ') == pos - 1) {
+            tConfItemBase *cfg = tConfItemBase::FindConfigItem(string.substr(0, pos-1));
+            if(cfg != 0) {
+                std::ostringstream toAdd("");
+                cfg->WriteVal(toAdd);
+                string << toAdd.str();
+                return string.size();
+            }
         }
-        else if (e.type==SDL_KEYDOWN &&
-                 uActionGlobal::IsBreakingGlobalBind(e.key.keysym.sym))
-            return su_HandleEvent(e, true);
-        else
-            return uMenuItemStringWithHistory::Event(e);
+        //delegate
+        return uAutoCompleter::Complete(string, pos);
     }
+    gAutoCompleterConsole(std::deque<tString> &words) : uAutoCompleter(words) {}
 };
 
+//! Handles the console prompt
+class gMemuItemConsole: uMenuItemStringWithHistory{
+public:
+    gMemuItemConsole(uMenu *M,tString &c, uAutoCompleter *completer); //!< Constructor
+    virtual ~gMemuItemConsole(){}
+    bool Event(SDL_Event &e);
+};
+
+//! @param M         passed on to uMenuItemStringWithHistory
+//! @param c         passed on to uMenuItemStringWithHistory
+//! @param completer the completer to be used
+gMemuItemConsole::gMemuItemConsole(uMenu *M,tString &c, uAutoCompleter *completer):
+        uMenuItemStringWithHistory(M,"Con:","", c, 1024, sg_consoleHistory, sg_consoleHistoryMaxSize, completer)
+{}
+
+
+bool gMemuItemConsole::Event(SDL_Event &e){
+    if (e.type==SDL_KEYDOWN &&
+            (e.key.keysym.sym==SDLK_KP_ENTER || e.key.keysym.sym==SDLK_RETURN)){
+
+        con << tColoredString::ColorString(.5,.5,1) << " > " << *content << '\n';
+
+        if ( tRecorder::IsPlayingBack() )
+        {
+            // the command was also recorded; better play it back,
+            // or the playback gets out of sync
+            tConfItemBase::LoadPlayback( false );
+        }
+        else
+        {
+            // pass the console command to the configuration system
+            std::stringstream s(&((*content)[0]));
+            tConfItemBase::LoadAll(s);
+        }
+
+        MyMenu()->Exit();
+        return true;
+    }
+    else if (e.type==SDL_KEYDOWN &&
+             uActionGlobal::IsBreakingGlobalBind(e.key.keysym.sym))
+        return su_HandleEvent(e, true);
+    else
+        return uMenuItemStringWithHistory::Event(e);
+}
+
 void do_con(){
-    su_ClearKeys();
-        
     se_ChatState( ePlayerNetID::ChatFlags_Console, true );
     sr_con.SetHeight(20,false);
     se_SetShowScoresAuto(false);
     tString c;
 
     uMenu con_menu("",false);
-    gMemuItemConsole s(&con_menu,c);
+    std::deque<tString> commands;
+    commands = tConfItemBase::GetCommands();
+    gAutoCompleterConsole completer(commands);
+    gMemuItemConsole s(&con_menu,c,&completer);
     con_menu.SetCenter(-.75);
     con_menu.SetBot(-2);
     con_menu.SetTop(-.7);
@@ -702,7 +646,7 @@ void do_con(){
 
 void sg_ConsoleInput(){
 #ifndef DEDICATED
-    st_ToDoOnce(&do_con);
+    st_ToDo(&do_con);
 #endif
 }
 
@@ -830,7 +774,7 @@ public:
         m->RequestSpaceBelow(.2);
     }
 
-    ~ArmageTron_color_menuitem(){}
+    ~ArmageTron_color_menuitem(){};
 
     virtual REAL SpaceRight(){return .2;}
 
@@ -847,14 +791,13 @@ public:
         }
         */
 #ifndef DEDICATED
-        uMenuItem::RenderBackground();
         if (!sr_glOut)
             return;
+        uMenuItem::RenderBackground();
         REAL r = rgb[0]/15.0;
         REAL g = rgb[1]/15.0;
         REAL b = rgb[2]/15.0;
         se_MakeColorValid(r, g, b, 1.0f);
-        RenderEnd();
         glColor3f(r, g, b);
         glRectf(.8,-.8,.98,-.98);
 #endif
@@ -876,7 +819,6 @@ void sg_PlayerMenu(int Player){
     uMenu camera_menu("$player_camera_text");
     uMenu chat_menu("$player_chat_text");
     //  name.Clear();
-    chat_menu.SetCenter(-.5);
 
     uMenuItemString *ic[MAX_INSTANT_CHAT];
 
@@ -895,21 +837,6 @@ void sg_PlayerMenu(int Player){
                p->instantChatString[i], se_SpamMaxLen);
     }
 
-    uMenuItemToggle al
-    (&playerMenu,"$player_autologin_text",
-     "$player_autologin_help",
-     p->autoLogin);
-
-    uMenuItemString gid(&playerMenu,
-                      "$player_global_id_text",
-                      "$player_global_id_help",
-                      p->globalID, 400);
-    gid.SetColorMode( rTextField::COLOR_IGNORE );
-
-    uMenuItemToggle st
-    (&playerMenu,"$player_stealth_text",
-     "$player_stealth_help",
-     p->stealth);
 
     uMenuItemToggle sp
     (&playerMenu,"$player_spectator_text",
@@ -1020,7 +947,7 @@ void sg_PlayerMenu(int Player){
     (&camera_menu,
      "$player_camera_fov_text",
      "$player_camera_fov_help",
-     p->startFOV,30,160,5);
+     p->startFOV,30,120,5);
 
     uMenuItemSelection<eCamMode> cam_s
     (&camera_menu,
@@ -1035,6 +962,11 @@ void sg_PlayerMenu(int Player){
     cam_s.NewChoice("$player_camera_initial_ext_text","$player_camera_initial_ext_help",CAMERA_FOLLOW);
     cam_s.NewChoice("$player_camera_initial_free_text","$player_camera_initial_free_help",CAMERA_FREE);
 
+    uMenuItemString tn(&playerMenu,
+                       "$player_teamname_text",
+                       "$player_teamname_help",
+                       p->teamname, 16);
+
     uMenuItemString n(&playerMenu,
                       "$player_name_text",
                       "$player_name_help",
@@ -1049,10 +981,7 @@ void sg_PlayerMenu(int Player){
     // request network synchronisation if the server can handle it
     static nVersionFeature inGameRenames( 5 );
     if ( inGameRenames.Supported() )
-    {
         ePlayerNetID::Update();
-        ePlayer::SendAuthNames();
-    }
 
     /*
     for (i=MAX_PLAYERS-1; i>=0; i--)
@@ -1167,20 +1096,11 @@ static bool con_func(REAL x){
 
 static bool toggle_fullscreen_func( REAL x )
 {
-#ifndef DEDICATED
-#ifdef DEBUG
-    // don't toggle fullscreen while playing back in debug mode, that's annoying
-    if ( tRecorder::IsPlayingBack() )
-        return true;
-#endif
-
-    // only do anything if the application is active (work around odd bug)
-    if ( x > 0 && ( SDL_GetAppState() & SDL_APPACTIVE ) )
+    if ( x > 0 )
     {
         currentScreensetting.fullscreen = !currentScreensetting.fullscreen;
         sr_ReinitDisplay();
     }
-#endif
 
     return true;
 }
