@@ -20,7 +20,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
 ***************************************************************************
 
 */
@@ -28,13 +28,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef ArmageTron_gSparks_H
 #define ArmageTron_gSparks_H
 
-#include "eSound.h"
+#include "config.h"
+
+#include "eSoundMixer.h"
 #include "eGameObject.h"
 #include "rModel.h"
 
-class gSpark: public eReferencableGameObject{ // When the player nearly hits a eWall
+class gSpark: public eGameObject{ // When the player nearly hits a eWall
     REAL createTime;
 
+#ifndef USEPARTICLES
 #define SPARKS 10
 
     Vec3 x         [SPARKS];
@@ -49,8 +52,14 @@ class gSpark: public eReferencableGameObject{ // When the player nearly hits a e
     REAL sparkenemycolor_r;
     REAL sparkenemycolor_g;
     REAL sparkenemycolor_b;
+#endif
 
 private:
+#ifdef USEPARTICLES
+    int particle_handle;
+#define SPARKS 100
+#endif
+
 public:
     gSpark(eGrid *grid, const eCoord &pos,const eCoord &dir,REAL time,REAL ocolor_r,REAL ocolor_g,REAL ocolor_b,REAL ecolor_r,REAL ecolor_g,REAL ecolor_b);
     virtual ~gSpark();
@@ -66,8 +75,8 @@ public:
 #ifndef DEDICATED
     virtual void Render(const eCamera *cam);
 
-    virtual void SoundMix(Uint8 *dest,unsigned int len,
-                          int viewer,REAL rvol,REAL lvol);
+    //virtual void SoundMix(Uint8 *dest,unsigned int len,
+    //                      int viewer,REAL rvol,REAL lvol);
 #endif
 };
 
