@@ -32,8 +32,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <vector>
 
-// get rid of this include (it's needed for gColor)
 #include "gCycle.h"
+#include "rColor.h"
 
 
 class eTeam;
@@ -90,12 +90,14 @@ public:
     gZone const &   GetExpansionSpeed   ( REAL & expansionSpeed ) const;//!< Gets the current expansion speed
     gZone &         SetRotationSpeed    ( REAL rotationSpeed );	        //!< Sets the current rotation speed
     REAL            GetRotationSpeed    ( void ) const;	                //!< Gets the current rotation speed
+    tCoord const &  GetRotation    ( void ) const;	                //!< Gets the current rotation state
     gZone const &   GetRotationSpeed    ( REAL & rotationSpeed ) const;	//!< Gets the current rotation speed
     gZone &         SetRotationAcceleration( REAL rotationAcceleration );	        //!< Sets the current acceleration of the rotation
     REAL            GetRotationAcceleration( void ) const;	                        //!< Gets the current acceleration of the rotation
     gZone const &   GetRotationAcceleration( REAL & rotationAcceleration ) const;	//!< Gets the current acceleration of the rotation
+    rColor const &  GetColor( void ) const;	//!< Gets the current color
 protected:
-    gRealColor color_;           //!< the zone's color
+    rColor color_;           //!< the zone's color
     REAL createTime_;            //!< the time the zone was created at
 
     REAL referenceTime_;         //!< reference time for function evaluations
