@@ -1,29 +1,29 @@
 /*
-
+ 
 *************************************************************************
-
+ 
 ArmageTron -- Just another Tron Lightcycle Game in 3D.
 Copyright (C) 2000  Manuel Moos (manuel@moosnet.de)
 Copyright (C) 2004  Armagetron Advanced Team (http://sourceforge.net/projects/armagetronad/)
-
+ 
 **************************************************************************
-
+ 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
-
+ 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
-
+ 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-
+  
 ***************************************************************************
-
+ 
 */
 
 #include "tSysTime.h"
@@ -31,6 +31,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rSysdep.h"
 #include "rScreen.h"
 #include "rViewport.h"
+#include "rTexture.h"
 #include "tString.h"
 #include "math.h"
 #include "uInputQueue.h"
@@ -46,12 +47,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rSDL.h"
 #endif
 
-#include <vector>
-
 FUNCPTR  uMenu::idle(NULL);
 
 bool uMenu::wrap=true;
-uMenu::QuickExit uMenu::quickexit=uMenu::QuickExit_Off;
+bool uMenu::quickexit=false;
 bool uMenu::exitToMain=false;
 
 // *****************************************************
@@ -64,7 +63,7 @@ uMenu::uMenu(const char *t="",bool exit_item)
     menuTop=.7;
     menuBot=-.7;
     yOffset=0;
-    selected = 10000000;
+    selected = -1;
 }
 #endif
 
@@ -75,17 +74,17 @@ uMenu::uMenu(const tOutput &t,bool exit_item)
     menuTop=.7;
     menuBot=-.7;
     yOffset=0;
-    selected = 100000000;
+    selected = -1;
 }
 
 uMenu::~uMenu(){
-    for (int i=items.Len()-1;i>=0;i--)
+    for(int i=items.Len()-1;i>=0;i--)
         delete items[i];
 }
 
 void uMenu::ReverseItems(){
-    tList<uMenuItem> dummy;
-    dummy.Swap( items );
+    tList<uMenuItem> dummy = items;
+    items.SetLen(0);
 
     for (int i=dummy.Len()-1; i>=0; i--){
         uMenuItem *x = dummy[i];
@@ -98,9 +97,6 @@ void uMenu::ReverseItems(){
 //static REAL text_width=rCWIDTH_NORMAL;
 
 static REAL text_height=.11;
-#ifndef DEDICATED
-static REAL text_width=.05;
-#endif
 
 #ifndef DEDICATED
 static REAL titlefac=1.2;
@@ -128,7 +124,7 @@ static inline void arrow(REAL x,REAL y,REAL dy,REAL size){
 #endif
 }
 
-static bool s_globalRepeat = false;
+static bool repeat = false;
 
 #ifndef DEDICATED
 static bool disphelp=false;
@@ -146,12 +142,9 @@ static rNoAutoDisplayAtNewlineCallback su_noNewline( uMenu::MenuActive );
 
 void uMenu::OnEnter(){
 #ifndef DEDICATED
-    bool localRepeat = false;
     float nextrepeat = 0.0f;
-    static const float repeatdelay = 0.2f;
-    static const float repeatrateStart  = 0.2f;
-    static const float repeatrateMin  = 0.05f;
-    static float repeatrate  = repeatrateStart;
+    static const float repeatdelay = 0.3f;
+    static const float repeatrate  = 0.05f;
     SDL_Event tEventRepeat;
 #else
     return;
@@ -173,49 +166,41 @@ void uMenu::OnEnter(){
 
 #ifndef DEDICATED
     lastkey=tSysTimeFloat();
-    static const REAL timeout=.5;
+    static const REAL timeout=3;
 #endif
-
+    // inverted logic (0 = last item! prev(0) = top most item)
+    selected = GetPrevSelectable(0);
     while (!exitFlag && !quickexit && !exitToMain){
         st_DoToDo();
         tAdvanceFrame();
 
+        if (selected < 0 || selected >= items.Len())
+            selected = items.Len()-1;
+
         ts=tSysTimeFloat()-lastt;
         lastt=tSysTimeFloat();
         if (ts>.2) ts=.2;
 
         menuentries=items.Len();
 
-        // clamp cursor
-        if (selected < 0 )
-            selected = 0;
-        if ( selected >= items.Len())
-            selected = items.Len()-1;
 
 #ifndef DEDICATED
         {
             SDL_Event tEvent;
             uInputProcessGuard inputProcessGuard;
-            while (su_GetSDLInput(tEvent))
+            while(su_GetSDLInput(tEvent))
             {
                 REAL entertime = tSysTimeFloat();
 
                 switch (tEvent.type)
                 {
                 case SDL_KEYDOWN:
-                    if ( tEvent.key.keysym.sym == SDLK_UNKNOWN )
-                    {
-                        // don't repeat unknown syms. They come from multi-key compositions and
-                        // don't send keyup events when released.
-                        break;
-                    }
-                    localRepeat = s_globalRepeat = true;
+                    repeat = true;
                     memcpy( &tEventRepeat, &tEvent, sizeof( SDL_Event ) );
                     nextrepeat = tSysTimeFloat() + repeatdelay;
                     break;
                 case SDL_KEYUP:
-                    localRepeat = s_globalRepeat = false;
-                    repeatrate = repeatrateStart;
+                    repeat = false;
                     break;
                 }
 
@@ -227,28 +212,16 @@ void uMenu::OnEnter(){
 
                 if ( tSysTimeFloat() - entertime > 1 )
                 {
-                    localRepeat = s_globalRepeat = false;
+                    repeat = false;
                 }
             }
 
-            if ( localRepeat && s_globalRepeat && tSysTimeFloat() > nextrepeat )
+            if ( repeat && tSysTimeFloat() > nextrepeat )
             {
                 this->HandleEvent( tEventRepeat );
                 nextrepeat = tSysTimeFloat() + repeatrate;
-                repeatrate *= .71;
-                if ( repeatrate < repeatrateMin )
-                    repeatrate = repeatrateMin;
             }
         }
-
-        // we're about to render, last chance to make changes to the menu
-        OnRender();
-
-        // clamp cursor
-        if (selected < 0 )
-            selected = 0;
-        if ( selected >= items.Len())
-            selected = items.Len()-1;
 #endif
         // quit shortcut
         if ( quickexit )
@@ -283,6 +256,7 @@ void uMenu::OnEnter(){
             yOffset+=menuTop-smallborder-YPos(menuentries-1);
 
 #ifndef DEDICATED
+        rSysDep::ClearGL();
         sr_ResetRenderState(true);
         items[selected]->RenderBackground();
 
@@ -293,14 +267,14 @@ void uMenu::OnEnter(){
         if (sr_glOut && !exitFlag && !quickexit){
             items[selected]->Render(center,YPos(selected),1,true);
 
-            for (int i=items.Len()-1;i>=0;i--)
-                if (i!=selected){
+            for(int i=items.Len()-1;i>=0;i--)
+                if(i!=selected){
                     REAL y=YPos(i);
                     REAL alpha=1;
                     const REAL b=.1;
-                    if (y<menuBot+b)
+                    if(y<menuBot+b)
                         alpha=(y-menuBot)/b;
-                    if (y>menuTop-b)
+                    if(y>menuTop-b)
                         alpha=(menuTop-y)/b;
                     if (y>menuBot && y<menuTop)
                     {
@@ -315,8 +289,8 @@ void uMenu::OnEnter(){
 
             Color(.6,.6,1,1);
             ::DisplayText(0,menuTop+text_height*titlefac
-                          ,text_width*titlefac*rTextField::AspectWidthMultiplier(),text_height*titlefac,
-                          title,0);
+                          ,text_height*titlefac,
+                          title,sr_fontMenuTitle,0);
 
             glDisable(GL_TEXTURE_2D);
             //glDisable(GL_TEXTURE);
@@ -326,41 +300,34 @@ void uMenu::OnEnter(){
             if (YPos(menuentries-1)>menuTop && (int(tSysTimeFloat())+1)%2)
                 arrow(.9,menuTop,1,.05);
 
-            REAL helpAlpha = tSysTimeFloat()-lastkey-timeout;
-            if( helpAlpha > 1 )
-            {
-                helpAlpha = 1;
-            }
-            
-            disphelp = helpAlpha > 0;
-            if ( items[selected]->DisplayHelp( disphelp, menuBot, helpAlpha ) )
-            {
+            if (tSysTimeFloat()-lastkey>timeout){
+                disphelp=true;
                 if (sr_alphaBlend)
-                    glColor4f(1,.8,.8, helpAlpha );
+                    glColor4f(1,.8,.8,tSysTimeFloat()-lastkey-timeout);
                 else
-                    Color(helpAlpha,
-                          .8*helpAlpha,
-                          .8*helpAlpha);
+                    Color(tSysTimeFloat()-lastkey-timeout,
+                          .8*(tSysTimeFloat()-lastkey-timeout),
+                          .8*(tSysTimeFloat()-lastkey-timeout));
 
-                rTextField c(-.95f,menuBot-.04f,rCWIDTH_NORMAL*rTextField::AspectWidthMultiplier());
-                c.SetWidth(static_cast<int>((1.9f-items[selected]->SpaceRight())/c.GetCWidth()));
+                rTextField c(-.95f,menuBot-.04f,rCHEIGHT_NORMAL, sr_fontMenu);
+                c.SetWidth(1.9f-items[selected]->SpaceRight());
+                c.EnableLineWrap();
                 c << items[selected]->Help();
             }
+            else disphelp=false;
         }
         else
 #endif
-            if ( !sr_glOut )
-            {
-                tDelay( 10000 );
-            }
+        {
+            tDelay( 100000 );
+        }
 
 #ifndef DEDICATED
         rSysDep::SwapGL();
-        rSysDep::ClearGL();
 #endif
     }
 
-    s_globalRepeat = false;
+    repeat = false;
 
     uCallbackMenuLeave::MenuLeave();
     su_inMenu = false;
@@ -371,6 +338,7 @@ void uMenu::HandleEvent( SDL_Event event )
 #ifndef DEDICATED
     if (!items[selected]->Event(event))
     {
+        // int newSelected = -1;
         switch (event.type){
         case SDL_KEYDOWN:
         {
@@ -379,34 +347,18 @@ void uMenu::HandleEvent( SDL_Event event )
             switch (event.key.keysym.sym){
 
             case(SDLK_ESCAPE):
-                s_globalRepeat = false;
+                            repeat = false;
                 lastkey=tSysTimeFloat();
                 Exit();
                 break;
 
             case(SDLK_UP):
                             lastkey=tSysTimeFloat();
-                selected++;
-                if (selected>=items.Len())
-                {
-                    if (wrap)
-                        selected=0;
-                    else
-                        selected=items.Len()-1;
-                }
+                selected = GetNextSelectable(selected);
                 break;
-
             case(SDLK_DOWN):
                             lastkey=tSysTimeFloat();
-                selected--;
-                if (selected<0)
-                {
-                    if (wrap)
-                        selected=items.Len()-1;
-                    else
-                        selected=0;
-                }
-
+                selected = GetPrevSelectable(selected);
                 break;
 
             case(SDLK_LEFT):
@@ -419,7 +371,7 @@ void uMenu::HandleEvent( SDL_Event event )
             case(SDLK_SPACE):
                         case(SDLK_KP_ENTER):
                             case(SDLK_RETURN):
-                                    s_globalRepeat = false;
+                                    repeat = false;
                 try
         {
                     su_inMenu = false;
@@ -436,13 +388,13 @@ void uMenu::HandleEvent( SDL_Event event )
 #pragma warning ( disable : 4286 )
                 // GRR. Visual C++ dones not handle generic exceptions with the above general statement.
                 // A specialized version is needed. The best part: it warns about the code below being redundant.
-                catch ( tGenericException const & e )
+                catch( tGenericException const & e )
                 {
                     try
                     {
                         tConsole::Message( e.GetName(), e.GetDescription(), 20 );
                     }
-                    catch (...)
+                    catch(...)
                     {
                     }
                 }
@@ -450,7 +402,7 @@ void uMenu::HandleEvent( SDL_Event event )
 
                 su_inMenu = true;
 
-                s_globalRepeat = false;
+                repeat = false;
                 lastkey=tSysTimeFloat();
                 break;
 
@@ -472,67 +424,67 @@ void uMenu::HandleEvent( SDL_Event event )
 #endif
 }
 
-#ifndef DEDICATED
-static bool s_idleBackground = false;
-#endif
+// select the menu item below "start"
+int uMenu::GetPrevSelectable(int start)
+{
+    int prev = start-1;
+    while (prev!=start)
+    {
+        if (prev<0)
+        {
+            if (wrap)
+                prev = items.Len()-1;
+            else
+                break;
+        }
+        if (items[prev]->IsSelectable())
+        {
+            return prev;
+        }
+        prev--;
+    }
+    return start;
+}
+
+// select the menu item above "start"
+int uMenu::GetNextSelectable(int start)
+{
+    int next = start+1;
+    while (next!=start)
+    {
+        if (next>=items.Len())
+        {
+            if (this->wrap)
+                next = 0;
+            else
+                break;
+        }
+        if (items[next]->IsSelectable())
+        {
+            return next;
+        }
+        next++;
+    }
+    return start;
+}
+
 
 // paints a nice background
-void uMenu::GenericBackground(REAL top){
+void uMenu::GenericBackground(){
 #ifndef DEDICATED
     if (idle)
     {
-        s_idleBackground = true;
-
         try
         {
             // throw tGenericException("test"); // (test exception throw to see if error handling works right)
             (*idle)();
-
-            // render the console so it appears behind the menu
-            if( sr_con.autoDisplayAtSwap )
-            {
-                sr_con.Render();
-            }
-
-            // fade everything rendered so far to black
-            if( sr_alphaBlend )
-            {
-                sr_ResetRenderState(true);
-
-                double time = tSysTimeFloat();
-                static double lastTime = time - 100;
-                static REAL alpha = 0.0f;
-                double timePassed = time - lastTime;
-                if( time - lastTime > 1.0 )
-                {
-                    alpha = 0.0f;
-                }
-                else
-                {
-                    alpha += timePassed;
-                    static const REAL limit = .5;
-
-                    if( alpha > limit )
-                    {
-                        alpha = limit;
-                    }
-                }
-                lastTime = time;
-
-                RenderEnd();
-                glColor4f(0, 0, 0, alpha);
-                glRectf(-1,-1,1,top);
-            }
         }
-        catch ( ... )
+        catch( ... )
         {
-            s_idleBackground = false;
-
             // the idle background function is broken. Disable it and rethrow.
             idle = 0;
             throw;
         }
-        s_idleBackground = false;
     }
     else if (sr_glOut){
         uCallbackMenuBackground::MenuBackground();
@@ -548,11 +500,6 @@ void uMenu::OnExit(){
     exitFlag=1;
 }
 
-//! called every frame before the menu is rendered
-void uMenu::OnRender()
-{
-}
-
 // *****************************************************
 
 // *******************************************************************************************
@@ -571,7 +518,7 @@ void uMenuItem::SetColor( bool selected, REAL alpha )
     //   rTextField::SetBlendColor( tColor(.8+.2*sin(time),.3-.1*sin(time),.3-.1*sin(time),alpha) );
     rTextField::SetDefaultColor( tColor(1,1,1,alpha) );
 
-    if (selected)
+    if(selected)
     {
         REAL time=tSysTimeFloat()*10;
         REAL intensity = 1+.3*sin(time);
@@ -582,33 +529,25 @@ void uMenuItem::SetColor( bool selected, REAL alpha )
 
 void uMenuItem::DisplayText(REAL x,REAL y,const char *text,
                             bool selected,REAL alpha,
-                            int center,int c,int cp, rTextField::ColorMode colorMode ){
+                            int center,int c,int cp, float maxWidth){
 #ifndef DEDICATED
     if (sr_glOut){
         SetColor( selected, alpha );
 
-        REAL tw = text_width;
         REAL th = text_height;
-        
-        //aspect ratio correction
-        tw *= (REAL(sr_screenHeight)/sr_screenWidth)*(4.0/3.0);
 
-        
-#if 0
-        // the function that is called takes care of that
         REAL availw = 1.9f;
         if (center < 0) availw = (.9f-x);
         if (center > 0) availw = (x + .9f);
+        if (availw > maxWidth) availw = maxWidth;
 
-        int len = strlen(text);
-        if (len * tw > availw)
+        float usedwidth=rTextField::GetTextLength(tString(text), th, true);
+        if (usedwidth > availw)
         {
-            th *= availw/(len * tw);
-            tw  = availw/len;
+            th *= availw/(usedwidth);
         }
-#endif
 
-        ::DisplayText(x,y,tw,th,text,center,c,cp, colorMode );
+        ::DisplayText(x,y,th,text,sr_fontMenu,center,c,cp);
     }
 #endif
 }
@@ -688,7 +627,7 @@ uMenuItemInt::uMenuItemInt
 (uMenu *m,const char *tit,const char *help,int &targ,
  int mi,int ma,int step)
         :uMenuItem(m,help),title(tit),target(targ),Min(mi),Max(ma),
-        Step(step){
+Step(step){
     if (target<Min) target=Min;
     if (target>Max) target=Max;
 }
@@ -698,7 +637,7 @@ uMenuItemInt::uMenuItemInt
 (uMenu *m,const tOutput &tit,const tOutput &help,int &targ,
  int mi,int ma,int step)
         :uMenuItem(m,help),title(tit),target(targ),Min(mi),Max(ma),
-        Step(step){
+Step(step){
     if (target<Min) target=Min;
     if (target>Max) target=Max;
 }
@@ -719,46 +658,6 @@ void uMenuItemInt::Render(REAL x,REAL y,REAL alpha,
     DisplayText(x+.02,y,s,selected,alpha,-1);
 }
 
-// *****************************************
-//               Float Choose
-// *****************************************
-
-#ifdef SLOPPYLOCALE
-uMenuItemReal::uMenuItemReal
-(uMenu *m,const char *tit,const char *help,REAL &targ,
- REAL mi,REAL ma,REAL step)
-        :uMenuItem(m,help),title(tit),target(targ),Min(mi),Max(ma),
-        Step(step){
-    if (target<Min) target=Min;
-    if (target>Max) target=Max;
-}
-#endif
-
-uMenuItemReal::uMenuItemReal
-(uMenu *m,const tOutput &tit,const tOutput &help,REAL &targ,
- REAL mi,REAL ma,REAL step)
-        :uMenuItem(m,help),title(tit),target(targ),Min(mi),Max(ma),
-        Step(step){
-    if (target<Min) target=Min;
-    if (target>Max) target=Max;
-}
-
-
-void uMenuItemReal::LeftRight(int dir){
-    target+=dir*Step;
-    if (target<Min) target=Min;
-    if (target>Max) target=Max;
-}
-
-void uMenuItemReal::Render(REAL x,REAL y,REAL alpha,
-                          bool selected){
-    DisplayText(x-.02,y,title,selected,alpha,1);
-
-    tString s;
-    s << target;
-    DisplayText(x+.02,y,s,selected,alpha,-1);
-}
-
 
 // *****************************************************
 
@@ -768,11 +667,10 @@ uMenuItemString::uMenuItemString(uMenu *M,
                                  tString &c,
                                  int maxLength )
         :uMenuItem(M,help),description(de),content(&c),cursorPos(0), maxLength_( maxLength ){
-    int len=content->Len();
-    if (len==0 || (*content)(len-1)!=0)
-        (*content)[len]=0;
+    // int len=content->Len();
+    // if (len==0 || (*content)(len-1)!=0)
+    //    (*content)[len]=0;
     cursorPos=content->Len()-1;
-    colorMode_ = rTextField::COLOR_SHOW;
 }
 
 void uMenuItemString::Render(REAL x,REAL y,
@@ -787,13 +685,8 @@ void uMenuItemString::Render(REAL x,REAL y,
         if (counter & 32) cmode=2;
     }
 
-    // unslected items with COLOR_SHOW should be rendered with COLOR_USE
-    rTextField::ColorMode colorMode = colorMode_;
-    if ( colorMode == rTextField::COLOR_SHOW && !selected )
-        colorMode = rTextField::COLOR_USE;
-
     DisplayText(x-.02,y,description,selected,alpha,1);
-    DisplayText(x+.02,y,&((*content)[0]),selected,alpha,-1,cmode,cursorPos, colorMode );
+    DisplayText(x+.02,y,*content,selected,alpha,-1,cmode,cursorPos);
 #endif
 }
 
@@ -924,12 +817,11 @@ bool uMenuItemString::Event(SDL_Event &e){
             // insert character if there is room
             if (content->Len() < maxLength_)
             {
-                for (int i=content->Len()-1;i>=cursorPos;i--)
-                    (*content)[i+1]=(*content)[i];
-
-                // guarantee proper null termination
-                (*content)[content->Len()-1]='\0';
-                (*content)[cursorPos]=c.unicode;
+                tString beg = content->SubStr(0,cursorPos);
+                tString end = content->SubStr(cursorPos);
+                *content = beg;
+                *content += tString::CHAR(c.unicode);
+                *content += end;
                 cursorPos++;
             }
         }
@@ -938,8 +830,8 @@ bool uMenuItemString::Event(SDL_Event &e){
         }
     }
 
-    if (cursorPos<0)    cursorPos=0;
-    if (cursorPos > content->Len()-1) cursorPos=content->Len()-1;
+    if(cursorPos<0)    cursorPos=0;
+    if(cursorPos > content->Len()-1) cursorPos=content->Len()-1;
 
     return ret;
 #else
@@ -947,64 +839,253 @@ bool uMenuItemString::Event(SDL_Event &e){
 #endif
 }
 
-uMenuItemStringWithHistory::uMenuItemStringWithHistory(uMenu *M,const tOutput& desc, const tOutput& help,tString &c, int maxLength, std::deque<tString> &history, int limit ):
+//! @param words a deque containing the possible words that can be used for completion
+uAutoCompleter::uAutoCompleter(std::deque<tString> &words) :
+        m_PossibleWords(words),
+        m_LastCompletion(-1),
+        m_ignorecase(true)
+{}
+
+//! @param string the string that should be tested
+//! @param pos    the cursor position within the string
+//! @return -1 if the cursor isn't at the end of a word, the number of characters to the previous space if it is
+int uAutoCompleter::FindLengthOfLastWord(tString &string, unsigned pos) {
+    int charright = ' ', charleft  = ' ';
+    if(string.size() >= 1) {
+        if (pos == string.size()) {
+            charleft=string.at(pos-1);
+        }
+        else if (pos < string.size()) {
+            charright=string.at(pos);
+            if(pos>1) {
+                charleft=string.at(pos-1);
+            }
+        }
+    }
+    if(charright != ' ' && charleft != ' ')
+        return -1; //no completion possible
+    if(charleft == ' ')
+        return 0;
+    else
+        return pos - string.find_last_of(' ', pos - 1) - 1;
+}
+
+//! @param word       the part of the word that is searched for
+//! @param results    the list where the possible completions will be saved
+void uAutoCompleter::FindPossibleWords(tString word, std::deque<tString> &results) {
+    if(m_ignorecase)
+        word = Simplify(word);
+    for(std::deque<tString>::iterator i=m_PossibleWords.begin(); i!=m_PossibleWords.end(); ++i) {
+        size_t pos;
+        if((pos = (m_ignorecase ? Simplify(*i) : *i).find(word)) != tString::npos) {
+            if(isalpha((*i)[pos]) && pos != 0 && isalpha((*i)[pos-1]))
+                continue; //both the char we're at and the one before is alphanumeric; we're in the middle of a word
+            results.push_back(*i);
+        }
+    }
+}
+
+//! @param word       the part of the word that is searched for
+//! @param results    the list of possible completions
+//! @return the match, if any
+tString uAutoCompleter::FindClosestMatch(tString &word, std::deque<tString> &results) {
+    tString ret(m_ignorecase?Simplify(word):word);
+    unsigned int len = ret.size();
+    while(true) { // complete right side
+        std::deque<tString>::iterator i;
+        i = results.begin();
+        bool found=true;
+        tString::size_type pos=(m_ignorecase?Simplify(*i):*i).find(ret);
+        if(pos!=tString::npos && pos+len < i->size()) {
+            found=false;
+            ret+=(m_ignorecase ? tolower(i->at(pos+len)) : i->at(pos+len));
+            ++i;
+            for(; i!=results.end(); ++i) {
+                if((m_ignorecase?Simplify(*i):*i).find(ret) == tString::npos) {
+                    found=true;
+                    ret.erase(ret.length()-1);
+                    break;
+                }
+            }
+        }
+        if (found) //we found a mismatch
+            break;
+        else
+            len++;
+    }
+    while(true) { // something similar, but now for the left side of the word
+        std::deque<tString>::iterator i;
+        i = results.begin();
+        bool found=true;
+        tString::size_type pos=(m_ignorecase?Simplify(*i):*i).find(ret);
+        if(pos!=tString::npos && pos > 0) {
+            found=false;
+            ret.insert(ret.begin(),(m_ignorecase ? tolower(i->at(pos-1)) : i->at(pos-1)));
+            ++i;
+            for(; i!=results.end(); ++i) {
+                if((m_ignorecase?Simplify(*i):*i).find(ret) == tString::npos) {
+                    found=true;
+                    ret.erase(0,1);
+                    break;
+                }
+            }
+        }
+        if (found)  //we found a mismatch
+            break;
+        else
+            len++;
+    }
+    return ret;
+}
+
+//! @param results    the list of possible completions
+//! @param word       the word that was already typed, it will be highlighted
+void uAutoCompleter::ShowPossibilities(std::deque<tString> &results, tString &word) {
+    if(results.size() > 10) {
+        con << tOutput("$tab_completion_toomanyresults");
+    }
+    else {
+        con << tOutput("$tab_completion_results");
+        tString::size_type len=word.length();
+        for(std::deque<tString>::iterator i=results.begin(); i!=results.end(); ++i) {
+            tString::size_type pos=(m_ignorecase?Simplify(*i):*i).find(word);
+            con << i->SubStr(0,pos)
+            << "0xff8888"
+            << i->SubStr(pos, len)
+            << "0xffffff"
+            << i->SubStr(pos+len)
+            << "\n";
+        }
+    }
+}
+
+//! @param string     the string in which the completion should take place
+//! @param pos        the position in strind where the replaced word ends
+//! @param len        the length of the word that is already there
+//! @param match      the string that will be inserted into the other one
+//! @return the new cursor position
+int uAutoCompleter::DoCompletion(tString &string, int pos, int len, tString &match) {
+    string.erase(pos-len, len);
+    string.insert(pos-len, match);
+    return pos - len + match.size();
+}
+
+//! @param string     the string in which the completion should take place
+//! @param pos        the position in string where the replaced word ends
+//! @param len        the length of the word that is already there
+//! @param match      the string that will be inserted into the other one
+//! @return the new cursor position
+int uAutoCompleter::DoFullCompletion(tString &string, int pos, int len, tString &match) {
+    tString actualString = match + " ";
+    return DoCompletion(string, pos, len, actualString);
+}
+
+//! @param string     the string in which the completion should take place
+//! @param pos        the cursor position
+//! @param len	      the length of the word
+//! @return the new position on success (something was found), -1 on failure
+int uAutoCompleter::TryCompletion(tString &string, unsigned pos, unsigned len) {
+    tString word(string.SubStr(pos-len, len));
+    std::deque<tString> results;
+    FindPossibleWords(word, results);
+    if(results.size() > 1){
+        tString match(FindClosestMatch(word, results));
+        if(match == word) { //no completion took place
+            ShowPossibilities(results, word);
+            return pos;
+        }
+        else { //do the completion
+            return DoCompletion(string, pos, len, match);
+        }
+    }
+    else if(!results.empty()){
+        return DoFullCompletion(string, pos, len, results.front());
+    }
+    return -1;
+}
+
+//! @param string     the string in which the completion should take place
+//! @param pos        the cursor position
+//! @return the new cursor position
+int uAutoCompleter::Complete(tString &string, unsigned pos) {
+    if(m_LastCompletion != -1 && (unsigned)m_LastCompletion < pos) {
+        int res = TryCompletion(string, pos, pos - m_LastCompletion);
+        if(res != -1) return res;
+    }
+    int len = FindLengthOfLastWord(string, pos);
+    if(len == -1) return pos; //in the middle of a word...
+    int res = TryCompletion(string, pos, len);
+    if(res != -1) {
+        m_LastCompletion = pos - len;
+        return res;
+    }
+    return pos;
+}
+
+//! @param ignorecase ignore upper and lower case?
+void uAutoCompleter::SetIgnorecase(bool ignorecase) {
+    m_ignorecase = ignorecase;
+}
+
+//! @param str the string to simplify
+//! @returns the simplified string(in this case everything converted to lowercase)
+tString uAutoCompleter::Simplify(tString const &str) {
+    return str.ToLower();
+}
+
+//! @param M         passed on to uMenuItemString
+//! @param desc      passed on to uMenuItemString
+//! @param help      passed on to uMenuItemString
+//! @param c         passed on to uMenuItemString
+//! @param maxLength passed on to uMenuItemString
+//! @param history   a reference to the history of commands
+//! @param limit     the limit for the history's size
+//! @param completer the autocompleter to be used, if 0, no completion will take place
+uMenuItemStringWithHistory::uMenuItemStringWithHistory(uMenu *M,const tOutput& desc, const tOutput& help,tString &c, int maxLength, std::deque<tString> &history, int limit, uAutoCompleter *completer ):
         uMenuItemString(M, desc,help,c, maxLength ),
         m_History(history),
         m_HistoryPos(0),
-        m_HistoryLimit(limit)
-{
+        m_HistoryLimit(limit),
+m_Completer(completer) {
     m_History.push_front(tString());
 }
 
-uMenuItemStringWithHistory::~uMenuItemStringWithHistory()
-{
-    if (content->Len() > 1)
-    {
-        for (std::deque<tString>::iterator i=m_History.begin(); i!=m_History.end(); ++i)
-        {
-            if (*i == *content)
-            {
+//! This destructor will write the currently displayed string into the history if it isn't already in it
+uMenuItemStringWithHistory::~uMenuItemStringWithHistory() {
+    if(content->Len() > 1){
+        for(std::deque<tString>::iterator i=m_History.begin(); i!=m_History.end(); ++i) {
+            if(*i == *content) {
                 m_History.erase(i);
                 break;
             }
         }
         m_History.front() = *content;
-    }
-    else
-    {
+    } else {
         m_History.pop_front();
     }
-    if (m_History.size() > m_HistoryLimit)
+    if(m_History.size() > m_HistoryLimit)
         m_History.pop_back();
 }
 
-bool uMenuItemStringWithHistory::Event(SDL_Event &e)
-{
-    // flag indicating that the event was handled
-    bool ret = false;
+//! @param e the event to process
+/// @returns true if the event was handled, false if it wasn't
+bool uMenuItemStringWithHistory::Event(SDL_Event &e){
 #ifndef DEDICATED
-    SDLMod mod = e.key.keysym.mod;
-
-    if (e.type == SDL_KEYDOWN
-            && ((e.key.keysym.sym == SDLK_UP)
-                || (e.key.keysym.sym == SDLK_p && mod & KMOD_CTRL)))
-    {
+    if (e.type==SDL_KEYDOWN &&
+            (e.key.keysym.sym==SDLK_UP)){
         if (m_History.size() - 1 > m_HistoryPos)
         {
-            // the new entry... save it before overwriting it
-            if (m_HistoryPos == 0)
+            if(m_HistoryPos == 0)  //the new entry... save it before overwriting it
                 m_History.front() = *content;
             m_HistoryPos++;
             *content = m_History[m_HistoryPos];
             cursorPos = content->Len() - 1;
         }
 
-        ret = true;
+        return true;
     }
-    else if (e.type == SDL_KEYDOWN
-             && ((e.key.keysym.sym == SDLK_DOWN)
-                 || (e.key.keysym.sym == SDLK_n && mod & KMOD_CTRL)))
-    {
+    else if (e.type==SDL_KEYDOWN &&
+             (e.key.keysym.sym==SDLK_DOWN)){
         if (m_HistoryPos > 0)
         {
             m_HistoryPos--;
@@ -1012,18 +1093,20 @@ bool uMenuItemStringWithHistory::Event(SDL_Event &e)
             cursorPos = content->Len() - 1;
         }
 
-        ret = true;
+        return true;
     }
+    else if (e.type==SDL_KEYDOWN &&
+             (e.key.keysym.sym==SDLK_TAB)){
+        if(m_Completer != 0) {
+            cursorPos = m_Completer->Complete(*content, cursorPos);
+        }
 
-    // clamp cursor position
-    if (cursorPos<0)
-        cursorPos=0;
-    if (cursorPos > content->Len() - 1)
-        cursorPos=content->Len() - 1;
+        return true;
+    }
+    else
+        return uMenuItemString::Event(e);
 #endif
-
-    // return result or delegate
-    return ret || uMenuItemString::Event(e);
+    return false;
 }
 
 // *****************************************************
@@ -1034,7 +1117,7 @@ bool uMenuItemStringWithHistory::Event(SDL_Event &e)
 uMenuItemSubmenu::uMenuItemSubmenu(uMenu *M,
                                    uMenu *s,
                                    const tOutput& help)
-        :uMenuItem(M,help),submenu(s){}
+    :uMenuItem(M,help),submenu(s){}
 
 
 void uMenuItemSubmenu::Render(REAL x,REAL y,REAL alpha,bool selected){
@@ -1136,7 +1219,7 @@ void uMenuItemFileSelection::AddFile( const char *fileName, const char *filePath
 static tCallback *enter_anchor=NULL,*leave_anchor=NULL, *background_anchor=NULL;
 
 uCallbackMenuEnter::uCallbackMenuEnter(VOIDFUNC *f)
-        :tCallback(enter_anchor,f){}
+    :tCallback(enter_anchor,f){}
 
 void uCallbackMenuEnter::MenuEnter(){
     Exec(enter_anchor);
@@ -1156,57 +1239,18 @@ void uCallbackMenuBackground::MenuBackground(){
     Exec(background_anchor);
 }
 
-// poll input, return true if ESC was pressed
-bool uMenu::IdleInput( bool processInput )
-{
-#ifndef DEDICATED
-    if( !processInput )
-    {
-        sr_LockSDL();
-        SDL_PumpEvents();
-        sr_UnlockSDL();
-        return uMenu::quickexit != uMenu::QuickExit_Off;
-    }
 
-    SDL_Event event;
-    uInputProcessGuard inputProcessGuard;
-    while (!s_idleBackground && su_GetSDLInput(event))
-    {   
-        switch (event.type)
-        {
-        case SDL_KEYDOWN:
-            switch (event.key.keysym.sym)
-            {
-            case(SDLK_ESCAPE):
-                s_globalRepeat = false;
-                lastkey=tSysTimeFloat();
-                return true;
-                break;
-            default:
-                break;
-            }   
-        default:
-            break;
-        }
-    }   
 
-    return uMenu::quickexit != uMenu::QuickExit_Off;
-#endif
-
-    return false;
-}
-
-// return value: false only if the user pressed ESC
-bool uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL to){
-    bool ret = true;
+void uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL to){
 #ifdef DEDICATED
     con << message << ":\n";
     con << interpretation << '\n';
 #else
-
     // reload textures (just in case)
     rITexture::UnloadAll();
 
+    tAdvanceFrame();
+
     bool textOutBack = sr_textOut;
     sr_textOut = false;
 
@@ -1218,11 +1262,11 @@ bool uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
 
     rSysDep::ClearGL();
     rSysDep::SwapGL();
-    if (sr_glOut)
-    {
-        rFont::s_defaultFont.Select();
-        rFont::s_defaultFontSmall.Select();
-    }
+    //    if (sr_glOut)
+    //    {
+    //        rFont::s_defaultFont.Select();
+    //        rFont::s_defaultFontSmall.Select();
+    //    }
     rSysDep::ClearGL();
     rSysDep::SwapGL();
 
@@ -1232,45 +1276,14 @@ bool uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
     // catch some keyboard input
     {
         uInputProcessGuard inputProcessGuard;
-        while (su_GetSDLInput(tEvent)) ;
+        while(su_GetSDLInput(tEvent));
     }
 
     {
         uInputProcessGuard inputProcessGuard;
+        while(  !quickexit && ( !su_GetSDLInput(tEvent) || tEvent.type!=SDL_KEYDOWN) &&
+                (to < 0 || tSysTimeFloat() < timeout)){
 
-        unsigned offset = 0; //amount of scrolling taking place
-        //convert to an array for scrolling
-        tString interpretationString;
-        interpretationString << interpretation << "\n";
-        std::vector<tString> lines;
-        int lastNewline = 0;
-        for (int i = 0; i < interpretationString.Len() - 1; ++i) {
-            if (interpretationString[i] == '\n' && i != 0) {
-                lines.push_back(interpretationString.SubStr(lastNewline, i - lastNewline));
-                lastNewline = i + 1;
-            }
-        }
-        while (  !quickexit &&
-                 (to < 0 || tSysTimeFloat() < timeout)){
-            //while(  !quickexit && ( !su_GetSDLInput(tEvent) || tEvent.type!=SDL_KEYDOWN) &&
-            //        (to < 0 || tSysTimeFloat() < timeout)){
-            if ( su_GetSDLInput(tEvent) && tEvent.type==SDL_KEYDOWN) {
-                switch (tEvent.key.keysym.sym) {
-                case SDLK_UP:
-                    if (offset > 0)
-                        offset -= 1;
-                    continue;
-                case SDLK_DOWN:
-                    offset += 1;
-                    continue;
-                case SDLK_ESCAPE:
-                    ret = false;
-                    break;
-                default:
-                    break;
-                }
-                break;
-            }
             if ( sr_glOut )
             {
                 sr_ResetRenderState(true);
@@ -1280,8 +1293,9 @@ bool uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
 
                 GenericBackground();
 
-                //16*3/640.0, 32*3/480.0
-                REAL w=0.1*(REAL(sr_screenHeight)/sr_screenWidth),h=0.2;
+                REAL w=16*3/640.0;
+                REAL h=32*3/480.0;
+
 
                 //REAL middle=-.6;
 
@@ -1294,21 +1308,20 @@ bool uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
                 }
 
                 Color(1,1,1);
-                DisplayText(0,.8,w,h, message);
+                DisplayText(0,.8,w,message,sr_fontError);
 
-                //16/640.0
-                w = 1/30.0*(REAL(sr_screenHeight)/sr_screenWidth);
+                w = 16/640.0;
                 h = 32/480.0;
 
-                if (offset >= lines.size()) offset = lines.size() - 1;
                 {
-                    rTextField c(-.8,.6, w, h);
+                    rTextField c(-.8,.6, h, sr_fontError);
+                    c.EnableLineWrap();
 
-                    for (unsigned i = offset; i < lines.size(); ++i)
-                        c << lines[i] << "\n";
+                    c << interpretation;
                 }
+
+                rSysDep::SwapGL();
             }
-            rSysDep::SwapGL();
             tAdvanceFrame();
         }
     }
@@ -1316,7 +1329,7 @@ bool uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
     // catch some keyboard input
     {
         uInputProcessGuard inputProcessGuard;
-        while (su_GetSDLInput(tEvent)) ;
+        while (su_GetSDLInput(tEvent));
     }
 
     uMenu::SetIdle(idle_back);
@@ -1326,7 +1339,5 @@ bool uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
 
     sr_textOut = textOutBack;
 #endif
-
-    return ret;
 }
 
