@@ -6,4 +6,7 @@
 // include common Windows header
 #include "config_common.h"
 
+// we're using SDL_Mixer
+#define HAVE_LIBSDL_MIXER 1
+
 #endif
