@@ -22,11 +22,6 @@
 // disable POD initialization behavior change warning in VisualC++ 2005
 #pragma warning ( disable: 4345 )
 
-// compatibility with later mingw versions
-#if defined(__GNUC__) && __GNUC__ >= 4
-#define HAVE_ISBLANK
-#endif
-
 // activate dirty OpenGL initialization option
 #ifndef DIRTY
 #define DIRTY
@@ -37,19 +32,19 @@
 #define WIN32
 #endif
 
-// exists in windows headers
-#define HAVE_ISBLANK
-
-// does not exist
-#undef HAVE_CLEARENV
-
 // defines for data directories in Windows
 #ifndef DEBUG
-#define USER_DATA_DIR  "${APPDATA}/Armagetron" 
-#define SCREENSHOT_DIR "${MYPICTURES}/Armagetron" 
+#define USER_DATA_DIR  "${APPDATA}/Armagetron"
+#define SCREENSHOT_DIR "${MYPICTURES}/Armagetron"
 #else
-#define USER_DATA_DIR  "." 
+#define USER_DATA_DIR  "."
 #endif
 
+// for now, no joystick support in Windows
+#define NOJOYSTICK 1
+
 // Define this for the particle library
 #define PARTICLEDLL_EXPORTS
+
+// define version
+#include "../version.h"
